/*-
 * Copyright (c) 2015-2016 Dag-Erling Smørgrav
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "cryb/impl.h"

#include <stdint.h>
#include <stdio.h>

#include <cryb/aes.h>

#include <cryb/test.h>

struct t_case {
	const char *desc;
	int keylen;
	const uint8_t key[32];
	const uint8_t ptext[AES_BLOCK_LEN];
	const uint8_t ctext[AES_BLOCK_LEN];
};

/***************************************************************************
 * Test cases
 */

static struct t_case t_cases[] = {
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   0",
		.keylen	 = 128,
		.key	 = {
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0e, 0xdd, 0x33, 0xd3, 0xc6, 0x21, 0xe5, 0x46,
			0x45, 0x5b, 0xd8, 0xba, 0x14, 0x18, 0xbe, 0xc8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   1",
		.keylen	 = 128,
		.key	 = {
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4b, 0xc3, 0xf8, 0x83, 0x45, 0x0c, 0x11, 0x3c,
			0x64, 0xca, 0x42, 0xe1, 0x11, 0x2a, 0x9e, 0x87,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   2",
		.keylen	 = 128,
		.key	 = {
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x72, 0xa1, 0xda, 0x77, 0x0f, 0x5d, 0x7a, 0xc4,
			0xc9, 0xef, 0x94, 0xd8, 0x22, 0xaf, 0xfd, 0x97,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   3",
		.keylen	 = 128,
		.key	 = {
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0x00, 0x14, 0xd6, 0x34, 0xe2, 0xb7, 0x65,
			0x07, 0x77, 0xe8, 0xe8, 0x4d, 0x03, 0xcc, 0xd8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   4",
		.keylen	 = 128,
		.key	 = {
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf1, 0x7e, 0x79, 0xae, 0xd0, 0xdb, 0x7e, 0x27,
			0x9e, 0x95, 0x5b, 0x5f, 0x49, 0x38, 0x75, 0xa7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   5",
		.keylen	 = 128,
		.key	 = {
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9e, 0xd5, 0xa7, 0x51, 0x36, 0xa9, 0x40, 0xd0,
			0x96, 0x3d, 0xa3, 0x79, 0xdb, 0x4a, 0xf2, 0x6a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   6",
		.keylen	 = 128,
		.key	 = {
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc4, 0x29, 0x5f, 0x83, 0x46, 0x5c, 0x77, 0x55,
			0xe8, 0xfa, 0x36, 0x4b, 0xac, 0x6a, 0x7e, 0xa5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   7",
		.keylen	 = 128,
		.key	 = {
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb1, 0xd7, 0x58, 0x25, 0x6b, 0x28, 0xfd, 0x85,
			0x0a, 0xd4, 0x94, 0x42, 0x08, 0xcf, 0x11, 0x55,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   8",
		.keylen	 = 128,
		.key	 = {
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x42, 0xff, 0xb3, 0x4c, 0x74, 0x3d, 0xe4, 0xd8,
			0x8c, 0xa3, 0x80, 0x11, 0xc9, 0x90, 0x89, 0x0b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128   9",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x99, 0x58, 0xf0, 0xec, 0xea, 0x8b, 0x21, 0x72,
			0xc0, 0xc1, 0x99, 0x5f, 0x91, 0x82, 0xc0, 0xf3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  10",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x95, 0x6d, 0x77, 0x98, 0xfa, 0xc2, 0x0f, 0x82,
			0xa8, 0x82, 0x3f, 0x98, 0x4d, 0x06, 0xf7, 0xf5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  11",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa0, 0x1b, 0xf4, 0x4f, 0x2d, 0x16, 0xbe, 0x92,
			0x8c, 0xa4, 0x4a, 0xaf, 0x7b, 0x9b, 0x10, 0x6b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  12",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb5, 0xf1, 0xa3, 0x3e, 0x50, 0xd4, 0x0d, 0x10,
			0x37, 0x64, 0xc7, 0x6b, 0xd4, 0xc6, 0xb6, 0xf8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  13",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x26, 0x37, 0x05, 0x0c, 0x9f, 0xc0, 0xd4, 0x81,
			0x7e, 0x2d, 0x69, 0xde, 0x87, 0x8a, 0xee, 0x8d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  14",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0x3e, 0xcb, 0xe4, 0xa4, 0x53, 0x26, 0x9a,
			0x0d, 0xd2, 0x60, 0x69, 0x46, 0x7f, 0xb5, 0xb5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  15",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0xd0, 0x75, 0x4f, 0xe6, 0x8f, 0x11, 0xb9,
			0xe3, 0x75, 0xd0, 0x70, 0xa6, 0x08, 0xc8, 0x84,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  16",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc6, 0xa0, 0xb3, 0xe9, 0x98, 0xd0, 0x50, 0x68,
			0xa5, 0x39, 0x97, 0x78, 0x40, 0x52, 0x00, 0xb4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  17",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdf, 0x55, 0x6a, 0x33, 0x43, 0x8d, 0xb8, 0x7b,
			0xc4, 0x1b, 0x17, 0x52, 0xc5, 0x5e, 0x5e, 0x49,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  18",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0xfb, 0x12, 0x8d, 0x3a, 0x1a, 0xf6, 0xe5,
			0x48, 0x52, 0x1b, 0xb9, 0x62, 0xbf, 0x1f, 0x05,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  19",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x26, 0x29, 0x8e, 0x9c, 0x1d, 0xb5, 0x17, 0xc2,
			0x15, 0xfa, 0xdf, 0xb7, 0xd2, 0xa8, 0xd6, 0x91,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  20",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa6, 0xcb, 0x76, 0x1d, 0x61, 0xf8, 0x29, 0x2d,
			0x0d, 0xf3, 0x93, 0xa2, 0x79, 0xad, 0x03, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  21",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x12, 0xac, 0xd8, 0x9b, 0x13, 0xcd, 0x5f, 0x87,
			0x26, 0xe3, 0x4d, 0x44, 0xfd, 0x48, 0x61, 0x08,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  22",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x95, 0xb1, 0x70, 0x3f, 0xc5, 0x7b, 0xa0, 0x9f,
			0xe0, 0xc3, 0x58, 0x0f, 0xeb, 0xdd, 0x7e, 0xd4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  23",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xde, 0x11, 0x72, 0x2d, 0x89, 0x3e, 0x9f, 0x91,
			0x21, 0xc3, 0x81, 0xbe, 0xcc, 0x1d, 0xa5, 0x9a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  24",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6d, 0x11, 0x4c, 0xcb, 0x27, 0xbf, 0x39, 0x10,
			0x12, 0xe8, 0x97, 0x4c, 0x54, 0x6d, 0x9b, 0xf2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  25",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5c, 0xe3, 0x7e, 0x17, 0xeb, 0x46, 0x46, 0xec,
			0xfa, 0xc2, 0x9b, 0x9c, 0xc3, 0x8d, 0x93, 0x40,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  26",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x18, 0xc1, 0xb6, 0xe2, 0x15, 0x71, 0x22, 0x05,
			0x6d, 0x02, 0x43, 0xd8, 0xa1, 0x65, 0xcd, 0xdb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  27",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x99, 0x69, 0x3e, 0x6a, 0x59, 0xd1, 0x36, 0x6c,
			0x74, 0xd8, 0x23, 0x56, 0x2d, 0x7e, 0x14, 0x31,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  28",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6c, 0x7c, 0x64, 0xdc, 0x84, 0xa8, 0xbb, 0xa7,
			0x58, 0xed, 0x17, 0xeb, 0x02, 0x5a, 0x57, 0xe3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  29",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe1, 0x7b, 0xc7, 0x9f, 0x30, 0xea, 0xab, 0x2f,
			0xac, 0x2c, 0xbb, 0xe3, 0x45, 0x8d, 0x68, 0x7a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  30",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0x14, 0xbc, 0x20, 0x28, 0x00, 0x9b, 0x92,
			0x3f, 0x0b, 0x01, 0x91, 0x5c, 0xe5, 0xe7, 0xc4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  31",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9c, 0x28, 0x52, 0x4a, 0x16, 0xa1, 0xe1, 0xc1,
			0x45, 0x29, 0x71, 0xca, 0xa8, 0xd1, 0x34, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  32",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xed, 0x62, 0xe1, 0x63, 0x63, 0x63, 0x83, 0x60,
			0xfd, 0xd6, 0xad, 0x62, 0x11, 0x27, 0x94, 0xf0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  33",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5a, 0x86, 0x88, 0xf0, 0xb2, 0xa2, 0xc1, 0x62,
			0x24, 0xc1, 0x61, 0x65, 0x8f, 0xfd, 0x40, 0x44,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  34",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x23, 0xf7, 0x10, 0x84, 0x2b, 0x9b, 0xb9, 0xc3,
			0x2f, 0x26, 0x64, 0x8c, 0x78, 0x68, 0x07, 0xca,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  35",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x44, 0xa9, 0x8b, 0xf1, 0x1e, 0x16, 0x3f, 0x63,
			0x2c, 0x47, 0xec, 0x6a, 0x49, 0x68, 0x3a, 0x89,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  36",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0f, 0x18, 0xaf, 0xf9, 0x42, 0x74, 0x69, 0x6d,
			0x9b, 0x61, 0x84, 0x8b, 0xd5, 0x0a, 0xc5, 0xe5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  37",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0x40, 0x85, 0x71, 0xc3, 0xe2, 0x42, 0x45,
			0x40, 0x20, 0x7f, 0x83, 0x3b, 0x6d, 0xda, 0x69,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  38",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x30, 0x3f, 0xf9, 0x96, 0x94, 0x7f, 0x0c, 0x7d,
			0x1f, 0x43, 0xc8, 0xf3, 0x02, 0x7b, 0x9b, 0x75,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  39",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7d, 0xf4, 0xda, 0xf4, 0xad, 0x29, 0xa3, 0x61,
			0x5a, 0x9b, 0x6e, 0xce, 0x5c, 0x99, 0x51, 0x8a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  40",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc7, 0x29, 0x54, 0xa4, 0x8d, 0x07, 0x74, 0xdb,
			0x0b, 0x49, 0x71, 0xc5, 0x26, 0x26, 0x04, 0x15,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  41",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1d, 0xf9, 0xb7, 0x61, 0x12, 0xdc, 0x65, 0x31,
			0xe0, 0x7d, 0x2c, 0xfd, 0xa0, 0x44, 0x11, 0xf0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  42",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8e, 0x4d, 0x8e, 0x69, 0x91, 0x19, 0xe1, 0xfc,
			0x87, 0x54, 0x5a, 0x64, 0x7f, 0xb1, 0xd3, 0x4f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  43",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe6, 0xc4, 0x80, 0x7a, 0xe1, 0x1f, 0x36, 0xf0,
			0x91, 0xc5, 0x7d, 0x9f, 0xb6, 0x85, 0x48, 0xd1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  44",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8e, 0xbf, 0x73, 0xaa, 0xd4, 0x9c, 0x82, 0x00,
			0x7f, 0x77, 0xa5, 0xc1, 0xcc, 0xec, 0x6a, 0xb4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  45",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4f, 0xb2, 0x88, 0xcc, 0x20, 0x40, 0x04, 0x90,
			0x01, 0xd2, 0xc7, 0x58, 0x5a, 0xd1, 0x23, 0xfc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  46",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x04, 0x49, 0x71, 0x10, 0xef, 0xb9, 0xdc, 0xeb,
			0x13, 0xe2, 0xb1, 0x3f, 0xb4, 0x46, 0x55, 0x64,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  47",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x75, 0x55, 0x0e, 0x6c, 0xb5, 0xa8, 0x8e, 0x49,
			0x63, 0x4c, 0x9a, 0xb6, 0x9e, 0xda, 0x04, 0x30,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  48",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb6, 0x76, 0x84, 0x73, 0xce, 0x98, 0x43, 0xea,
			0x66, 0xa8, 0x14, 0x05, 0xdd, 0x50, 0xb3, 0x45,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  49",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcb, 0x2f, 0x43, 0x03, 0x83, 0xf9, 0x08, 0x4e,
			0x03, 0xa6, 0x53, 0x57, 0x1e, 0x06, 0x5d, 0xe6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  50",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xff, 0x4e, 0x66, 0xc0, 0x7b, 0xae, 0x3e, 0x79,
			0xfb, 0x7d, 0x21, 0x08, 0x47, 0xa3, 0xb0, 0xba,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  51",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7b, 0x90, 0x78, 0x51, 0x25, 0x50, 0x5f, 0xad,
			0x59, 0xb1, 0x3c, 0x18, 0x6d, 0xd6, 0x6c, 0xe3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  52",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8b, 0x52, 0x7a, 0x6a, 0xeb, 0xda, 0xec, 0x9e,
			0xae, 0xf8, 0xed, 0xa2, 0xcb, 0x77, 0x83, 0xe5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  53",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x43, 0xfd, 0xaf, 0x53, 0xeb, 0xbc, 0x98, 0x80,
			0xc2, 0x28, 0x61, 0x7d, 0x6a, 0x9b, 0x54, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  54",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x53, 0x78, 0x61, 0x04, 0xb9, 0x74, 0x4b, 0x98,
			0xf0, 0x52, 0xc4, 0x6f, 0x1c, 0x85, 0x0d, 0x0b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  55",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb5, 0xab, 0x30, 0x13, 0xdd, 0x1e, 0x61, 0xdf,
			0x06, 0xcb, 0xaf, 0x34, 0xca, 0x2a, 0xee, 0x78,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  56",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x74, 0x70, 0x46, 0x9b, 0xe9, 0x72, 0x30, 0x30,
			0xfd, 0xcc, 0x73, 0xa8, 0xcd, 0x4f, 0xbb, 0x10,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  57",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa3, 0x5a, 0x63, 0xf5, 0x34, 0x3e, 0xbe, 0x9e,
			0xf8, 0x16, 0x7b, 0xcb, 0x48, 0xad, 0x12, 0x2e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  58",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfd, 0x86, 0x87, 0xf0, 0x75, 0x7a, 0x21, 0x0e,
			0x9f, 0xdf, 0x18, 0x12, 0x04, 0xc3, 0x08, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  59",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7a, 0x18, 0x1e, 0x84, 0xbd, 0x54, 0x57, 0xd2,
			0x6a, 0x88, 0xfb, 0xae, 0x96, 0x01, 0x8f, 0xb0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  60",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x65, 0x33, 0x17, 0xb9, 0x36, 0x2b, 0x6f, 0x9b,
			0x9e, 0x1a, 0x58, 0x0e, 0x68, 0xd4, 0x94, 0xb5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  61",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x99, 0x5c, 0x9d, 0xc0, 0xb6, 0x89, 0xf0, 0x3c,
			0x45, 0x86, 0x7b, 0x5f, 0xaa, 0x5c, 0x18, 0xd1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  62",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0xa4, 0xd9, 0x6d, 0x56, 0xdd, 0xa3, 0x98,
			0xb9, 0xaa, 0xbe, 0xcf, 0xc7, 0x57, 0x29, 0xfd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  63",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x84, 0xbe, 0x19, 0xe0, 0x53, 0x63, 0x5f, 0x09,
			0xf2, 0x66, 0x5e, 0x7b, 0xae, 0x85, 0xb4, 0x2d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  64",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x32, 0xcd, 0x65, 0x28, 0x42, 0x92, 0x6a, 0xea,
			0x4a, 0xa6, 0x13, 0x7b, 0xb2, 0xbe, 0x2b, 0x5e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  65",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x49, 0x3d, 0x4a, 0x4f, 0x38, 0xeb, 0xb3, 0x37,
			0xd1, 0x0a, 0xa8, 0x4e, 0x91, 0x71, 0xa5, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  66",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd9, 0xbf, 0xf7, 0xff, 0x45, 0x4b, 0x0e, 0xc5,
			0xa4, 0xa2, 0xa6, 0x95, 0x66, 0xe2, 0xcb, 0x84,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  67",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x35, 0x35, 0xd5, 0x65, 0xac, 0xe3, 0xf3, 0x1e,
			0xb2, 0x49, 0xba, 0x2c, 0xc6, 0x76, 0x5d, 0x7a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  68",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf6, 0x0e, 0x91, 0xfc, 0x32, 0x69, 0xee, 0xcf,
			0x32, 0x31, 0xc6, 0xe9, 0x94, 0x56, 0x97, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  69",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xab, 0x69, 0xcf, 0xad, 0xf5, 0x1f, 0x8e, 0x60,
			0x4d, 0x9c, 0xc3, 0x71, 0x82, 0xf6, 0x63, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  70",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x78, 0x66, 0x37, 0x3f, 0x24, 0xa0, 0xb6, 0xed,
			0x56, 0xe0, 0xd9, 0x6f, 0xcd, 0xaf, 0xb8, 0x77,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  71",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1e, 0xa4, 0x48, 0xc2, 0xaa, 0xc9, 0x54, 0xf5,
			0xd8, 0x12, 0xe9, 0xd7, 0x84, 0x94, 0x44, 0x6a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  72",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xac, 0xc5, 0x59, 0x9d, 0xd8, 0xac, 0x02, 0x23,
			0x9a, 0x0f, 0xef, 0x4a, 0x36, 0xdd, 0x16, 0x68,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  73",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd8, 0x76, 0x44, 0x68, 0xbb, 0x10, 0x38, 0x28,
			0xcf, 0x7e, 0x14, 0x73, 0xce, 0x89, 0x50, 0x73,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  74",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0x0d, 0x02, 0x89, 0x36, 0x83, 0xb9, 0xf1,
			0x80, 0x45, 0x8e, 0x4a, 0xa6, 0xb7, 0x39, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  75",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0xd9, 0xb0, 0x17, 0xd3, 0x02, 0xdf, 0x41,
			0x0a, 0x93, 0x7d, 0xcd, 0xb8, 0xbb, 0x6e, 0x43,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  76",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xef, 0x16, 0x23, 0xcc, 0x44, 0x31, 0x3c, 0xff,
			0x44, 0x0b, 0x15, 0x94, 0xa7, 0xe2, 0x1c, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  77",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x28, 0x4c, 0xa2, 0xfa, 0x35, 0x80, 0x7b, 0x8b,
			0x0a, 0xe4, 0xd1, 0x9e, 0x11, 0xd7, 0xdb, 0xd7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  78",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf2, 0xe9, 0x76, 0x87, 0x57, 0x55, 0xf9, 0x40,
			0x1d, 0x54, 0xf3, 0x6e, 0x2a, 0x23, 0xa5, 0x94,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  79",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xec, 0x19, 0x8a, 0x18, 0xe1, 0x0e, 0x53, 0x24,
			0x03, 0xb7, 0xe2, 0x08, 0x87, 0xc8, 0xdd, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  80",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x54, 0x5d, 0x50, 0xeb, 0xd9, 0x19, 0xe4, 0xa6,
			0x94, 0x9d, 0x96, 0xad, 0x47, 0xe4, 0x6a, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  81",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdb, 0xdf, 0xb5, 0x27, 0x06, 0x0e, 0x0a, 0x71,
			0x00, 0x9c, 0x7b, 0xb0, 0xc6, 0x8f, 0x1d, 0x44,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  82",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9c, 0xfa, 0x13, 0x22, 0xea, 0x33, 0xda, 0x21,
			0x73, 0xa0, 0x24, 0xf2, 0xff, 0x0d, 0x89, 0x6d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  83",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x87, 0x85, 0xb1, 0xa7, 0x5b, 0x0f, 0x3b, 0xd9,
			0x58, 0xdc, 0xd0, 0xe2, 0x93, 0x18, 0xc5, 0x21,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  84",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x38, 0xf6, 0x7b, 0x9e, 0x98, 0xe4, 0xa9, 0x7b,
			0x6d, 0xf0, 0x30, 0xa9, 0xfc, 0xdd, 0x01, 0x04,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  85",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0x2a, 0xff, 0xfb, 0x2c, 0x88, 0x0e, 0x82,
			0xb0, 0x59, 0x26, 0xd0, 0xfc, 0x6c, 0x44, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  86",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6a, 0x79, 0x80, 0xce, 0x7b, 0x10, 0x5c, 0xf5,
			0x30, 0x95, 0x2d, 0x74, 0xda, 0xaf, 0x79, 0x8c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  87",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xea, 0x36, 0x95, 0xe1, 0x35, 0x1b, 0x9d, 0x68,
			0x58, 0xbd, 0x95, 0x8c, 0xf5, 0x13, 0xef, 0x6c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  88",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6d, 0xa0, 0x49, 0x0b, 0xa0, 0xba, 0x03, 0x43,
			0xb9, 0x35, 0x68, 0x1d, 0x2c, 0xce, 0x5b, 0xa1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  89",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf0, 0xea, 0x23, 0xaf, 0x08, 0x53, 0x40, 0x11,
			0xc6, 0x00, 0x09, 0xab, 0x29, 0xad, 0xa2, 0xf1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  90",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xff, 0x13, 0x80, 0x6c, 0xf1, 0x9c, 0xc3, 0x87,
			0x21, 0x55, 0x4d, 0x7c, 0x0f, 0xcd, 0xcd, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  91",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x68, 0x38, 0xaf, 0x1f, 0x4f, 0x69, 0xba, 0xe9,
			0xd8, 0x5d, 0xd1, 0x88, 0xdc, 0xdf, 0x06, 0x88,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  92",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0xcf, 0x44, 0xc9, 0x2d, 0x55, 0x0b, 0xfb,
			0x1e, 0xd2, 0x8e, 0xf5, 0x83, 0xdd, 0xf5, 0xd7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  93",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd0, 0x6e, 0x31, 0x95, 0xb5, 0x37, 0x6f, 0x10,
			0x9d, 0x5c, 0x4e, 0xc6, 0xc5, 0xd6, 0x2c, 0xed,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  94",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc4, 0x40, 0xde, 0x01, 0x4d, 0x3d, 0x61, 0x07,
			0x07, 0x27, 0x9b, 0x13, 0x24, 0x2a, 0x5c, 0x36,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  95",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf0, 0xc5, 0xc6, 0xff, 0xa5, 0xe0, 0xbd, 0x3a,
			0x94, 0xc8, 0x8f, 0x6b, 0x6f, 0x7c, 0x16, 0xb9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  96",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3e, 0x40, 0xc3, 0x90, 0x1c, 0xd7, 0xef, 0xfc,
			0x22, 0xbf, 0xfc, 0x35, 0xde, 0xe0, 0xb4, 0xd9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  97",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb6, 0x33, 0x05, 0xc7, 0x2b, 0xed, 0xfa, 0xb9,
			0x73, 0x82, 0xc4, 0x06, 0xd0, 0xc4, 0x9b, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  98",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0xbb, 0xaa, 0xb2, 0x2a, 0x6b, 0xd4, 0x92,
			0x5a, 0x99, 0xa2, 0xb4, 0x08, 0xd2, 0xdb, 0xae,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128  99",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x30, 0x7c, 0x5b, 0x8f, 0xcd, 0x05, 0x33, 0xab,
			0x98, 0xbc, 0x51, 0xe2, 0x7a, 0x6c, 0xe4, 0x61,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 100",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0x9c, 0x04, 0xff, 0x4c, 0x07, 0x51, 0x3c,
			0x0b, 0x3e, 0xf0, 0x5c, 0x03, 0xe3, 0x37, 0xb5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 101",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf1, 0x7a, 0xf0, 0xe8, 0x95, 0xdd, 0xa5, 0xeb,
			0x98, 0xef, 0xc6, 0x80, 0x66, 0xe8, 0x4c, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 102",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0x71, 0x67, 0xf3, 0x81, 0x2a, 0xff, 0xf1,
			0xff, 0xac, 0xb4, 0xa9, 0x34, 0x37, 0x9f, 0xc3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 103",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2c, 0xb1, 0xdc, 0x3a, 0x9c, 0x72, 0x97, 0x2e,
			0x42, 0x5a, 0xe2, 0xef, 0x3e, 0xb5, 0x97, 0xcd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 104",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0xae, 0xaa, 0x3a, 0x21, 0x3e, 0x96, 0x8d,
			0x4b, 0x5b, 0x67, 0x9d, 0x3a, 0x2c, 0x97, 0xfe,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 105",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x92, 0x41, 0xda, 0xca, 0x4f, 0xdd, 0x03, 0x4a,
			0x82, 0x37, 0x2d, 0xb5, 0x0e, 0x1a, 0x0f, 0x3f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 106",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc1, 0x45, 0x74, 0xd9, 0xcd, 0x00, 0xcf, 0x2b,
			0x5a, 0x7f, 0x77, 0xe5, 0x3c, 0xd5, 0x78, 0x85,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 107",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x79, 0x3d, 0xe3, 0x92, 0x36, 0x57, 0x0a, 0xba,
			0x83, 0xab, 0x9b, 0x73, 0x7c, 0xb5, 0x21, 0xc9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 108",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x16, 0x59, 0x1c, 0x0f, 0x27, 0xd6, 0x0e, 0x29,
			0xb8, 0x5a, 0x96, 0xc3, 0x38, 0x61, 0xa7, 0xef,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 109",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x44, 0xfb, 0x5c, 0x4d, 0x4f, 0x5c, 0xb7, 0x9b,
			0xe5, 0xc1, 0x74, 0xa3, 0xb1, 0xc9, 0x73, 0x48,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 110",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x67, 0x4d, 0x2b, 0x61, 0x63, 0x3d, 0x16, 0x2b,
			0xe5, 0x9d, 0xde, 0x04, 0x22, 0x2f, 0x47, 0x40,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 111",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb4, 0x75, 0x0f, 0xf2, 0x63, 0xa6, 0x5e, 0x1f,
			0x9e, 0x92, 0x4c, 0xcf, 0xd9, 0x8f, 0x3e, 0x37,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 112",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x62, 0xd0, 0x66, 0x2d, 0x6e, 0xae, 0xdd, 0xed,
			0xeb, 0xae, 0x7f, 0x7e, 0xa3, 0xa4, 0xf6, 0xb6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 113",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0xc4, 0x6b, 0xb3, 0x06, 0x92, 0xbe, 0x65,
			0x7f, 0x7e, 0xaa, 0x93, 0xeb, 0xad, 0x98, 0x97,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 114",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x32, 0x39, 0x94, 0xcf, 0xb9, 0xda, 0x28, 0x5a,
			0x5d, 0x96, 0x42, 0xe1, 0x75, 0x9b, 0x22, 0x4a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 115",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1d, 0xbf, 0x57, 0x87, 0x7b, 0x7b, 0x17, 0x38,
			0x5c, 0x85, 0xd0, 0xb5, 0x48, 0x51, 0xe3, 0x71,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 116",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdf, 0xa5, 0xc0, 0x97, 0xcd, 0xc1, 0x53, 0x2a,
			0xc0, 0x71, 0xd5, 0x7b, 0x1d, 0x28, 0xd1, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 117",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0x0c, 0x53, 0xfa, 0x37, 0x31, 0x1f, 0xc1,
			0x0b, 0xd2, 0xa9, 0x98, 0x1f, 0x51, 0x31, 0x74,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 118",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xba, 0x4f, 0x97, 0x0c, 0x0a, 0x25, 0xc4, 0x18,
			0x14, 0xbd, 0xae, 0x2e, 0x50, 0x6b, 0xe3, 0xb4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 119",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2d, 0xce, 0x3a, 0xcb, 0x72, 0x7c, 0xd1, 0x3c,
			0xcd, 0x76, 0xd4, 0x25, 0xea, 0x56, 0xe4, 0xf6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 120",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x51, 0x60, 0x47, 0x4d, 0x50, 0x4b, 0x9b, 0x3e,
			0xef, 0xb6, 0x8d, 0x35, 0xf2, 0x45, 0xf4, 0xb3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 121",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x41, 0xa8, 0xa9, 0x47, 0x76, 0x66, 0x35, 0xde,
			0xc3, 0x75, 0x53, 0xd9, 0xa6, 0xc0, 0xcb, 0xb7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 122",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x25, 0xd6, 0xcf, 0xe6, 0x88, 0x1f, 0x2b, 0xf4,
			0x97, 0xdd, 0x14, 0xcd, 0x4d, 0xdf, 0x44, 0x5b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 123",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x41, 0xc7, 0x8c, 0x13, 0x5e, 0xd9, 0xe9, 0x8c,
			0x09, 0x66, 0x40, 0x64, 0x72, 0x65, 0xda, 0x1e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 124",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5a, 0x4d, 0x40, 0x4d, 0x89, 0x17, 0xe3, 0x53,
			0xe9, 0x2a, 0x21, 0x07, 0x2c, 0x3b, 0x23, 0x05,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 125",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x02, 0xbc, 0x96, 0x84, 0x6b, 0x3f, 0xdc, 0x71,
			0x64, 0x3f, 0x38, 0x4c, 0xd3, 0xcc, 0x3e, 0xaf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 126",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9b, 0xa4, 0xa9, 0x14, 0x3f, 0x4e, 0x5d, 0x40,
			0x48, 0x52, 0x1c, 0x4f, 0x88, 0x77, 0xd8, 0x8e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey128 127",
		.keylen	 = 128,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa1, 0xf6, 0x25, 0x8c, 0x87, 0x7d, 0x5f, 0xcd,
			0x89, 0x64, 0x48, 0x45, 0x38, 0xbf, 0xc9, 0x2c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   0",
		.keylen	 = 192,
		.key	 = {
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xde, 0x88, 0x5d, 0xc8, 0x7f, 0x5a, 0x92, 0x59,
			0x40, 0x82, 0xd0, 0x2c, 0xc1, 0xe1, 0xb4, 0x2c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   1",
		.keylen	 = 192,
		.key	 = {
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x13, 0x2b, 0x07, 0x4e, 0x80, 0xf2, 0xa5, 0x97,
			0xbf, 0x5f, 0xeb, 0xd8, 0xea, 0x5d, 0xa5, 0x5e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   2",
		.keylen	 = 192,
		.key	 = {
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6e, 0xcc, 0xed, 0xf8, 0xde, 0x59, 0x2c, 0x22,
			0xfb, 0x81, 0x34, 0x7b, 0x79, 0xf2, 0xdb, 0x1f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   3",
		.keylen	 = 192,
		.key	 = {
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x18, 0x0b, 0x09, 0xf2, 0x67, 0xc4, 0x51, 0x45,
			0xdb, 0x2f, 0x82, 0x6c, 0x25, 0x82, 0xd3, 0x5c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   4",
		.keylen	 = 192,
		.key	 = {
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xed, 0xd8, 0x07, 0xef, 0x76, 0x52, 0xd7, 0xeb,
			0x0e, 0x13, 0xc8, 0xb5, 0xe1, 0x5b, 0x3b, 0xc0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   5",
		.keylen	 = 192,
		.key	 = {
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x99, 0x78, 0xbc, 0xf8, 0xdd, 0x8f, 0xd7, 0x22,
			0x41, 0x22, 0x3a, 0xd2, 0x4b, 0x31, 0xb8, 0xa4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   6",
		.keylen	 = 192,
		.key	 = {
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x53, 0x10, 0xf6, 0x54, 0x34, 0x3e, 0x8f, 0x27,
			0xe1, 0x2c, 0x83, 0xa4, 0x8d, 0x24, 0xff, 0x81,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   7",
		.keylen	 = 192,
		.key	 = {
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x83, 0x3f, 0x71, 0x25, 0x8d, 0x53, 0x03, 0x6b,
			0x02, 0x95, 0x2c, 0x76, 0xc7, 0x44, 0xf5, 0xa1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   8",
		.keylen	 = 192,
		.key	 = {
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xeb, 0xa8, 0x3f, 0xf2, 0x00, 0xcf, 0xf9, 0x31,
			0x8a, 0x92, 0xf8, 0x69, 0x1a, 0x06, 0xb0, 0x9f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192   9",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xff, 0x62, 0x0c, 0xcb, 0xe9, 0xf3, 0x29, 0x2a,
			0xbd, 0xf2, 0x17, 0x6b, 0x09, 0xf0, 0x4e, 0xba,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  10",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7a, 0xba, 0xbc, 0x4b, 0x3f, 0x51, 0x6c, 0x9a,
			0xaf, 0xb3, 0x5f, 0x41, 0x40, 0xb5, 0x48, 0xf9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  11",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaa, 0x18, 0x78, 0x24, 0xd9, 0xc4, 0x58, 0x2b,
			0x09, 0x16, 0x49, 0x3e, 0xcb, 0xde, 0x8c, 0x57,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  12",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1c, 0x0a, 0xd5, 0x53, 0x17, 0x7f, 0xd5, 0xea,
			0x10, 0x92, 0xc9, 0xd6, 0x26, 0xa2, 0x9d, 0xc4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  13",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa5, 0xdc, 0x46, 0xc3, 0x72, 0x61, 0x19, 0x41,
			0x24, 0xec, 0xae, 0xbd, 0x68, 0x04, 0x08, 0xec,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  14",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe4, 0xf2, 0xf2, 0xae, 0x23, 0xe9, 0xb1, 0x0b,
			0xac, 0xfa, 0x58, 0x60, 0x15, 0x31, 0xba, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  15",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb7, 0xd6, 0x7c, 0xf1, 0xa1, 0xe9, 0x1e, 0x8f,
			0xf3, 0xa5, 0x7a, 0x17, 0x2c, 0x7b, 0xf4, 0x12,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  16",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x26, 0x70, 0x6b, 0xe0, 0x69, 0x67, 0x88, 0x4e,
			0x84, 0x7d, 0x13, 0x71, 0x28, 0xce, 0x47, 0xb3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  17",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb2, 0xf8, 0xb4, 0x09, 0xb0, 0x58, 0x59, 0x09,
			0xaa, 0xd3, 0xa7, 0xb5, 0xa2, 0x19, 0x07, 0x2a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  18",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5e, 0x4b, 0x7b, 0xff, 0x02, 0x90, 0xc7, 0x83,
			0x44, 0xc5, 0x4a, 0x23, 0xb7, 0x22, 0xcd, 0x20,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  19",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x07, 0x09, 0x36, 0x57, 0x55, 0x2d, 0x44, 0x14,
			0x22, 0x7c, 0xe1, 0x61, 0xe9, 0xeb, 0xf7, 0xdd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  20",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe1, 0xaf, 0x1e, 0x7d, 0x8b, 0xc2, 0x25, 0xed,
			0x4d, 0xff, 0xb7, 0x71, 0xec, 0xbb, 0x9e, 0x67,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  21",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xef, 0x65, 0x55, 0x25, 0x36, 0x35, 0xd8, 0x43,
			0x21, 0x56, 0xcf, 0xd9, 0xc1, 0x1b, 0x14, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  22",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfb, 0x40, 0x35, 0x07, 0x4a, 0x5d, 0x42, 0x60,
			0xc9, 0x0c, 0xbd, 0x6d, 0xa6, 0xc3, 0xfc, 0xeb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  23",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x44, 0x6e, 0xe4, 0x16, 0xf9, 0xad, 0x1c, 0x10,
			0x3e, 0xb0, 0xcc, 0x96, 0x75, 0x1c, 0x88, 0xe1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  24",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0x8a, 0xe2, 0xa4, 0x63, 0x7a, 0xc0, 0xa7,
			0x89, 0x0a, 0x8f, 0xd1, 0x48, 0x54, 0x45, 0xc9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  25",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0x20, 0x12, 0xec, 0x8f, 0xad, 0xed, 0x08,
			0x25, 0xfb, 0x2f, 0xa7, 0x0a, 0xb3, 0x0c, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  26",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0x8a, 0x64, 0xb4, 0x6b, 0x5d, 0x88, 0xbf,
			0x7f, 0x24, 0x7d, 0x4d, 0xba, 0xf3, 0x8f, 0x05,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  27",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa1, 0x68, 0x25, 0x37, 0x62, 0xe2, 0xcc, 0x81,
			0xb4, 0x2d, 0x1e, 0x50, 0x01, 0x76, 0x26, 0x99,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  28",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0x41, 0xf8, 0x3b, 0x38, 0xce, 0x50, 0x32,
			0xc6, 0xcd, 0x7a, 0xf9, 0x8c, 0xf6, 0x20, 0x61,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  29",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x61, 0xa8, 0x99, 0x90, 0xcd, 0x14, 0x11, 0x75,
			0x0d, 0x5f, 0xb0, 0xdc, 0x98, 0x84, 0x47, 0xd4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  30",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb5, 0xac, 0xcc, 0x8e, 0xd6, 0x29, 0xed, 0xf8,
			0xc6, 0x8a, 0x53, 0x91, 0x83, 0xb1, 0xea, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  31",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb1, 0x6f, 0xa7, 0x1f, 0x84, 0x6b, 0x81, 0xa1,
			0x3f, 0x36, 0x1c, 0x43, 0xa8, 0x51, 0xf2, 0x90,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  32",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4f, 0xad, 0x6e, 0xfd, 0xff, 0x59, 0x75, 0xae,
			0xe7, 0x69, 0x22, 0x34, 0xbc, 0xd5, 0x44, 0x88,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  33",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xeb, 0xfd, 0xb0, 0x5a, 0x78, 0x3d, 0x03, 0x08,
			0x2d, 0xfe, 0x5f, 0xdd, 0x80, 0xa0, 0x0b, 0x17,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  34",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xeb, 0x81, 0xb5, 0x84, 0x76, 0x69, 0x97, 0xaf,
			0x6b, 0xa5, 0x52, 0x9d, 0x3b, 0xdd, 0x86, 0x09,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  35",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0c, 0xf4, 0xff, 0x4f, 0x49, 0xc8, 0xa0, 0xca,
			0x06, 0x0c, 0x44, 0x34, 0x99, 0xe2, 0x93, 0x13,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  36",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0x4b, 0xa8, 0xa8, 0xe0, 0x29, 0xf8, 0xb2,
			0x6d, 0x8a, 0xff, 0xf9, 0xdf, 0x13, 0x3b, 0xb6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  37",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfe, 0xfe, 0xbf, 0x64, 0x36, 0x0f, 0x38, 0xe4,
			0xe6, 0x35, 0x58, 0xf0, 0xff, 0xc5, 0x50, 0xc3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  38",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x12, 0xad, 0x98, 0xcb, 0xf7, 0x25, 0x13, 0x7d,
			0x6a, 0x81, 0x08, 0xc2, 0xbe, 0xd9, 0x93, 0x22,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  39",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6a, 0xfa, 0xa9, 0x96, 0x22, 0x61, 0x98, 0xb3,
			0xe2, 0x61, 0x04, 0x13, 0xce, 0x1b, 0x3f, 0x78,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  40",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2a, 0x8c, 0xe6, 0x74, 0x7a, 0x7e, 0x39, 0x36,
			0x78, 0x28, 0xe2, 0x90, 0x84, 0x85, 0x02, 0xd9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  41",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x22, 0x37, 0x36, 0xe8, 0xb8, 0xf8, 0x9c, 0xa1,
			0xe3, 0x7b, 0x6d, 0xea, 0xb4, 0x0f, 0xac, 0xf1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  42",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc0, 0xf7, 0x97, 0xe5, 0x04, 0x18, 0xb9, 0x5f,
			0xa6, 0x01, 0x33, 0x33, 0x91, 0x7a, 0x94, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  43",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa7, 0x58, 0xde, 0x37, 0xc2, 0xec, 0xe2, 0xa0,
			0x2c, 0x73, 0xc0, 0x1f, 0xed, 0xc9, 0xa1, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  44",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0x9b, 0x87, 0xae, 0x77, 0xba, 0xe7, 0x06,
			0x80, 0x39, 0x66, 0xc6, 0x6c, 0x73, 0xad, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  45",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd3, 0x65, 0xab, 0x8d, 0xf8, 0xff, 0xd7, 0x82,
			0xe3, 0x58, 0x12, 0x1a, 0x4a, 0x4f, 0xc5, 0x41,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  46",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc8, 0xdc, 0xd9, 0xe6, 0xf7, 0x5e, 0x6c, 0x36,
			0xc8, 0xda, 0xee, 0x04, 0x66, 0xf0, 0xed, 0x74,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  47",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc7, 0x9a, 0x63, 0x7b, 0xeb, 0x1c, 0x03, 0x04,
			0xf1, 0x40, 0x14, 0xc0, 0x37, 0xe7, 0x36, 0xdd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  48",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x10, 0x5f, 0x0a, 0x25, 0xe8, 0x4a, 0xc9, 0x30,
			0xd9, 0x96, 0x28, 0x1a, 0x5f, 0x95, 0x4d, 0xd9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  49",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x42, 0xe4, 0x07, 0x4b, 0x29, 0x27, 0x97, 0x3e,
			0x8d, 0x17, 0xff, 0xa9, 0x2f, 0x7f, 0xe6, 0x15,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  50",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4f, 0xe2, 0xa9, 0xd2, 0xc1, 0x82, 0x44, 0x49,
			0xc6, 0x9e, 0x3e, 0x03, 0x98, 0xf1, 0x29, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  51",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb7, 0xf2, 0x9c, 0x1e, 0x1f, 0x62, 0x84, 0x7a,
			0x15, 0x25, 0x3b, 0x28, 0xa1, 0xe9, 0xd7, 0x12,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  52",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0xed, 0x5d, 0x29, 0xb9, 0x03, 0xf3, 0x1e,
			0x89, 0x83, 0xef, 0x8b, 0x0a, 0x2b, 0xf9, 0x90,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  53",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0xb8, 0x07, 0x02, 0x70, 0x81, 0x0f, 0x9d,
			0x02, 0x3f, 0x9d, 0xd7, 0xff, 0x3b, 0x4a, 0xa2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  54",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x94, 0xd4, 0x6e, 0x15, 0x5c, 0x12, 0x28, 0xf6,
			0x1d, 0x1a, 0x0d, 0xb4, 0x81, 0x5e, 0xcc, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  55",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xca, 0x61, 0x08, 0xd1, 0xd9, 0x80, 0x71, 0x42,
			0x8e, 0xec, 0xee, 0xf1, 0x71, 0x4b, 0x96, 0xdd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  56",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdc, 0x5b, 0x25, 0xb7, 0x1b, 0x62, 0x96, 0xcf,
			0x73, 0xdd, 0x2c, 0xdc, 0xac, 0x2f, 0x70, 0xb1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  57",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x44, 0xab, 0xa9, 0x5e, 0x8a, 0x06, 0xa2, 0xd9,
			0xd3, 0x53, 0x0d, 0x26, 0x77, 0x87, 0x8c, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  58",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa5, 0x70, 0xd2, 0x0e, 0x89, 0xb4, 0x67, 0xe8,
			0xf5, 0x17, 0x60, 0x61, 0xb8, 0x1d, 0xd3, 0x96,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  59",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x75, 0x8f, 0x44, 0x67, 0xa5, 0xd8, 0xf1, 0xe7,
			0x30, 0x7d, 0xc3, 0x0b, 0x34, 0xe4, 0x04, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  60",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbc, 0xea, 0x28, 0xe9, 0x07, 0x1b, 0x5a, 0x23,
			0x02, 0x97, 0x0f, 0xf3, 0x52, 0x45, 0x1b, 0xc5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  61",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x75, 0x23, 0xc0, 0x0b, 0xc1, 0x77, 0xd3, 0x31,
			0xad, 0x31, 0x2e, 0x09, 0xc9, 0x01, 0x5c, 0x1c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  62",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0xac, 0x61, 0xe3, 0x18, 0x37, 0x47, 0xb3,
			0xf5, 0x83, 0x6d, 0xa2, 0x1a, 0x1b, 0xc4, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  63",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0x7b, 0x07, 0x57, 0x91, 0x87, 0x88, 0x80,
			0xb4, 0x41, 0x89, 0xd3, 0x52, 0x2b, 0x8c, 0x30,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  64",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x71, 0x32, 0xd0, 0xc0, 0xe4, 0xa0, 0x75, 0x93,
			0xcf, 0x12, 0xeb, 0xb1, 0x2b, 0xe7, 0x68, 0x8c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  65",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xef, 0xfb, 0xac, 0x16, 0x44, 0xde, 0xb0, 0xc7,
			0x84, 0x27, 0x5f, 0xe5, 0x6e, 0x19, 0xea, 0xd3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  66",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa0, 0x05, 0x06, 0x3f, 0x30, 0xf4, 0x22, 0x8b,
			0x37, 0x4e, 0x24, 0x59, 0x73, 0x8f, 0x26, 0xbb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  67",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x29, 0x97, 0x5b, 0x5f, 0x48, 0xbb, 0x68, 0xfc,
			0xbb, 0xc7, 0xce, 0xa9, 0x3b, 0x45, 0x2e, 0xd7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  68",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcf, 0x3f, 0x25, 0x76, 0xe2, 0xaf, 0xed, 0xc7,
			0x4b, 0xb1, 0xca, 0x7e, 0xee, 0xc1, 0xc0, 0xe7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  69",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x07, 0xc4, 0x03, 0xf5, 0xf9, 0x66, 0xe0, 0xe3,
			0xd9, 0xf2, 0x96, 0xd6, 0x22, 0x6d, 0xca, 0x28,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  70",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc8, 0xc2, 0x09, 0x08, 0x24, 0x9a, 0xb4, 0xa3,
			0x4d, 0x6d, 0xd0, 0xa3, 0x13, 0x27, 0xff, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  71",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc0, 0x54, 0x13, 0x29, 0xec, 0xb6, 0x15, 0x9a,
			0xb2, 0x3b, 0x7f, 0xc5, 0xe6, 0xa2, 0x1b, 0xca,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  72",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7a, 0xa1, 0xac, 0xf1, 0xa2, 0xed, 0x9b, 0xa7,
			0x2b, 0xc6, 0xde, 0xb3, 0x1d, 0x88, 0xb8, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  73",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x80, 0x8b, 0xd8, 0xed, 0xda, 0xbb, 0x6f, 0x3b,
			0xf0, 0xd5, 0xa8, 0xa2, 0x7b, 0xe1, 0xfe, 0x8a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  74",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0x3c, 0x7d, 0x76, 0x85, 0xe1, 0x4e, 0xc6,
			0x6b, 0xbb, 0x96, 0xb8, 0xf0, 0x5b, 0x6d, 0xdd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  75",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x32, 0x75, 0x2e, 0xef, 0xc8, 0xc2, 0xa9, 0x3f,
			0x91, 0xb6, 0xe7, 0x3e, 0xb0, 0x7c, 0xca, 0x6e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  76",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd8, 0x93, 0xe7, 0xd6, 0x2f, 0x6c, 0xe5, 0x02,
			0xc6, 0x4f, 0x75, 0xe2, 0x81, 0xf9, 0xc0, 0x00,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  77",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8d, 0xfd, 0x99, 0x9b, 0xe5, 0xd0, 0xcf, 0xa3,
			0x57, 0x32, 0xc0, 0xdd, 0xc8, 0x8f, 0xf5, 0xa5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  78",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x02, 0x64, 0x7c, 0x76, 0xa3, 0x00, 0xc3, 0x17,
			0x3b, 0x84, 0x14, 0x87, 0xeb, 0x2b, 0xae, 0x9f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  79",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x17, 0x2d, 0xf8, 0xb0, 0x2f, 0x04, 0xb5, 0x3a,
			0xda, 0xb0, 0x28, 0xb4, 0xe0, 0x1a, 0xcd, 0x87,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  80",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x05, 0x4b, 0x3b, 0xf4, 0x99, 0x8a, 0xeb, 0x05,
			0xaf, 0xd8, 0x7e, 0xc5, 0x36, 0x53, 0x3a, 0x36,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  81",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x37, 0x83, 0xf7, 0xbf, 0x44, 0xc9, 0x7f, 0x06,
			0x52, 0x58, 0xa6, 0x66, 0xca, 0xe0, 0x30, 0x20,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  82",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaa, 0xd4, 0xc8, 0xa6, 0x3f, 0x80, 0x95, 0x41,
			0x04, 0xde, 0x7b, 0x92, 0xce, 0xde, 0x1b, 0xe1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  83",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcb, 0xfe, 0x61, 0x81, 0x0f, 0xd5, 0x46, 0x7c,
			0xcd, 0xac, 0xb7, 0x58, 0x00, 0xf3, 0xac, 0x07,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  84",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x83, 0x0d, 0x8a, 0x25, 0x90, 0xf7, 0xd8, 0xe1,
			0xb5, 0x5a, 0x73, 0x7f, 0x4a, 0xf4, 0x5f, 0x34,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  85",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xff, 0xfc, 0xd4, 0x68, 0x3f, 0x85, 0x80, 0x58,
			0xe7, 0x43, 0x14, 0x67, 0x1d, 0x43, 0xfa, 0x2c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  86",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x52, 0x3d, 0x0b, 0xab, 0xbb, 0x82, 0xf4, 0x6e,
			0xbc, 0x9e, 0x70, 0xb1, 0xcd, 0x41, 0xdd, 0xd0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  87",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x34, 0x4a, 0xab, 0x37, 0x08, 0x0d, 0x74, 0x86,
			0xf7, 0xd5, 0x42, 0xa3, 0x09, 0xe5, 0x3e, 0xed,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  88",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0xc5, 0x60, 0x9d, 0x09, 0x06, 0xb2, 0x3a,
			0xb9, 0xca, 0xca, 0x81, 0x6f, 0x5d, 0xbe, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  89",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0x26, 0x02, 0x6e, 0xed, 0xd9, 0x1a, 0xdc,
			0x6d, 0x83, 0x1c, 0xdf, 0x98, 0x94, 0xbd, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  90",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x88, 0x33, 0x0b, 0xaa, 0x4f, 0x2b, 0x61, 0x8f,
			0xc9, 0xd9, 0xb0, 0x21, 0xbf, 0x50, 0x3d, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  91",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfc, 0x9e, 0x0e, 0xa2, 0x24, 0x80, 0xb0, 0xba,
			0xc9, 0x35, 0xc8, 0xa8, 0xeb, 0xef, 0xcd, 0xcf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  92",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x29, 0xca, 0x77, 0x9f, 0x39, 0x8f, 0xb0, 0x4f,
			0x86, 0x7d, 0xa7, 0xe8, 0xa4, 0x47, 0x56, 0xcb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  93",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x51, 0xf8, 0x9c, 0x42, 0x98, 0x57, 0x86, 0xbf,
			0xc4, 0x3c, 0x6d, 0xf8, 0xad, 0xa3, 0x68, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  94",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6a, 0xc1, 0xde, 0x5f, 0xb8, 0xf2, 0x1d, 0x87,
			0x4e, 0x91, 0xc5, 0x3b, 0x56, 0x0c, 0x50, 0xe3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  95",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x03, 0xaa, 0x90, 0x58, 0x49, 0x0e, 0xda, 0x30,
			0x60, 0x01, 0xa8, 0xa9, 0xf4, 0x8d, 0x0c, 0xa7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  96",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe3, 0x4e, 0xc7, 0x1d, 0x61, 0x28, 0xd4, 0x87,
			0x18, 0x65, 0xd6, 0x17, 0xc3, 0x0b, 0x37, 0xe3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  97",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x14, 0xbe, 0x1c, 0x53, 0x5b, 0x17, 0xca, 0xbd,
			0x0c, 0x4d, 0x93, 0x52, 0x9d, 0x69, 0xbf, 0x47,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  98",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc9, 0xef, 0x67, 0x75, 0x65, 0x07, 0xbe, 0xec,
			0x9d, 0xd3, 0x86, 0x28, 0x83, 0x47, 0x80, 0x44,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192  99",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0xe2, 0x31, 0xfa, 0x5a, 0x59, 0x48, 0xce,
			0x21, 0x34, 0xe9, 0x2f, 0xc0, 0x66, 0x4d, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 100",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x03, 0x19, 0x4b, 0x8e, 0x5d, 0xda, 0x55, 0x30,
			0xd0, 0xc6, 0x78, 0xc0, 0xb4, 0x8f, 0x5d, 0x92,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 101",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0xbd, 0x08, 0x6f, 0x23, 0x7c, 0xc4, 0xfd,
			0x99, 0xf4, 0xd7, 0x6b, 0xde, 0x6b, 0x48, 0x26,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 102",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0x25, 0x97, 0x61, 0xca, 0x17, 0x13, 0x0d,
			0x6e, 0xd8, 0x6d, 0x57, 0xcd, 0x79, 0x51, 0xee,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 103",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd7, 0xcb, 0xb3, 0xf3, 0x4b, 0x9b, 0x45, 0x0f,
			0x24, 0xb0, 0xe8, 0x51, 0x8e, 0x54, 0xda, 0x6d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 104",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x72, 0x5b, 0x9c, 0xae, 0xbe, 0x9f, 0x7f, 0x41,
			0x7f, 0x40, 0x68, 0xd0, 0xd2, 0xee, 0x20, 0xb3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 105",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9d, 0x92, 0x4b, 0x93, 0x4a, 0x90, 0xce, 0x1f,
			0xd3, 0x9b, 0x8a, 0x97, 0x94, 0xf8, 0x26, 0x72,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 106",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc5, 0x05, 0x62, 0xbf, 0x09, 0x45, 0x26, 0xa9,
			0x1c, 0x5b, 0xc6, 0x3c, 0x0c, 0x22, 0x49, 0x95,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 107",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0xf1, 0x18, 0x05, 0x04, 0x67, 0x43, 0xbd,
			0x74, 0xf5, 0x71, 0x88, 0xd9, 0x18, 0x8d, 0xf7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 108",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8d, 0xd2, 0x74, 0xbd, 0x0f, 0x1b, 0x58, 0xae,
			0x34, 0x5d, 0x9e, 0x72, 0x33, 0xf9, 0xb8, 0xf3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 109",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9d, 0x6b, 0xdc, 0x8f, 0x4c, 0xe5, 0xfe, 0xb0,
			0xf3, 0xbe, 0xd2, 0xe4, 0xb9, 0xa9, 0xbb, 0x0b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 110",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfd, 0x55, 0x48, 0xbc, 0xf3, 0xf4, 0x25, 0x65,
			0xf7, 0xef, 0xa9, 0x45, 0x62, 0x52, 0x8d, 0x46,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 111",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0xcc, 0xae, 0xbd, 0x3a, 0x4c, 0x3e, 0x80,
			0xb0, 0x63, 0x74, 0x81, 0x31, 0xba, 0x4a, 0x71,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 112",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe0, 0x3c, 0xb2, 0x3d, 0x9e, 0x11, 0xc9, 0xd9,
			0x3f, 0x11, 0x7e, 0x9c, 0x0a, 0x91, 0xb5, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 113",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x78, 0xf9, 0x33, 0xa2, 0x08, 0x1a, 0xc1, 0xdb,
			0x84, 0xf6, 0x9d, 0x10, 0xf4, 0x52, 0x3f, 0xe0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 114",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0x61, 0xf7, 0x41, 0x2e, 0xd3, 0x20, 0xde,
			0x0e, 0xdc, 0x88, 0x51, 0xc2, 0xe2, 0x43, 0x6f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 115",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0x64, 0xba, 0x1c, 0xd0, 0x4c, 0xe6, 0xba,
			0xb9, 0x84, 0x74, 0x33, 0x08, 0x14, 0xb4, 0xd4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 116",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x48, 0x39, 0x1b, 0xff, 0xb9, 0xcf, 0xff, 0x80,
			0xac, 0x23, 0x8c, 0x88, 0x6e, 0xf0, 0xa4, 0x61,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 117",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb8, 0xd2, 0xa6, 0x7d, 0xf5, 0xa9, 0x99, 0xfd,
			0xbf, 0x93, 0xed, 0xd0, 0x34, 0x32, 0x96, 0xc9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 118",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaa, 0xca, 0x73, 0x67, 0x39, 0x6b, 0x69, 0xa2,
			0x21, 0xbd, 0x63, 0x2b, 0xea, 0x38, 0x6e, 0xec,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 119",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0x0f, 0xd5, 0x02, 0x0d, 0xfe, 0x65, 0xf5,
			0xf1, 0x62, 0x93, 0xec, 0x92, 0xc6, 0xfd, 0x89,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 120",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x21, 0x62, 0x99, 0x5b, 0x82, 0x17, 0xa6, 0x7f,
			0x1a, 0xbc, 0x34, 0x2e, 0x14, 0x64, 0x06, 0xf8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 121",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc6, 0xa6, 0x16, 0x4b, 0x7a, 0x60, 0xba, 0xe4,
			0xe9, 0x86, 0xff, 0xac, 0x28, 0xdf, 0xad, 0xd9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 122",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x64, 0xe0, 0xd7, 0xf9, 0x00, 0xe3, 0xd9, 0xc8,
			0x3e, 0x4b, 0x8f, 0x96, 0x71, 0x7b, 0x21, 0x46,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 123",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1a, 0xd2, 0x56, 0x1d, 0xe8, 0xc1, 0x23, 0x2f,
			0x5d, 0x8d, 0xba, 0xb4, 0x73, 0x9b, 0x6c, 0xbb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 124",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0x96, 0x89, 0xe9, 0xa5, 0x57, 0xf5, 0x8b,
			0x1c, 0x3b, 0xf4, 0x0c, 0x97, 0xa9, 0x09, 0x64,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 125",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc4, 0x63, 0x7e, 0x4a, 0x5e, 0x63, 0x77, 0xf9,
			0xcc, 0x5a, 0x86, 0x38, 0x04, 0x5d, 0xe0, 0x29,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 126",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x49, 0x2e, 0x60, 0x7e, 0x5a, 0xea, 0x46, 0x88,
			0x59, 0x4b, 0x45, 0xf3, 0xae, 0xe3, 0xdf, 0x90,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 127",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe8, 0xc4, 0xe4, 0x38, 0x1f, 0xee, 0xc7, 0x40,
			0x54, 0x95, 0x4c, 0x05, 0xb7, 0x77, 0xa0, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 128",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x91, 0x54, 0x95, 0x14, 0x60, 0x5f, 0x38, 0x24,
			0x6c, 0x9b, 0x72, 0x4a, 0xd8, 0x39, 0xf0, 0x1d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 129",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x74, 0xb2, 0x4e, 0x3b, 0x6f, 0xef, 0xe4, 0x0a,
			0x4f, 0x9e, 0xf7, 0xac, 0x6e, 0x44, 0xd7, 0x6a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 130",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x24, 0x37, 0xa6, 0x83, 0xdc, 0x5d, 0x4b, 0x52,
			0xab, 0xb4, 0xa1, 0x23, 0xa8, 0xdf, 0x86, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 131",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbb, 0x28, 0x52, 0xc8, 0x91, 0xc5, 0x94, 0x7d,
			0x2e, 0xd4, 0x40, 0x32, 0xc4, 0x21, 0xb8, 0x5f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 132",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0x9f, 0x5f, 0xbd, 0x5e, 0x8a, 0x42, 0x64,
			0xc0, 0xa8, 0x5b, 0x80, 0x40, 0x9a, 0xfa, 0x5e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 133",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x30, 0xda, 0xb8, 0x09, 0xf8, 0x5a, 0x91, 0x7f,
			0xe9, 0x24, 0x73, 0x3f, 0x42, 0x4a, 0xc5, 0x89,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 134",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xea, 0xef, 0x5c, 0x1f, 0x8d, 0x60, 0x51, 0x92,
			0x64, 0x66, 0x95, 0xce, 0xad, 0xc6, 0x5f, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 135",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb8, 0xaa, 0x90, 0x04, 0x0b, 0x4c, 0x15, 0xa1,
			0x23, 0x16, 0xb7, 0x8e, 0x0f, 0x95, 0x86, 0xfc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 136",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0xfa, 0xc8, 0x29, 0x7c, 0xea, 0xab, 0xc8,
			0x7d, 0x45, 0x43, 0x50, 0x60, 0x1e, 0x06, 0x73,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 137",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9b, 0x47, 0xef, 0x56, 0x7a, 0xc2, 0x8d, 0xfe,
			0x48, 0x84, 0x92, 0xf1, 0x57, 0xe2, 0xb2, 0xe0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 138",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0x84, 0x26, 0x02, 0x7d, 0xdb, 0x96, 0x2b,
			0x5c, 0x5b, 0xa7, 0xeb, 0x8b, 0xc9, 0xab, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 139",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe9, 0x17, 0xfc, 0x77, 0xe7, 0x19, 0x92, 0xa1,
			0x2d, 0xbe, 0x4c, 0x18, 0x06, 0x8b, 0xec, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 140",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdc, 0xee, 0xbb, 0xc9, 0x88, 0x40, 0xf8, 0xae,
			0x6d, 0xaf, 0x76, 0x57, 0x3b, 0x7e, 0x56, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 141",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4e, 0x11, 0xa9, 0xf7, 0x42, 0x05, 0x12, 0x5b,
			0x61, 0xe0, 0xae, 0xe0, 0x47, 0xec, 0xa2, 0x0d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 142",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf6, 0x04, 0x67, 0xf5, 0x5a, 0x1f, 0x17, 0xea,
			0xb8, 0x8e, 0x80, 0x01, 0x20, 0xcb, 0xc2, 0x84,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 143",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd4, 0x36, 0x64, 0x9f, 0x60, 0x0b, 0x44, 0x9e,
			0xe2, 0x76, 0x53, 0x0f, 0x0c, 0xd8, 0x3c, 0x11,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 144",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3b, 0xc0, 0xe3, 0x65, 0x6a, 0x9e, 0x3a, 0xc7,
			0xcd, 0x37, 0x8a, 0x73, 0x7f, 0x53, 0xb6, 0x37,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 145",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6b, 0xac, 0xae, 0x63, 0xd3, 0x3b, 0x92, 0x8a,
			0xa8, 0x38, 0x0f, 0x8d, 0x54, 0xd8, 0x8c, 0x17,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 146",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x89, 0x35, 0xff, 0xbc, 0x75, 0xae, 0x62, 0x51,
			0xbf, 0x8e, 0x85, 0x9f, 0x08, 0x5a, 0xdc, 0xb9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 147",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x93, 0xdc, 0x49, 0x70, 0xfe, 0x35, 0xf6, 0x77,
			0x47, 0xcb, 0x05, 0x62, 0xc0, 0x6d, 0x87, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 148",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x14, 0xf9, 0xdf, 0x85, 0x89, 0x75, 0x85, 0x17,
			0x97, 0xba, 0x60, 0x4f, 0xb0, 0xd1, 0x6c, 0xc7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 149",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x02, 0xea, 0x0c, 0x98, 0xdc, 0xa1, 0x0b, 0x38,
			0xc2, 0x1b, 0x3b, 0x14, 0xe8, 0xd1, 0xb7, 0x1f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 150",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8f, 0x09, 0x1b, 0x1b, 0x5b, 0x07, 0x49, 0xb2,
			0xad, 0xc8, 0x03, 0xe6, 0x3d, 0xda, 0x9b, 0x72,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 151",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x05, 0xb3, 0x89, 0xe3, 0x32, 0x2c, 0x6d, 0xa0,
			0x83, 0x84, 0x34, 0x5a, 0x41, 0x37, 0xfd, 0x08,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 152",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x38, 0x13, 0x08, 0xc4, 0x38, 0xf3, 0x5b, 0x39,
			0x9f, 0x10, 0xad, 0x71, 0xb0, 0x50, 0x27, 0xd8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 153",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x68, 0xc2, 0x30, 0xfc, 0xfa, 0x92, 0x79, 0xc3,
			0x40, 0x9f, 0xc4, 0x23, 0xe2, 0xac, 0xbe, 0x04,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 154",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1c, 0x84, 0xa4, 0x75, 0xac, 0xb0, 0x11, 0xf3,
			0xf5, 0x9f, 0x4f, 0x46, 0xb7, 0x62, 0x74, 0xc0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 155",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x45, 0x11, 0x9b, 0x68, 0xcb, 0x3f, 0x83, 0x99,
			0xee, 0x60, 0x06, 0x6b, 0x56, 0x11, 0xa4, 0xd7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 156",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x94, 0x23, 0x76, 0x2f, 0x52, 0x7a, 0x40, 0x60,
			0xff, 0xca, 0x31, 0x2d, 0xcc, 0xa2, 0x2a, 0x16,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 157",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf3, 0x61, 0xa2, 0x74, 0x5a, 0x33, 0xf0, 0x56,
			0xa5, 0xac, 0x6a, 0xce, 0x2f, 0x08, 0xe3, 0x44,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 158",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5e, 0xf1, 0x45, 0x76, 0x6e, 0xca, 0x84, 0x9f,
			0x5d, 0x01, 0x15, 0x36, 0xa6, 0x55, 0x7f, 0xdb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 159",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc9, 0xaf, 0x27, 0xb2, 0xc8, 0x9c, 0x9b, 0x4c,
			0xf4, 0xa0, 0xc4, 0x10, 0x6a, 0xc8, 0x03, 0x18,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 160",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfb, 0x9c, 0x4f, 0x16, 0xc6, 0x21, 0xf4, 0xea,
			0xb7, 0xe9, 0xac, 0x1d, 0x75, 0x51, 0xdd, 0x57,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 161",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x13, 0x8e, 0x06, 0xfb, 0xa4, 0x66, 0xfa, 0x70,
			0x85, 0x4d, 0x8c, 0x2e, 0x52, 0x4c, 0xff, 0xb2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 162",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfb, 0x4b, 0xc7, 0x8b, 0x22, 0x50, 0x70, 0x77,
			0x3f, 0x04, 0xc4, 0x04, 0x66, 0xd4, 0xe9, 0x0c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 163",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8b, 0x2c, 0xbf, 0xf1, 0xed, 0x01, 0x50, 0xfe,
			0xda, 0x8a, 0x47, 0x99, 0xbe, 0x94, 0x55, 0x1f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 164",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x08, 0xb3, 0x0d, 0x7b, 0x3f, 0x27, 0x96, 0x27,
			0x09, 0xa3, 0x6b, 0xca, 0xdf, 0xb9, 0x74, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 165",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfd, 0xf6, 0xd3, 0x2e, 0x04, 0x4d, 0x77, 0xad,
			0xcf, 0x37, 0xfb, 0x97, 0xac, 0x21, 0x33, 0x26,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 166",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x93, 0xcb, 0x28, 0x4e, 0xcd, 0xcf, 0xd7, 0x81,
			0xa8, 0xaf, 0xe3, 0x20, 0x77, 0x94, 0x9e, 0x88,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 167",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7b, 0x01, 0x7b, 0xb0, 0x2e, 0xc8, 0x7b, 0x2b,
			0x94, 0xc9, 0x6e, 0x40, 0xa2, 0x6f, 0xc7, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 168",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc5, 0xc0, 0x38, 0xb6, 0x99, 0x06, 0x64, 0xab,
			0x08, 0xa3, 0xaa, 0xa5, 0xdf, 0x9f, 0x32, 0x66,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 169",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4b, 0x70, 0x20, 0xbe, 0x37, 0xfa, 0xb6, 0x25,
			0x9b, 0x2a, 0x27, 0xf4, 0xec, 0x55, 0x15, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 170",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x60, 0x13, 0x67, 0x03, 0x37, 0x4f, 0x64, 0xe8,
			0x60, 0xb4, 0x8c, 0xe3, 0x1f, 0x93, 0x07, 0x16,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 171",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8d, 0x63, 0xa2, 0x69, 0xb1, 0x4d, 0x50, 0x6c,
			0xcc, 0x40, 0x1a, 0xb8, 0xa9, 0xf1, 0xb5, 0x91,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 172",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd3, 0x17, 0xf8, 0x1d, 0xc6, 0xaa, 0x45, 0x4a,
			0xee, 0x4b, 0xd4, 0xa5, 0xa5, 0xcf, 0xf4, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 173",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdd, 0xde, 0xce, 0xcd, 0x53, 0x54, 0xf0, 0x4d,
			0x53, 0x0d, 0x76, 0xed, 0x88, 0x42, 0x46, 0xeb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 174",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x41, 0xc5, 0x20, 0x5c, 0xc8, 0xfd, 0x8e, 0xda,
			0x9a, 0x3c, 0xff, 0xd2, 0x51, 0x8f, 0x36, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 175",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcf, 0x42, 0xfb, 0x47, 0x42, 0x93, 0xd9, 0x6e,
			0xca, 0x9d, 0xb1, 0xb3, 0x7b, 0x1b, 0xa6, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 176",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa2, 0x31, 0x69, 0x26, 0x07, 0x16, 0x9b, 0x4e,
			0xcd, 0xea, 0xd5, 0xcd, 0x3b, 0x10, 0xdb, 0x3e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 177",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xac, 0xe4, 0xb9, 0x1c, 0x9c, 0x66, 0x9e, 0x77,
			0xe7, 0xac, 0xac, 0xd1, 0x98, 0x59, 0xed, 0x49,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 178",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x75, 0xdb, 0x7c, 0xfd, 0x4a, 0x7b, 0x2b, 0x62,
			0xab, 0x78, 0xa4, 0x8f, 0x3d, 0xda, 0xf4, 0xaf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 179",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc1, 0xfa, 0xba, 0x2d, 0x46, 0xe2, 0x59, 0xcf,
			0x48, 0x0d, 0x7c, 0x38, 0xe4, 0x57, 0x2a, 0x58,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 180",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x24, 0x1c, 0x45, 0xbc, 0x6a, 0xe1, 0x6d, 0xee,
			0x6e, 0xb7, 0xbe, 0xa1, 0x28, 0x70, 0x15, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 181",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8f, 0xd0, 0x30, 0x57, 0xcf, 0x13, 0x64, 0x42,
			0x0c, 0x2b, 0x78, 0x06, 0x9a, 0x3e, 0x25, 0x02,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 182",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdd, 0xb5, 0x05, 0xe6, 0xcc, 0x13, 0x84, 0xcb,
			0xae, 0xc1, 0xdf, 0x90, 0xb8, 0x0b, 0xeb, 0x20,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 183",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0x74, 0xa3, 0xbe, 0xd2, 0x7b, 0xf4, 0xbd,
			0x36, 0x22, 0xf9, 0xf5, 0xfe, 0x20, 0x83, 0x06,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 184",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb6, 0x87, 0xf2, 0x6a, 0x89, 0xcf, 0xbf, 0xbb,
			0x8e, 0x5e, 0xea, 0xc5, 0x40, 0x55, 0x31, 0x5e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 185",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x05, 0x47, 0xdd, 0x32, 0xd3, 0xb2, 0x9a, 0xb6,
			0xa4, 0xca, 0xeb, 0x60, 0x6c, 0x5b, 0x6f, 0x78,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 186",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x18, 0x68, 0x61, 0xf8, 0xbc, 0x53, 0x86, 0xd3,
			0x1f, 0xb7, 0x7f, 0x72, 0x0c, 0x32, 0x26, 0xe6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 187",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xea, 0xcf, 0x1e, 0x6c, 0x42, 0x24, 0xef, 0xb3,
			0x89, 0x00, 0xb1, 0x85, 0xab, 0x1d, 0xfd, 0x42,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 188",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0x41, 0xaa, 0xb0, 0x5a, 0x42, 0xd3, 0x19,
			0xde, 0x81, 0xd8, 0x74, 0xf5, 0xc7, 0xb9, 0x0d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 189",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5e, 0xb9, 0xbc, 0x75, 0x9e, 0x2a, 0xd8, 0xd2,
			0x14, 0x0a, 0x6c, 0x76, 0x2a, 0xe9, 0xe1, 0xab,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 190",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x01, 0x85, 0x96, 0xe1, 0x5e, 0x78, 0xe2, 0xc0,
			0x64, 0x15, 0x9d, 0xef, 0xce, 0x5f, 0x30, 0x85,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey192 191",
		.keylen	 = 192,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdd, 0x8a, 0x49, 0x35, 0x14, 0x23, 0x1c, 0xbf,
			0x56, 0xec, 0xce, 0xe4, 0xc4, 0x08, 0x89, 0xfb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   0",
		.keylen	 = 256,
		.key	 = {
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe3, 0x5a, 0x6d, 0xcb, 0x19, 0xb2, 0x01, 0xa0,
			0x1e, 0xbc, 0xfa, 0x8a, 0xa2, 0x2b, 0x57, 0x59,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   1",
		.keylen	 = 256,
		.key	 = {
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb2, 0x91, 0x69, 0xcd, 0xcf, 0x2d, 0x83, 0xe8,
			0x38, 0x12, 0x5a, 0x12, 0xee, 0x6a, 0xa4, 0x00,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   2",
		.keylen	 = 256,
		.key	 = {
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd8, 0xf3, 0xa7, 0x2f, 0xc3, 0xcd, 0xf7, 0x4d,
			0xfa, 0xf6, 0xc3, 0xe6, 0xb9, 0x7b, 0x2f, 0xa6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   3",
		.keylen	 = 256,
		.key	 = {
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1c, 0x77, 0x76, 0x79, 0xd5, 0x00, 0x37, 0xc7,
			0x94, 0x91, 0xa9, 0x4d, 0xa7, 0x6a, 0x9a, 0x35,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   4",
		.keylen	 = 256,
		.key	 = {
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9c, 0xf4, 0x89, 0x3e, 0xca, 0xfa, 0x0a, 0x02,
			0x47, 0xa8, 0x98, 0xe0, 0x40, 0x69, 0x15, 0x59,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   5",
		.keylen	 = 256,
		.key	 = {
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8f, 0xbb, 0x41, 0x37, 0x03, 0x73, 0x53, 0x26,
			0x31, 0x0a, 0x26, 0x9b, 0xd3, 0xaa, 0x94, 0xb2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   6",
		.keylen	 = 256,
		.key	 = {
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x60, 0xe3, 0x22, 0x46, 0xbe, 0xd2, 0xb0, 0xe8,
			0x59, 0xe5, 0x5c, 0x1c, 0xc6, 0xb2, 0x65, 0x02,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   7",
		.keylen	 = 256,
		.key	 = {
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xec, 0x52, 0xa2, 0x12, 0xf8, 0x0a, 0x09, 0xdf,
			0x63, 0x17, 0x02, 0x1b, 0xc2, 0xa9, 0x81, 0x9e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   8",
		.keylen	 = 256,
		.key	 = {
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf2, 0x3e, 0x5b, 0x60, 0x0e, 0xb7, 0x0d, 0xbc,
			0xcf, 0x6c, 0x0b, 0x1d, 0x9a, 0x68, 0x18, 0x2c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256   9",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa3, 0xf5, 0x99, 0xd6, 0x3a, 0x82, 0xa9, 0x68,
			0xc3, 0x3f, 0xe2, 0x65, 0x90, 0x74, 0x59, 0x70,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  10",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd1, 0xcc, 0xb9, 0xb1, 0x33, 0x70, 0x02, 0xcb,
			0xac, 0x42, 0xc5, 0x20, 0xb5, 0xd6, 0x77, 0x22,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  11",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0x11, 0x1f, 0x6c, 0x37, 0xcf, 0x40, 0xa1,
			0x15, 0x9d, 0x00, 0xfb, 0x59, 0xfb, 0x04, 0x88,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  12",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdc, 0x43, 0xb5, 0x1a, 0xb6, 0x09, 0x05, 0x23,
			0x72, 0x98, 0x9a, 0x26, 0xe9, 0xcd, 0xd7, 0x14,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  13",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4d, 0xce, 0xde, 0x8d, 0xa9, 0xe2, 0x57, 0x8f,
			0x39, 0x70, 0x3d, 0x44, 0x33, 0xdc, 0x64, 0x59,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  14",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1a, 0x4c, 0x1c, 0x26, 0x3b, 0xbc, 0xcf, 0xaf,
			0xc1, 0x17, 0x82, 0x89, 0x46, 0x85, 0xe3, 0xa8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  15",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x93, 0x7a, 0xd8, 0x48, 0x80, 0xdb, 0x50, 0x61,
			0x34, 0x23, 0xd6, 0xd5, 0x27, 0xa2, 0x82, 0x3d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  16",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x61, 0x0b, 0x71, 0xdf, 0xc6, 0x88, 0xe1, 0x50,
			0xd8, 0x15, 0x2c, 0x5b, 0x35, 0xeb, 0xc1, 0x4d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  17",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0xef, 0x24, 0x95, 0xda, 0xbf, 0x32, 0x38,
			0x85, 0xaa, 0xb3, 0x9c, 0x80, 0xf1, 0x8d, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  18",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x63, 0x3c, 0xaf, 0xea, 0x39, 0x5b, 0xc0, 0x3a,
			0xda, 0xe3, 0xa1, 0xe2, 0x06, 0x8e, 0x4b, 0x4e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  19",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6e, 0x1b, 0x48, 0x2b, 0x53, 0x76, 0x1c, 0xf6,
			0x31, 0x81, 0x9b, 0x74, 0x9a, 0x6f, 0x37, 0x24,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  20",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0x6e, 0x6f, 0x85, 0x1a, 0xb5, 0x2c, 0x77,
			0x19, 0x98, 0xdb, 0xb2, 0xd7, 0x1c, 0x75, 0xa9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  21",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x85, 0xf2, 0xba, 0x84, 0xf8, 0xc3, 0x07, 0xcf,
			0x52, 0x5e, 0x12, 0x4c, 0x3e, 0x22, 0xe6, 0xcc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  22",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6b, 0xcc, 0xa9, 0x8b, 0xf6, 0xa8, 0x35, 0xfa,
			0x64, 0x95, 0x5f, 0x72, 0xde, 0x41, 0x15, 0xfe,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  23",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2c, 0x75, 0xe2, 0xd3, 0x6e, 0xeb, 0xd6, 0x54,
			0x11, 0xf1, 0x4f, 0xd0, 0xeb, 0x1d, 0x2a, 0x06,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  24",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbd, 0x49, 0x29, 0x50, 0x06, 0x25, 0x0f, 0xfc,
			0xa5, 0x10, 0x0b, 0x60, 0x07, 0xa0, 0xea, 0xde,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  25",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa1, 0x90, 0x52, 0x7d, 0x0e, 0xf7, 0xc7, 0x0f,
			0x45, 0x9c, 0xd3, 0x94, 0x0d, 0xf3, 0x16, 0xec,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  26",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbb, 0xd1, 0x09, 0x7a, 0x62, 0x43, 0x3f, 0x79,
			0x44, 0x9f, 0xa9, 0x7d, 0x4e, 0xe8, 0x0d, 0xbf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  27",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x07, 0x05, 0x8e, 0x40, 0x8f, 0x5b, 0x99, 0xb0,
			0xe0, 0xf0, 0x61, 0xa1, 0x76, 0x1b, 0x5b, 0x3b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  28",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5f, 0xd1, 0xf1, 0x3f, 0xa0, 0xf3, 0x1e, 0x37,
			0xfa, 0xbd, 0xe3, 0x28, 0xf8, 0x94, 0xea, 0xc2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  29",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfc, 0x4a, 0xf7, 0xc9, 0x48, 0xdf, 0x26, 0xe2,
			0xef, 0x3e, 0x01, 0xc1, 0xee, 0x5b, 0x8f, 0x6f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  30",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0x9f, 0xd7, 0x20, 0x8f, 0xb9, 0x2d, 0x44,
			0xa0, 0x74, 0xa6, 0x77, 0xee, 0x98, 0x61, 0xac,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  31",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xad, 0x9f, 0xc6, 0x13, 0xa7, 0x03, 0x25, 0x1b,
			0x54, 0xc6, 0x4a, 0x0e, 0x76, 0x43, 0x17, 0x11,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  32",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x33, 0xac, 0x9e, 0xcc, 0xc4, 0xcc, 0x75, 0xe2,
			0x71, 0x16, 0x18, 0xf8, 0x0b, 0x15, 0x48, 0xe8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  33",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x20, 0x25, 0xc7, 0x4b, 0x8a, 0xd8, 0xf4, 0xcd,
			0xa1, 0x7e, 0xe2, 0x04, 0x9c, 0x4c, 0x90, 0x2d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  34",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf8, 0x5c, 0xa0, 0x5f, 0xe5, 0x28, 0xf1, 0xce,
			0x9b, 0x79, 0x01, 0x66, 0xe8, 0xd5, 0x51, 0xe7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  35",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6f, 0x62, 0x38, 0xd8, 0x96, 0x60, 0x48, 0xd4,
			0x96, 0x71, 0x54, 0xe0, 0xda, 0xd5, 0xa6, 0xc9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  36",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf2, 0xb2, 0x1b, 0x4e, 0x76, 0x40, 0xa9, 0xb3,
			0x34, 0x6d, 0xe8, 0xb8, 0x2f, 0xb4, 0x1e, 0x49,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  37",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf8, 0x36, 0xf2, 0x51, 0xad, 0x1d, 0x11, 0xd4,
			0x9d, 0xc3, 0x44, 0x62, 0x8b, 0x18, 0x84, 0xe1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  38",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x07, 0x7e, 0x94, 0x70, 0xae, 0x7a, 0xbe, 0xa5,
			0xa9, 0x76, 0x9d, 0x49, 0x18, 0x26, 0x28, 0xc3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  39",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe0, 0xdc, 0xc2, 0xd2, 0x7f, 0xc9, 0x86, 0x56,
			0x33, 0xf8, 0x52, 0x23, 0xcf, 0x0d, 0x61, 0x1f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  40",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbe, 0x66, 0xcf, 0xea, 0x2f, 0xec, 0xd6, 0xbf,
			0x0e, 0xc7, 0xb4, 0x35, 0x2c, 0x99, 0xbc, 0xaa,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  41",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdf, 0x31, 0x14, 0x4f, 0x87, 0xa2, 0xef, 0x52,
			0x3f, 0xac, 0xdc, 0xf2, 0x1a, 0x42, 0x78, 0x04,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  42",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb5, 0xbb, 0x0f, 0x56, 0x29, 0xfb, 0x6a, 0xae,
			0x5e, 0x18, 0x39, 0xa3, 0xc3, 0x62, 0x5d, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  43",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3c, 0x9d, 0xb3, 0x33, 0x53, 0x06, 0xfe, 0x1e,
			0xc6, 0x12, 0xbd, 0xbf, 0xae, 0x6b, 0x60, 0x28,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  44",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3d, 0xd5, 0xc3, 0x46, 0x34, 0xa7, 0x9d, 0x3c,
			0xfc, 0xc8, 0x33, 0x97, 0x60, 0xe6, 0xf5, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  45",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0xbd, 0xa1, 0x18, 0xa3, 0xed, 0x7a, 0xf3,
			0x14, 0xfa, 0x2c, 0xcc, 0x5c, 0x07, 0xb7, 0x61,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  46",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x29, 0x37, 0xa6, 0x4f, 0x7d, 0x4f, 0x46, 0xfe,
			0x6f, 0xea, 0x3b, 0x34, 0x9e, 0xc7, 0x8e, 0x38,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  47",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x22, 0x5f, 0x06, 0x8c, 0x28, 0x47, 0x66, 0x05,
			0x73, 0x5a, 0xd6, 0x71, 0xbb, 0x8f, 0x39, 0xf3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  48",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xae, 0x68, 0x2c, 0x5e, 0xcd, 0x71, 0x89, 0x8e,
			0x08, 0x94, 0x2a, 0xc9, 0xaa, 0x89, 0x87, 0x5c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  49",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5e, 0x03, 0x1c, 0xb9, 0xd6, 0x76, 0xc3, 0x02,
			0x2d, 0x7f, 0x26, 0x22, 0x7e, 0x85, 0xc3, 0x8f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  50",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa7, 0x84, 0x63, 0xfb, 0x06, 0x4d, 0xb5, 0xd5,
			0x2b, 0xb6, 0x4b, 0xfe, 0xf6, 0x4f, 0x2d, 0xda,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  51",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8a, 0xa9, 0xb7, 0x5e, 0x78, 0x45, 0x93, 0x87,
			0x6c, 0x53, 0xa0, 0x0e, 0xae, 0x5a, 0xf5, 0x2b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  52",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3f, 0x84, 0x56, 0x6d, 0xf2, 0x3d, 0xa4, 0x8a,
			0xf6, 0x92, 0x72, 0x2f, 0xe9, 0x80, 0x57, 0x3a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  53",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x31, 0x69, 0x0b, 0x5e, 0xd4, 0x1c, 0x7e, 0xb4,
			0x2a, 0x1e, 0x83, 0x27, 0x0a, 0x7f, 0xf0, 0xe6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  54",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0xdd, 0x77, 0x02, 0x64, 0x6d, 0x55, 0xf0,
			0x83, 0x65, 0xe4, 0x77, 0xd3, 0x59, 0x0e, 0xda,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  55",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4c, 0x02, 0x2a, 0xc6, 0x2b, 0x3c, 0xb7, 0x8d,
			0x73, 0x9c, 0xc6, 0x7b, 0x3e, 0x20, 0xbb, 0x7e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  56",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x09, 0x2f, 0xa1, 0x37, 0xce, 0x18, 0xb5, 0xdf,
			0xe7, 0x90, 0x6f, 0x55, 0x0b, 0xb1, 0x33, 0x70,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  57",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3e, 0x0c, 0xda, 0xdf, 0x2e, 0x68, 0x35, 0x3c,
			0x00, 0x27, 0x67, 0x2c, 0x97, 0x14, 0x4d, 0xd3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  58",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd8, 0xc4, 0xb2, 0x00, 0xb3, 0x83, 0xfc, 0x1f,
			0x2b, 0x2e, 0xa6, 0x77, 0x61, 0x8a, 0x1d, 0x27,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  59",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0x82, 0x5f, 0x99, 0xb0, 0xe9, 0xbb, 0x34,
			0x77, 0xc1, 0xc0, 0x71, 0x3b, 0x01, 0x5a, 0xac,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  60",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf8, 0xb9, 0xff, 0xfb, 0x5c, 0x18, 0x7f, 0x7d,
			0xdc, 0x7a, 0xb1, 0x0f, 0x4f, 0xb7, 0x75, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  61",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xff, 0xb4, 0xe8, 0x7a, 0x32, 0xb3, 0x7d, 0x6f,
			0x2c, 0x83, 0x28, 0xd3, 0xb5, 0x37, 0x78, 0x02,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  62",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0x76, 0xc1, 0x3a, 0x5d, 0x22, 0x0f, 0x4d,
			0xa9, 0x22, 0x4e, 0x74, 0x89, 0x63, 0x91, 0xce,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  63",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x94, 0xef, 0xe7, 0xa0, 0xe2, 0xe0, 0x31, 0xe2,
			0x53, 0x6d, 0xa0, 0x1d, 0xf7, 0x99, 0xc9, 0x27,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  64",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8f, 0x8f, 0xd8, 0x22, 0x68, 0x0a, 0x85, 0x97,
			0x4e, 0x53, 0xa5, 0xa8, 0xeb, 0x9d, 0x38, 0xde,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  65",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe0, 0xf0, 0xa9, 0x1b, 0x2e, 0x45, 0xf8, 0xcc,
			0x37, 0xb7, 0x80, 0x5a, 0x30, 0x42, 0x58, 0x8d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  66",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x59, 0x7a, 0x62, 0x52, 0x25, 0x5e, 0x46, 0xd6,
			0x36, 0x4d, 0xbe, 0xed, 0xa3, 0x1e, 0x27, 0x9c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  67",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf5, 0x1a, 0x0f, 0x69, 0x44, 0x42, 0xb8, 0xf0,
			0x55, 0x71, 0x79, 0x7f, 0xec, 0x7e, 0xe8, 0xbf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  68",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9f, 0xf0, 0x71, 0xb1, 0x65, 0xb5, 0x19, 0x8a,
			0x93, 0xdd, 0xde, 0xeb, 0xc5, 0x4d, 0x09, 0xb5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  69",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc2, 0x0a, 0x19, 0xfd, 0x57, 0x58, 0xb0, 0xc4,
			0xbc, 0x1a, 0x5d, 0xf8, 0x9c, 0xf7, 0x38, 0x77,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  70",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0x12, 0x01, 0x66, 0x30, 0x71, 0x19, 0xca,
			0x22, 0x80, 0xe9, 0x31, 0x56, 0x68, 0xe9, 0x6f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  71",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4b, 0x3b, 0x9f, 0x1e, 0x09, 0x9c, 0x2a, 0x09,
			0xdc, 0x09, 0x1e, 0x90, 0xe4, 0xf1, 0x8f, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  72",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xeb, 0x04, 0x0b, 0x89, 0x1d, 0x4b, 0x37, 0xf6,
			0x85, 0x1f, 0x7e, 0xc2, 0x19, 0xcd, 0x3f, 0x6d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  73",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9f, 0x0f, 0xde, 0xc0, 0x8b, 0x7f, 0xd7, 0x9a,
			0xa3, 0x95, 0x35, 0xbe, 0xa4, 0x2d, 0xb9, 0x2a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  74",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2e, 0x70, 0xf1, 0x68, 0xfc, 0x74, 0xbf, 0x91,
			0x1d, 0xf2, 0x40, 0xbc, 0xd2, 0xce, 0xf2, 0x36,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  75",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x46, 0x2c, 0xcd, 0x7f, 0x5f, 0xd1, 0x10, 0x8d,
			0xbc, 0x15, 0x2f, 0x3c, 0xac, 0xad, 0x32, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  76",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa4, 0xaf, 0x53, 0x4a, 0x7d, 0x0b, 0x64, 0x3a,
			0x01, 0x86, 0x87, 0x85, 0xd8, 0x6d, 0xfb, 0x95,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  77",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xab, 0x98, 0x02, 0x96, 0x19, 0x7e, 0x1a, 0x50,
			0x22, 0x32, 0x6c, 0x31, 0xda, 0x4b, 0xf6, 0xf3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  78",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf9, 0x7d, 0x57, 0xb3, 0x33, 0x3b, 0x62, 0x81,
			0xb0, 0x7d, 0x48, 0x6d, 0xb2, 0xd4, 0xe2, 0x0c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  79",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf3, 0x3f, 0xa3, 0x67, 0x20, 0x23, 0x1a, 0xfe,
			0x4c, 0x75, 0x9a, 0xde, 0x6b, 0xd6, 0x2e, 0xb6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  80",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfd, 0xcf, 0xac, 0x0c, 0x02, 0xca, 0x53, 0x83,
			0x43, 0xc6, 0x81, 0x17, 0xe0, 0xa1, 0x59, 0x38,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  81",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xad, 0x49, 0x16, 0xf5, 0xee, 0x57, 0x72, 0xbe,
			0x76, 0x4f, 0xc0, 0x27, 0xb8, 0xa6, 0xe5, 0x39,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  82",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2e, 0x16, 0x87, 0x3e, 0x16, 0x78, 0x61, 0x0d,
			0x7e, 0x14, 0xc0, 0x2d, 0x00, 0x2e, 0xa8, 0x45,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  83",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4e, 0x6e, 0x62, 0x7c, 0x1a, 0xcc, 0x51, 0x34,
			0x00, 0x53, 0xa8, 0x23, 0x6d, 0x57, 0x95, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  84",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xab, 0x0c, 0x84, 0x10, 0xae, 0xee, 0xad, 0x92,
			0xfe, 0xec, 0x1e, 0xb4, 0x30, 0xd6, 0x52, 0xcb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  85",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe8, 0x6f, 0x7e, 0x23, 0xe8, 0x35, 0xe1, 0x14,
			0x97, 0x7f, 0x60, 0xe1, 0xa5, 0x92, 0x20, 0x2e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  86",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe6, 0x8a, 0xd5, 0x05, 0x5a, 0x36, 0x70, 0x41,
			0xfa, 0xde, 0x09, 0xd9, 0xa7, 0x0a, 0x79, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  87",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x07, 0x91, 0x82, 0x3a, 0x3c, 0x66, 0x6b, 0xb6,
			0x16, 0x28, 0x25, 0xe7, 0x86, 0x06, 0xa7, 0xfe,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  88",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdc, 0xca, 0x36, 0x6a, 0x9b, 0xf4, 0x7b, 0x7b,
			0x86, 0x8b, 0x77, 0xe2, 0x5c, 0x18, 0xa3, 0x64,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  89",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x68, 0x4c, 0x9e, 0xfc, 0x23, 0x7e, 0x4a, 0x44,
			0x29, 0x65, 0xf8, 0x4b, 0xce, 0x20, 0x24, 0x7a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  90",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0x58, 0x41, 0x1f, 0xfb, 0xe6, 0x3f, 0xdb,
			0x9c, 0x8a, 0xa1, 0xbf, 0xae, 0xd6, 0x7b, 0x52,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  91",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x04, 0xbc, 0x3d, 0xa2, 0x17, 0x9c, 0x30, 0x15,
			0x49, 0x8b, 0x0e, 0x03, 0x91, 0x0d, 0xb5, 0xb8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  92",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0x07, 0x1e, 0xea, 0xb3, 0xf9, 0x35, 0xdb,
			0xc2, 0x5d, 0x00, 0x84, 0x14, 0x60, 0x26, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  93",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0e, 0xbd, 0x7c, 0x30, 0xed, 0x20, 0x16, 0xe0,
			0x8b, 0xa8, 0x06, 0xdd, 0xb0, 0x08, 0xbc, 0xc8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  94",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x15, 0xc6, 0xbe, 0xcf, 0x0f, 0x4c, 0xec, 0x71,
			0x29, 0xcb, 0xd2, 0x2d, 0x1a, 0x79, 0xb1, 0xb8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  95",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0a, 0xee, 0xde, 0x5b, 0x91, 0xf7, 0x21, 0x70,
			0x0e, 0x9e, 0x62, 0xed, 0xbf, 0x60, 0xb7, 0x81,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  96",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x26, 0x65, 0x81, 0xaf, 0x0d, 0xcf, 0xbe, 0xd1,
			0x58, 0x5e, 0x0a, 0x24, 0x2c, 0x64, 0xb8, 0xdf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  97",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x66, 0x93, 0xdc, 0x91, 0x16, 0x62, 0xae, 0x47,
			0x32, 0x16, 0xba, 0x22, 0x18, 0x9a, 0x51, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  98",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x76, 0x06, 0xfa, 0x36, 0xd8, 0x64, 0x73, 0xe6,
			0xfb, 0x3a, 0x1b, 0xb0, 0xe2, 0xc0, 0xad, 0xf5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256  99",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0x20, 0x78, 0xe9, 0xe1, 0x1f, 0xbb, 0x78,
			0xe2, 0x6f, 0xfb, 0x88, 0x99, 0xe9, 0x6b, 0x9a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 100",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0xb2, 0x64, 0xe9, 0x21, 0xe9, 0xe4, 0xa8,
			0x26, 0x94, 0x58, 0x9e, 0xf3, 0x79, 0x82, 0x62,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 101",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8d, 0x45, 0x95, 0xcb, 0x4f, 0xa7, 0x02, 0x67,
			0x15, 0xf5, 0x5b, 0xd6, 0x8e, 0x28, 0x82, 0xf9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 102",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb5, 0x88, 0xa3, 0x02, 0xbd, 0xbc, 0x09, 0x19,
			0x7d, 0xf1, 0xed, 0xae, 0x68, 0x92, 0x6e, 0xd9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 103",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x33, 0xf7, 0x50, 0x23, 0x90, 0xb8, 0xa4, 0xa2,
			0x21, 0xcf, 0xec, 0xd0, 0x66, 0x66, 0x24, 0xba,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 104",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3d, 0x20, 0x25, 0x3a, 0xdb, 0xce, 0x3b, 0xe2,
			0x37, 0x37, 0x67, 0xc4, 0xd8, 0x22, 0xc5, 0x66,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 105",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa4, 0x27, 0x34, 0xa3, 0x92, 0x9b, 0xf8, 0x4c,
			0xf0, 0x11, 0x6c, 0x98, 0x56, 0xa3, 0xc1, 0x8c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 106",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe3, 0xab, 0xc4, 0x93, 0x94, 0x57, 0x42, 0x2b,
			0xb9, 0x57, 0xda, 0x3c, 0x56, 0x93, 0x8c, 0x6d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 107",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0x2b, 0xdd, 0x2e, 0x7c, 0x52, 0x51, 0x30,
			0xfa, 0xdc, 0x8f, 0x76, 0xfc, 0x6f, 0x4b, 0x3f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 108",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x84, 0xa8, 0x3d, 0x7b, 0x94, 0xc6, 0x99, 0xcb,
			0xcb, 0x8a, 0x7d, 0x9b, 0x61, 0xf6, 0x40, 0x93,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 109",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xce, 0x61, 0xd6, 0x35, 0x14, 0xad, 0xed, 0x03,
			0xd4, 0x3e, 0x6e, 0xbf, 0xc3, 0xa9, 0x00, 0x1f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 110",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6c, 0x83, 0x9d, 0xd5, 0x8e, 0xea, 0xe6, 0xb8,
			0xa3, 0x6a, 0xf4, 0x8e, 0xd6, 0x3d, 0x2d, 0xc9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 111",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcd, 0x5e, 0xce, 0x55, 0xb8, 0xda, 0x3b, 0xf6,
			0x22, 0xc4, 0x10, 0x0d, 0xf5, 0xde, 0x46, 0xf9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 112",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3b, 0x6f, 0x46, 0xf4, 0x0e, 0x0a, 0xc5, 0xfc,
			0x0a, 0x9c, 0x11, 0x05, 0xf8, 0x00, 0xf4, 0x8d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 113",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xba, 0x26, 0xd4, 0x7d, 0xa3, 0xae, 0xb0, 0x28,
			0xde, 0x4f, 0xb5, 0xb3, 0xa8, 0x54, 0xa2, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 114",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x87, 0xf5, 0x3b, 0xf6, 0x20, 0xd3, 0x67, 0x72,
			0x68, 0x44, 0x52, 0x12, 0x90, 0x43, 0x89, 0xd5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 115",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x10, 0x61, 0x7d, 0x28, 0xb5, 0xe0, 0xf4, 0x60,
			0x54, 0x92, 0xb1, 0x82, 0xa5, 0xd7, 0xf9, 0xf6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 116",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9a, 0xae, 0xc4, 0xfa, 0xbb, 0xf6, 0xfa, 0xe2,
			0xa7, 0x1f, 0xef, 0xf0, 0x2e, 0x37, 0x2b, 0x39,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 117",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0x90, 0xc6, 0x2d, 0x88, 0xb5, 0xc4, 0x28,
			0x09, 0xab, 0xf7, 0x82, 0x48, 0x8e, 0xd1, 0x30,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 118",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf1, 0xf1, 0xc5, 0xa4, 0x08, 0x99, 0xe1, 0x57,
			0x72, 0x85, 0x7c, 0xcb, 0x65, 0xc7, 0xa0, 0x9a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 119",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0x08, 0x43, 0xd2, 0x9b, 0x25, 0xa3, 0x89,
			0x7c, 0x69, 0x2c, 0xe1, 0xdd, 0x81, 0xee, 0x52,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 120",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0x66, 0xbc, 0x65, 0xb6, 0x94, 0x1d, 0x86,
			0xe8, 0x42, 0x0a, 0x7f, 0xfb, 0x09, 0x64, 0xdb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 121",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x81, 0x93, 0xc6, 0xff, 0x85, 0x22, 0x5c, 0xed,
			0x42, 0x55, 0xe9, 0x2f, 0x6e, 0x07, 0x8a, 0x14,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 122",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0x61, 0xcb, 0x24, 0x24, 0xd7, 0xd4, 0xa3,
			0x80, 0xd5, 0x47, 0xf9, 0xe7, 0xec, 0x1c, 0xb9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 123",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x86, 0xf9, 0x3d, 0x9e, 0xc0, 0x84, 0x53, 0xa0,
			0x71, 0xe2, 0xe2, 0x87, 0x78, 0x77, 0xa9, 0xc8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 124",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0xee, 0xfa, 0x80, 0xce, 0x6a, 0x4a, 0x9d,
			0x59, 0x8e, 0x3f, 0xec, 0x36, 0x54, 0x34, 0xd2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 125",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd6, 0x20, 0x68, 0x44, 0x45, 0x78, 0xe3, 0xab,
			0x39, 0xce, 0x7e, 0xc9, 0x5d, 0xd0, 0x45, 0xdc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 126",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb5, 0xf7, 0x1d, 0x4d, 0xd9, 0xa7, 0x1f, 0xe5,
			0xd8, 0xbc, 0x8b, 0xa7, 0xe6, 0xea, 0x30, 0x48,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 127",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x68, 0x25, 0xa3, 0x47, 0xac, 0x47, 0x9d, 0x4f,
			0x9d, 0x95, 0xc5, 0xcb, 0x8d, 0x3f, 0xd7, 0xe9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 128",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe3, 0x71, 0x4e, 0x94, 0xa5, 0x77, 0x89, 0x55,
			0xcc, 0x03, 0x46, 0x35, 0x8e, 0x94, 0x78, 0x3a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 129",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd8, 0x36, 0xb4, 0x4b, 0xb2, 0x9e, 0x0c, 0x7d,
			0x89, 0xfa, 0x4b, 0x2d, 0x4b, 0x67, 0x7d, 0x2a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 130",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5d, 0x45, 0x4b, 0x75, 0x02, 0x1d, 0x76, 0xd4,
			0xb8, 0x4f, 0x87, 0x3a, 0x8f, 0x87, 0x7b, 0x92,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 131",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc3, 0x49, 0x8f, 0x7e, 0xce, 0xd2, 0x09, 0x53,
			0x14, 0xfc, 0x28, 0x11, 0x58, 0x85, 0xb3, 0x3f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 132",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6e, 0x66, 0x88, 0x56, 0x53, 0x9a, 0xd8, 0xe4,
			0x05, 0xbd, 0x12, 0x3f, 0xe6, 0xc8, 0x85, 0x30,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 133",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x86, 0x80, 0xdb, 0x7f, 0x3a, 0x87, 0xb8, 0x60,
			0x55, 0x43, 0xcf, 0xdb, 0xe6, 0x75, 0x40, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 134",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6c, 0x5d, 0x03, 0xb1, 0x30, 0x69, 0xc3, 0x65,
			0x8b, 0x31, 0x79, 0xbe, 0x91, 0xb0, 0x80, 0x0c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 135",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xef, 0x1b, 0x38, 0x4a, 0xc4, 0xd9, 0x3e, 0xda,
			0x00, 0xc9, 0x2a, 0xdd, 0x09, 0x95, 0xea, 0x5f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 136",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbf, 0x81, 0x15, 0x80, 0x54, 0x71, 0x74, 0x1b,
			0xd5, 0xad, 0x20, 0xa0, 0x39, 0x44, 0x79, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 137",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc6, 0x4c, 0x24, 0xb6, 0x89, 0x4b, 0x03, 0x8b,
			0x3c, 0x0d, 0x09, 0xb1, 0xdf, 0x06, 0x8b, 0x0b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 138",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x39, 0x67, 0xa1, 0x0c, 0xff, 0xe2, 0x7d, 0x01,
			0x78, 0x54, 0x5f, 0xbf, 0x6a, 0x40, 0x54, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 139",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7c, 0x85, 0xe9, 0xc9, 0x5d, 0xe1, 0xa9, 0xec,
			0x5a, 0x53, 0x63, 0xa8, 0xa0, 0x53, 0x47, 0x2d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 140",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa9, 0xee, 0xc0, 0x3c, 0x8a, 0xbe, 0xc7, 0xba,
			0x68, 0x31, 0x5c, 0x2c, 0x8c, 0x23, 0x16, 0xe0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 141",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xca, 0xc8, 0xe4, 0x14, 0xc2, 0xf3, 0x88, 0x22,
			0x7a, 0xe1, 0x49, 0x86, 0xfc, 0x98, 0x35, 0x24,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 142",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5d, 0x94, 0x2b, 0x7f, 0x46, 0x22, 0xce, 0x05,
			0x6c, 0x3c, 0xe3, 0xce, 0x5f, 0x1d, 0xd9, 0xd6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 143",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0x40, 0xd6, 0x48, 0xce, 0x21, 0xa3, 0x02,
			0x02, 0x82, 0xc3, 0xf1, 0xb5, 0x28, 0xa0, 0xb6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 144",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x45, 0xd0, 0x89, 0xc3, 0x6d, 0x5c, 0x5a, 0x4e,
			0xfc, 0x68, 0x9e, 0x3b, 0x0d, 0xe1, 0x0d, 0xd5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 145",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb4, 0xda, 0x5d, 0xf4, 0xbe, 0xcb, 0x54, 0x62,
			0xe0, 0x3a, 0x0e, 0xd0, 0x0d, 0x29, 0x56, 0x29,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 146",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdc, 0xf4, 0xe1, 0x29, 0x13, 0x6c, 0x1a, 0x4b,
			0x7a, 0x0f, 0x38, 0x93, 0x5c, 0xc3, 0x4b, 0x2b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 147",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd9, 0xa4, 0xc7, 0x61, 0x8b, 0x0c, 0xe4, 0x8a,
			0x3d, 0x5a, 0xee, 0x1a, 0x1c, 0x01, 0x14, 0xc4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 148",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xca, 0x35, 0x2d, 0xf0, 0x25, 0xc6, 0x5c, 0x7b,
			0x0b, 0xf3, 0x06, 0xfb, 0xee, 0x0f, 0x36, 0xba,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 149",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x23, 0x8a, 0xca, 0x23, 0xfd, 0x34, 0x09, 0xf3,
			0x8a, 0xf6, 0x33, 0x78, 0xed, 0x2f, 0x54, 0x73,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 150",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x59, 0x83, 0x6a, 0x0e, 0x06, 0xa7, 0x96, 0x91,
			0xb3, 0x66, 0x67, 0xd5, 0x38, 0x0d, 0x81, 0x88,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 151",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x33, 0x90, 0x50, 0x80, 0xf7, 0xac, 0xf1, 0xcd,
			0xae, 0x0a, 0x91, 0xfc, 0x3e, 0x85, 0xae, 0xe4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 152",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x72, 0xc9, 0xe4, 0x64, 0x6d, 0xbc, 0x3d, 0x63,
			0x20, 0xfc, 0x66, 0x89, 0xd9, 0x3e, 0x88, 0x33,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 153",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xba, 0x77, 0x41, 0x3d, 0xea, 0x59, 0x25, 0xb7,
			0xf5, 0x41, 0x7e, 0xa4, 0x7f, 0xf1, 0x9f, 0x59,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 154",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6c, 0xae, 0x81, 0x29, 0xf8, 0x43, 0xd8, 0x6d,
			0xc7, 0x86, 0xa0, 0xfb, 0x1a, 0x18, 0x49, 0x70,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 155",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfc, 0xfe, 0xfb, 0x53, 0x41, 0x00, 0x79, 0x6e,
			0xeb, 0xbd, 0x99, 0x02, 0x06, 0x75, 0x4e, 0x19,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 156",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8c, 0x79, 0x1d, 0x5f, 0xdd, 0xdf, 0x47, 0x0d,
			0xa0, 0x4f, 0x3e, 0x6d, 0xc4, 0xa5, 0xb5, 0xb5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 157",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc9, 0x3b, 0xbd, 0xc0, 0x7a, 0x46, 0x11, 0xae,
			0x4b, 0xb2, 0x66, 0xea, 0x50, 0x34, 0xa3, 0x87,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 158",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc1, 0x02, 0xe3, 0x8e, 0x48, 0x9a, 0xa7, 0x47,
			0x62, 0xf3, 0xef, 0xc5, 0xbb, 0x23, 0x20, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 159",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x93, 0x20, 0x14, 0x81, 0x66, 0x5c, 0xba, 0xfc,
			0x1f, 0xcc, 0x22, 0x0b, 0xc5, 0x45, 0xfb, 0x3d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 160",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x49, 0x60, 0x75, 0x7e, 0xc6, 0xce, 0x68, 0xcf,
			0x19, 0x5e, 0x45, 0x4c, 0xfd, 0x0f, 0x32, 0xca,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 161",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfe, 0xec, 0x7c, 0xe6, 0xa6, 0xcb, 0xd0, 0x7c,
			0x04, 0x34, 0x16, 0x73, 0x7f, 0x1b, 0xbb, 0x33,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 162",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0xc5, 0x41, 0x39, 0x04, 0x48, 0x7a, 0x80,
			0x5d, 0x70, 0xa8, 0xed, 0xd9, 0xc3, 0x55, 0x27,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 163",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x34, 0x78, 0x46, 0xb2, 0xb2, 0xe3, 0x6f, 0x1f,
			0x03, 0x24, 0xc8, 0x6f, 0x7f, 0x1b, 0x98, 0xe2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 164",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x33, 0x2e, 0xee, 0x1a, 0x0c, 0xbd, 0x19, 0xca,
			0x2d, 0x69, 0xb4, 0x26, 0x89, 0x40, 0x44, 0xf0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 165",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x86, 0x6b, 0x5b, 0x39, 0x77, 0xba, 0x6e, 0xfa,
			0x51, 0x28, 0xef, 0xbd, 0xa9, 0xff, 0x03, 0xcd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 166",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0x14, 0x45, 0xee, 0x94, 0xc0, 0xf0, 0x8c,
			0xde, 0xe5, 0xc3, 0x44, 0xec, 0xd1, 0xe2, 0x33,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 167",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbe, 0x28, 0x83, 0x19, 0x02, 0x93, 0x63, 0xc2,
			0x62, 0x2f, 0xeb, 0xa4, 0xb0, 0x5d, 0xfd, 0xfe,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 168",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcf, 0xd1, 0x87, 0x55, 0x23, 0xf3, 0xcd, 0x21,
			0xc3, 0x95, 0x65, 0x1e, 0x6e, 0xe1, 0x5e, 0x56,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 169",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcb, 0x5a, 0x40, 0x86, 0x57, 0x83, 0x7c, 0x53,
			0xbf, 0x16, 0xf9, 0xd8, 0x46, 0x5d, 0xce, 0x19,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 170",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xca, 0x0b, 0xf4, 0x2c, 0xb1, 0x07, 0xf5, 0x5c,
			0xcf, 0xf2, 0xfc, 0x09, 0xee, 0x08, 0xca, 0x15,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 171",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfd, 0xd9, 0xbb, 0xb4, 0xa7, 0xdc, 0x2e, 0x4a,
			0x23, 0x53, 0x6a, 0x58, 0x80, 0xa2, 0xdb, 0x67,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 172",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xed, 0xe4, 0x47, 0xb3, 0x62, 0xc4, 0x84, 0x99,
			0x3d, 0xec, 0x94, 0x42, 0xa3, 0xb4, 0x6a, 0xef,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 173",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x10, 0xdf, 0xfb, 0x05, 0x90, 0x4b, 0xff, 0x7c,
			0x47, 0x81, 0xdf, 0x78, 0x0a, 0xd2, 0x68, 0x37,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 174",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc3, 0x3b, 0xc1, 0x3e, 0x8d, 0xe8, 0x8a, 0xc2,
			0x52, 0x32, 0xaa, 0x74, 0x96, 0x39, 0x87, 0x83,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 175",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xca, 0x35, 0x9c, 0x70, 0x80, 0x3a, 0x3b, 0x2a,
			0x3d, 0x54, 0x2e, 0x87, 0x81, 0xde, 0xa9, 0x75,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 176",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbc, 0xc6, 0x5b, 0x52, 0x6f, 0x88, 0xd0, 0x5b,
			0x89, 0xce, 0x8a, 0x52, 0x02, 0x1f, 0xdb, 0x06,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 177",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdb, 0x91, 0xa3, 0x88, 0x55, 0xc8, 0xc4, 0x64,
			0x38, 0x51, 0xfb, 0xfb, 0x35, 0x8b, 0x01, 0x09,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 178",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xca, 0x6e, 0x88, 0x93, 0xa1, 0x14, 0xae, 0x8e,
			0x27, 0xd5, 0xab, 0x03, 0xa5, 0x49, 0x96, 0x10,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 179",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x66, 0x29, 0xd2, 0xb8, 0xdf, 0x97, 0xda, 0x72,
			0x8c, 0xdd, 0x8b, 0x1e, 0x7f, 0x94, 0x50, 0x77,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 180",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x45, 0x70, 0xa5, 0xa1, 0x8c, 0xfc, 0x0d, 0xd5,
			0x82, 0xf1, 0xd8, 0x8d, 0x5c, 0x9a, 0x17, 0x20,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 181",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x72, 0xbc, 0x65, 0xaa, 0x8e, 0x89, 0x56, 0x2e,
			0x3f, 0x27, 0x4d, 0x45, 0xaf, 0x1c, 0xd1, 0x0b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 182",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x98, 0x55, 0x1d, 0xa1, 0xa6, 0x50, 0x32, 0x76,
			0xae, 0x1c, 0x77, 0x62, 0x5f, 0x9e, 0xa6, 0x15,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 183",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0d, 0xdf, 0xe5, 0x1c, 0xed, 0x7e, 0x3f, 0x4a,
			0xe9, 0x27, 0xda, 0xa3, 0xfe, 0x45, 0x2c, 0xee,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 184",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdb, 0x82, 0x62, 0x51, 0xe4, 0xce, 0x38, 0x4b,
			0x80, 0x21, 0x8b, 0x0e, 0x1d, 0xa1, 0xdd, 0x4c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 185",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2c, 0xac, 0xf7, 0x28, 0xb8, 0x8a, 0xbb, 0xad,
			0x70, 0x11, 0xed, 0x0e, 0x64, 0xa1, 0x68, 0x0c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 186",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x33, 0x0d, 0x8e, 0xe7, 0xc5, 0x67, 0x7e, 0x09,
			0x9a, 0xc7, 0x4c, 0x99, 0x94, 0xee, 0x4c, 0xfb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 187",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xed, 0xf6, 0x1a, 0xe3, 0x62, 0xe8, 0x82, 0xdd,
			0xc0, 0x16, 0x74, 0x74, 0xa7, 0xa7, 0x7f, 0x3a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 188",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x61, 0x68, 0xb0, 0x0b, 0xa7, 0x85, 0x9e, 0x09,
			0x70, 0xec, 0xfd, 0x75, 0x7e, 0xfe, 0xcf, 0x7c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 189",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd1, 0x41, 0x54, 0x47, 0x86, 0x62, 0x30, 0xd2,
			0x8b, 0xb1, 0xea, 0x18, 0xa4, 0xcd, 0xfd, 0x02,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 190",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x51, 0x61, 0x83, 0x39, 0x2f, 0x7a, 0x87, 0x63,
			0xaf, 0xec, 0x68, 0xa0, 0x60, 0x26, 0x41, 0x41,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 191",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0x56, 0x5c, 0x8d, 0x73, 0xcf, 0xd4, 0x13,
			0x0b, 0x4a, 0xa1, 0x4d, 0x89, 0x11, 0x71, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 192",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x37, 0x23, 0x2a, 0x4e, 0xd2, 0x1c, 0xcc, 0x27,
			0xc1, 0x9c, 0x96, 0x10, 0x07, 0x8c, 0xab, 0xac,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 193",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x80, 0x4f, 0x32, 0xea, 0x71, 0x82, 0x8c, 0x7d,
			0x32, 0x90, 0x77, 0xe7, 0x12, 0x23, 0x16, 0x66,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 194",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd6, 0x44, 0x24, 0xf2, 0x3c, 0xb9, 0x72, 0x15,
			0xe9, 0xc2, 0xc6, 0xf2, 0x8d, 0x29, 0xea, 0xb7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 195",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x02, 0x3e, 0x82, 0xb5, 0x33, 0xf6, 0x8c, 0x75,
			0xc2, 0x38, 0xce, 0xbd, 0xb2, 0xee, 0x89, 0xa2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 196",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0x3a, 0x3d, 0x24, 0x15, 0x7a, 0x51, 0xf1,
			0xee, 0x08, 0x93, 0xf6, 0x77, 0x74, 0x17, 0xe7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 197",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x84, 0xec, 0xac, 0xfc, 0xd4, 0x00, 0x08, 0x4d,
			0x07, 0x86, 0x12, 0xb1, 0x94, 0x5f, 0x2e, 0xf5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 198",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1d, 0xcd, 0x8b, 0xb1, 0x73, 0x25, 0x9e, 0xb3,
			0x3a, 0x52, 0x42, 0xb0, 0xde, 0x31, 0xa4, 0x55,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 199",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x35, 0xe9, 0xed, 0xdb, 0xc3, 0x75, 0xe7, 0x92,
			0xc1, 0x99, 0x92, 0xc1, 0x91, 0x65, 0x01, 0x2b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 200",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8a, 0x77, 0x22, 0x31, 0xc0, 0x1d, 0xfd, 0xd7,
			0xc9, 0x8e, 0x4c, 0xfd, 0xdc, 0xc0, 0x80, 0x7a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 201",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6e, 0xda, 0x7f, 0xf6, 0xb8, 0x31, 0x91, 0x80,
			0xff, 0x0d, 0x6e, 0x65, 0x62, 0x9d, 0x01, 0xc3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 202",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc2, 0x67, 0xef, 0x0e, 0x2d, 0x01, 0xa9, 0x93,
			0x94, 0x4d, 0xd3, 0x97, 0x10, 0x14, 0x13, 0xcb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 203",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe9, 0xf8, 0x0e, 0x9d, 0x84, 0x5b, 0xcc, 0x0f,
			0x62, 0x92, 0x6a, 0xf7, 0x2e, 0xab, 0xca, 0x39,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 204",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x67, 0x02, 0x99, 0x07, 0x27, 0xaa, 0x08, 0x78,
			0x63, 0x7b, 0x45, 0xdc, 0xd3, 0xa3, 0xb0, 0x74,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 205",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2e, 0x2e, 0x64, 0x7d, 0x53, 0x60, 0xe0, 0x92,
			0x30, 0xa5, 0xd7, 0x38, 0xca, 0x33, 0x47, 0x1e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 206",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1f, 0x56, 0x41, 0x3c, 0x7a, 0xdd, 0x6f, 0x43,
			0xd1, 0xd5, 0x6e, 0x4f, 0x02, 0x19, 0x03, 0x30,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 207",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x69, 0xcd, 0x06, 0x06, 0xe1, 0x5a, 0xf7, 0x29,
			0xd6, 0xbc, 0xa1, 0x43, 0x01, 0x6d, 0x98, 0x42,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 208",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa0, 0x85, 0xd7, 0xc1, 0xa5, 0x00, 0x87, 0x3a,
			0x20, 0x09, 0x9c, 0x4c, 0xaa, 0x3c, 0x3f, 0x5b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 209",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4f, 0xc0, 0xd2, 0x30, 0xf8, 0x89, 0x14, 0x15,
			0xb8, 0x7b, 0x83, 0xf9, 0x5f, 0x2e, 0x09, 0xd1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 210",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x43, 0x27, 0xd0, 0x8c, 0x52, 0x3d, 0x8e, 0xba,
			0x69, 0x7a, 0x43, 0x36, 0x50, 0x7d, 0x1f, 0x42,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 211",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7a, 0x15, 0xaa, 0xb8, 0x27, 0x01, 0xef, 0xa5,
			0xae, 0x36, 0xab, 0x1d, 0x6b, 0x76, 0x29, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 212",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5b, 0xf0, 0x05, 0x18, 0x93, 0xa1, 0x8b, 0xb3,
			0x0e, 0x13, 0x9a, 0x58, 0xfe, 0xd0, 0xfa, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 213",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x97, 0xe8, 0xad, 0xf6, 0x56, 0x38, 0xfd, 0x9c,
			0xdf, 0x3b, 0xc2, 0x2c, 0x17, 0xfe, 0x4d, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 214",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1e, 0xe6, 0xee, 0x32, 0x65, 0x83, 0xa0, 0x58,
			0x64, 0x91, 0xc9, 0x64, 0x18, 0xd1, 0xa3, 0x5d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 215",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x26, 0xb5, 0x49, 0xc2, 0xec, 0x75, 0x6f, 0x82,
			0xec, 0xc4, 0x80, 0x08, 0xe5, 0x29, 0x95, 0x6b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 216",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0x37, 0x7b, 0x6d, 0xa6, 0x69, 0xb0, 0x72,
			0x12, 0x9e, 0x05, 0x7c, 0xc2, 0x8e, 0x9c, 0xa5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 217",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9c, 0x94, 0xb8, 0xb0, 0xcb, 0x8b, 0xcc, 0x91,
			0x90, 0x72, 0x26, 0x2b, 0x3f, 0xa0, 0x5a, 0xd9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 218",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2f, 0xbb, 0x83, 0xdf, 0xd0, 0xd7, 0xab, 0xcb,
			0x05, 0xcd, 0x28, 0xca, 0xd2, 0xdf, 0xb5, 0x23,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 219",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0x87, 0x78, 0x03, 0xde, 0x77, 0x74, 0x4b,
			0xb9, 0x70, 0xd0, 0xa9, 0x1f, 0x4d, 0xeb, 0xae,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 220",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x73, 0x79, 0xf3, 0x37, 0x0c, 0xf6, 0xe5, 0xce,
			0x12, 0xae, 0x59, 0x69, 0xc8, 0xee, 0xa3, 0x12,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 221",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x02, 0xdc, 0x99, 0xfa, 0x3d, 0x4f, 0x98, 0xce,
			0x80, 0x98, 0x5e, 0x72, 0x33, 0x88, 0x93, 0x13,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 222",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1e, 0x38, 0xe7, 0x59, 0x07, 0x5b, 0xa5, 0xca,
			0xb6, 0x45, 0x7d, 0xa5, 0x18, 0x44, 0x29, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 223",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0xbe, 0xd8, 0xdb, 0xf6, 0x15, 0x86, 0x8a,
			0x1f, 0x9d, 0x9b, 0x05, 0xd3, 0xe7, 0xa2, 0x67,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 224",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x23, 0x4b, 0x14, 0x8b, 0x8c, 0xb1, 0xd8, 0xc3,
			0x2b, 0x28, 0x7e, 0x89, 0x69, 0x03, 0xd1, 0x50,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 225",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x29, 0x4b, 0x03, 0x3d, 0xf4, 0xda, 0x85, 0x3f,
			0x4b, 0xe3, 0xe2, 0x43, 0xf7, 0xe5, 0x13, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 226",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3f, 0x58, 0xc9, 0x50, 0xf0, 0x36, 0x71, 0x60,
			0xad, 0xec, 0x45, 0xf2, 0x44, 0x1e, 0x74, 0x11,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 227",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x37, 0xf6, 0x55, 0x53, 0x6a, 0x70, 0x4e, 0x5a,
			0xce, 0x18, 0x2d, 0x74, 0x2a, 0x82, 0x0c, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 228",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xea, 0x7b, 0xd6, 0xbb, 0x63, 0x41, 0x87, 0x31,
			0xae, 0xac, 0x79, 0x0f, 0xe4, 0x2d, 0x61, 0xe8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 229",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe7, 0x4a, 0x4c, 0x99, 0x9b, 0x4c, 0x06, 0x4e,
			0x48, 0xbb, 0x1e, 0x41, 0x3f, 0x51, 0xe5, 0xea,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 230",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xba, 0x9e, 0xbe, 0xfd, 0xb4, 0xcc, 0xf3, 0x0f,
			0x29, 0x6c, 0xec, 0xb3, 0xbc, 0x19, 0x43, 0xe8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 231",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x31, 0x94, 0x36, 0x7a, 0x48, 0x98, 0xc5, 0x02,
			0xc1, 0x3b, 0xb7, 0x47, 0x86, 0x40, 0xa7, 0x2d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 232",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xda, 0x79, 0x77, 0x13, 0x26, 0x3d, 0x6f, 0x33,
			0xa5, 0x47, 0x8a, 0x65, 0xef, 0x60, 0xd4, 0x12,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 233",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd1, 0xac, 0x39, 0xbb, 0x1e, 0xf8, 0x6b, 0x9c,
			0x13, 0x44, 0xf2, 0x14, 0x67, 0x9a, 0xa3, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 234",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2f, 0xde, 0xa9, 0xe6, 0x50, 0x53, 0x2b, 0xe5,
			0xbc, 0x0e, 0x73, 0x25, 0x33, 0x7f, 0xd3, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 235",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd3, 0xa2, 0x04, 0xdb, 0xd9, 0xc2, 0xaf, 0x15,
			0x8b, 0x6c, 0xa6, 0x7a, 0x51, 0x56, 0xce, 0x4a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 236",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0x0a, 0x0e, 0x75, 0xa8, 0xda, 0x36, 0x73,
			0x5a, 0xee, 0x66, 0x84, 0xd9, 0x65, 0xa7, 0x78,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 237",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x52, 0xfc, 0x3e, 0x62, 0x04, 0x92, 0xea, 0x99,
			0x64, 0x1e, 0xa1, 0x68, 0xda, 0x5b, 0x6d, 0x52,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 238",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0xe0, 0xc7, 0xf1, 0x5b, 0x47, 0x72, 0x46,
			0x7d, 0x2c, 0xfc, 0x87, 0x30, 0x00, 0xb2, 0xca,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 239",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0x35, 0x31, 0x13, 0x5e, 0x0c, 0x4d, 0x70,
			0xa3, 0x8f, 0x8b, 0xdb, 0x19, 0x0b, 0xa0, 0x4e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 240",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0xa3, 0x9a, 0x0f, 0x56, 0x63, 0xf4, 0xc0,
			0xfe, 0x5f, 0x2d, 0x3c, 0xaf, 0xff, 0x42, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 241",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd9, 0x4b, 0x5e, 0x90, 0xdb, 0x35, 0x4c, 0x1e,
			0x42, 0xf6, 0x1f, 0xab, 0xe1, 0x67, 0xb2, 0xc0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 242",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x50, 0xe6, 0xd3, 0xc9, 0xb6, 0x69, 0x8a, 0x7c,
			0xd2, 0x76, 0xf9, 0x6b, 0x14, 0x73, 0xf3, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 243",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x93, 0x38, 0xf0, 0x8e, 0x0e, 0xbe, 0xe9, 0x69,
			0x05, 0xd8, 0xf2, 0xe8, 0x25, 0x20, 0x8f, 0x43,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 244",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8b, 0x37, 0x8c, 0x86, 0x67, 0x2a, 0xa5, 0x4a,
			0x3a, 0x26, 0x6b, 0xa1, 0x9d, 0x25, 0x80, 0xca,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 245",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0xa7, 0xc3, 0x08, 0x6f, 0x5f, 0x95, 0x11,
			0xb3, 0x12, 0x33, 0xda, 0x7c, 0xab, 0x91, 0x60,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 246",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x5b, 0x40, 0xff, 0x4e, 0xc9, 0xbe, 0x53, 0x6b,
			0xa2, 0x30, 0x35, 0xfa, 0x4f, 0x06, 0x06, 0x4c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 247",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x60, 0xeb, 0x5a, 0xf8, 0x41, 0x6b, 0x25, 0x71,
			0x49, 0x37, 0x21, 0x94, 0xe8, 0xb8, 0x87, 0x49,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 248",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2f, 0x00, 0x5a, 0x8a, 0xed, 0x8a, 0x36, 0x1c,
			0x92, 0xe4, 0x40, 0xc1, 0x55, 0x20, 0xcb, 0xd1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 249",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7b, 0x03, 0x62, 0x76, 0x11, 0x67, 0x8a, 0x99,
			0x77, 0x17, 0x57, 0x88, 0x07, 0xa8, 0x00, 0xe2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 250",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcf, 0x78, 0x61, 0x8f, 0x74, 0xf6, 0xf3, 0x69,
			0x6e, 0x0a, 0x47, 0x79, 0xb9, 0x0b, 0x5a, 0x77,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 251",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x03, 0x72, 0x03, 0x71, 0xa0, 0x49, 0x62, 0xea,
			0xea, 0x0a, 0x85, 0x2e, 0x69, 0x97, 0x28, 0x58,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 252",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1f, 0x8a, 0x81, 0x33, 0xaa, 0x8c, 0xcf, 0x70,
			0xe2, 0xbd, 0x32, 0x85, 0x83, 0x1c, 0xa6, 0xb7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 253",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0x93, 0x6b, 0xd2, 0x7f, 0xb1, 0x46, 0x8f,
			0xc8, 0xb4, 0x8b, 0xc4, 0x83, 0x32, 0x17, 0x25,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 254",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb0, 0x7d, 0x4f, 0x3e, 0x2c, 0xd2, 0xef, 0x2e,
			0xb5, 0x45, 0x98, 0x07, 0x54, 0xdf, 0xea, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarKey256 255",
		.keylen	 = 256,
		.key	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		},
		.ptext	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4b, 0xf8, 0x5f, 0x1b, 0x5d, 0x54, 0xad, 0xbc,
			0x30, 0x7b, 0x0a, 0x04, 0x83, 0x89, 0xad, 0xcb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   0",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0xd7, 0x8e, 0x72, 0x6c, 0x1e, 0xc0, 0x2b,
			0x7e, 0xbf, 0xe9, 0x2b, 0x23, 0xd9, 0xec, 0x34,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   1",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaa, 0xe5, 0x93, 0x9c, 0x8e, 0xfd, 0xf2, 0xf0,
			0x4e, 0x60, 0xb9, 0xfe, 0x71, 0x17, 0xb2, 0xc2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   2",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf0, 0x31, 0xd4, 0xd7, 0x4f, 0x5d, 0xcb, 0xf3,
			0x9d, 0xaa, 0xf8, 0xca, 0x3a, 0xf6, 0xe5, 0x27,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   3",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0xd9, 0xfd, 0x5c, 0xc4, 0xf0, 0x74, 0x41,
			0x72, 0x7d, 0xf0, 0xf3, 0x3e, 0x40, 0x1a, 0x36,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   4",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x30, 0xcc, 0xdb, 0x04, 0x46, 0x46, 0xd7, 0xe1,
			0xf3, 0xcc, 0xea, 0x3d, 0xca, 0x08, 0xb8, 0xc0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   5",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x16, 0xae, 0x4c, 0xe5, 0x04, 0x2a, 0x67, 0xee,
			0x8e, 0x17, 0x7b, 0x7c, 0x58, 0x7e, 0xcc, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   6",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb6, 0xda, 0x0b, 0xb1, 0x1a, 0x23, 0x85, 0x5d,
			0x9c, 0x5c, 0xb1, 0xb4, 0xc6, 0x41, 0x2e, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   7",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdb, 0x4f, 0x1a, 0xa5, 0x30, 0x96, 0x7d, 0x67,
			0x32, 0xce, 0x47, 0x15, 0xeb, 0x0e, 0xe2, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   8",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0x17, 0x38, 0x25, 0x26, 0x21, 0xdd, 0x18,
			0x0a, 0x34, 0xf3, 0x45, 0x5b, 0x4b, 0xaa, 0x2f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128   9",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0xe2, 0xb5, 0x08, 0xdb, 0x7f, 0xd8, 0x92,
			0x34, 0xca, 0xf7, 0x93, 0x9e, 0xe5, 0x62, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  10",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb8, 0x49, 0x9c, 0x25, 0x1f, 0x84, 0x42, 0xee,
			0x13, 0xf0, 0x93, 0x3b, 0x68, 0x8f, 0xcd, 0x19,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  11",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0x51, 0x35, 0xf8, 0xa8, 0x1f, 0x25, 0xc9,
			0xd6, 0x30, 0xb1, 0x75, 0x02, 0xf6, 0x8e, 0x53,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  12",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8b, 0x87, 0x14, 0x5a, 0x01, 0xad, 0x1c, 0x6c,
			0xed, 0xe9, 0x95, 0xea, 0x36, 0x70, 0x45, 0x4f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  13",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8e, 0xae, 0x3b, 0x10, 0xa0, 0xc8, 0xca, 0x6d,
			0x1d, 0x3b, 0x0f, 0xa6, 0x1e, 0x56, 0xb0, 0xb2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  14",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x64, 0xb4, 0xd6, 0x29, 0x81, 0x0f, 0xda, 0x6b,
			0xaf, 0xdf, 0x08, 0xf3, 0xb0, 0xd8, 0xd2, 0xc5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  15",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd7, 0xe5, 0xdb, 0xd3, 0x32, 0x45, 0x95, 0xf8,
			0xfd, 0xc7, 0xd7, 0xc5, 0x71, 0xda, 0x6c, 0x2a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  16",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf3, 0xf7, 0x23, 0x75, 0x26, 0x4e, 0x16, 0x7f,
			0xca, 0x9d, 0xe2, 0xc1, 0x52, 0x7d, 0x96, 0x06,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  17",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8e, 0xe7, 0x9d, 0xd4, 0xf4, 0x01, 0xff, 0x9b,
			0x7e, 0xa9, 0x45, 0xd8, 0x66, 0x66, 0xc1, 0x3b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  18",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdd, 0x35, 0xce, 0xa2, 0x79, 0x99, 0x40, 0xb4,
			0x0d, 0xb3, 0xf8, 0x19, 0xcb, 0x94, 0xc0, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  19",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x69, 0x41, 0xcb, 0x6b, 0x3e, 0x08, 0xc2, 0xb7,
			0xaf, 0xa5, 0x81, 0xeb, 0xdd, 0x60, 0x7b, 0x87,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  20",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2c, 0x20, 0xf4, 0x39, 0xf6, 0xbb, 0x09, 0x7b,
			0x29, 0xb8, 0xbd, 0x6d, 0x99, 0xaa, 0xd7, 0x99,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  21",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x62, 0x5d, 0x01, 0xf0, 0x58, 0xe5, 0x65, 0xf7,
			0x7a, 0xe8, 0x63, 0x78, 0xbd, 0x2c, 0x49, 0xb3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  22",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc0, 0xb5, 0xfd, 0x98, 0x19, 0x0e, 0xf4, 0x5f,
			0xbb, 0x43, 0x01, 0x43, 0x8d, 0x09, 0x59, 0x50,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  23",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x13, 0x00, 0x1f, 0xf5, 0xd9, 0x98, 0x06, 0xef,
			0xd2, 0x5d, 0xa3, 0x4f, 0x56, 0xbe, 0x85, 0x4b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  24",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3b, 0x59, 0x4c, 0x60, 0xf5, 0xc8, 0x27, 0x7a,
			0x51, 0x13, 0x67, 0x7f, 0x94, 0x20, 0x8d, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  25",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe9, 0xc0, 0xfc, 0x18, 0x18, 0xe4, 0xaa, 0x46,
			0xbd, 0x2e, 0x39, 0xd6, 0x38, 0xf8, 0x9e, 0x05,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  26",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf8, 0x02, 0x3e, 0xe9, 0xc3, 0xfd, 0xc4, 0x5a,
			0x01, 0x9b, 0x4e, 0x98, 0x5c, 0x7e, 0x1a, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  27",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x35, 0xf4, 0x01, 0x82, 0xab, 0x46, 0x62, 0xf3,
			0x02, 0x3b, 0xae, 0xc1, 0xee, 0x79, 0x6b, 0x57,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  28",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0xeb, 0xba, 0xd7, 0x30, 0x36, 0x49, 0xb4,
			0x19, 0x4a, 0x69, 0x45, 0xc6, 0xcc, 0x36, 0x94,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  29",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa2, 0x12, 0x4b, 0xea, 0x53, 0xec, 0x28, 0x34,
			0x27, 0x9b, 0xed, 0x7f, 0x7e, 0xb0, 0xf9, 0x38,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  30",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb9, 0xfb, 0x43, 0x99, 0xfa, 0x4f, 0xac, 0xc7,
			0x30, 0x9e, 0x14, 0xec, 0x98, 0x36, 0x0b, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  31",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc2, 0x62, 0x77, 0x43, 0x74, 0x20, 0xc5, 0xd6,
			0x34, 0xf7, 0x15, 0xae, 0xa8, 0x1a, 0x91, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  32",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x17, 0x1a, 0x0e, 0x1b, 0x2d, 0xd4, 0x24, 0xf0,
			0xe0, 0x89, 0xaf, 0x2c, 0x4c, 0x10, 0xf3, 0x2f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  33",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7c, 0xad, 0xbe, 0x40, 0x2d, 0x1b, 0x20, 0x8f,
			0xe7, 0x35, 0xed, 0xce, 0x00, 0xae, 0xe7, 0xce,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  34",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x43, 0xb0, 0x2f, 0xf9, 0x29, 0xa1, 0x48, 0x5a,
			0xf6, 0xf5, 0xc6, 0xd6, 0x55, 0x8b, 0xaa, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  35",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x09, 0x2f, 0xaa, 0xcc, 0x9b, 0xf4, 0x35, 0x08,
			0xbf, 0x8f, 0xa8, 0x61, 0x3c, 0xa7, 0x5d, 0xea,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  36",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcb, 0x2b, 0xf8, 0x28, 0x0f, 0x3f, 0x97, 0x42,
			0xc7, 0xed, 0x51, 0x3f, 0xe8, 0x02, 0x62, 0x9c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  37",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x21, 0x5a, 0x41, 0xee, 0x44, 0x2f, 0xa9, 0x92,
			0xa6, 0xe3, 0x23, 0x98, 0x6d, 0xed, 0x3f, 0x68,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  38",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf2, 0x1e, 0x99, 0xcf, 0x4f, 0x0f, 0x77, 0xce,
			0xa8, 0x36, 0xe1, 0x1a, 0x2f, 0xe7, 0x5f, 0xb1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  39",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x95, 0xe3, 0xa0, 0xca, 0x90, 0x79, 0xe6, 0x46,
			0x33, 0x1d, 0xf8, 0xb4, 0xe7, 0x0d, 0x2c, 0xd6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  40",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4a, 0xfe, 0x7f, 0x12, 0x0c, 0xe7, 0x61, 0x3f,
			0x74, 0xfc, 0x12, 0xa0, 0x1a, 0x82, 0x80, 0x73,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  41",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0x7f, 0x00, 0x0e, 0x75, 0xe2, 0xc8, 0xb9,
			0xd4, 0x79, 0xbe, 0xed, 0x91, 0x3f, 0xe6, 0x78,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  42",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x35, 0x83, 0x0c, 0x8e, 0x7a, 0xae, 0xfe, 0x2d,
			0x30, 0x31, 0x0e, 0xf3, 0x81, 0xcb, 0xf6, 0x91,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  43",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0x1a, 0xa0, 0xf2, 0xc8, 0x57, 0x01, 0x44,
			0xf3, 0x86, 0x57, 0xea, 0x40, 0x85, 0xeb, 0xe5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  44",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x85, 0x06, 0x2c, 0x2c, 0x90, 0x9f, 0x15, 0xd9,
			0x26, 0x9b, 0x6c, 0x18, 0xce, 0x99, 0xc4, 0xf0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  45",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x67, 0x80, 0x34, 0xdc, 0x9e, 0x41, 0xb5, 0xa5,
			0x60, 0xed, 0x23, 0x9e, 0xea, 0xb1, 0xbc, 0x78,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  46",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc2, 0xf9, 0x3a, 0x4c, 0xe5, 0xab, 0x6d, 0x5d,
			0x56, 0xf1, 0xb9, 0x3c, 0xf1, 0x99, 0x11, 0xc1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  47",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1c, 0x31, 0x12, 0xbc, 0xb0, 0xc1, 0xdc, 0xc7,
			0x49, 0xd7, 0x99, 0x74, 0x36, 0x91, 0xbf, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  48",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x00, 0xc5, 0x5b, 0xd7, 0x5c, 0x7f, 0x9c, 0x88,
			0x19, 0x89, 0xd3, 0xec, 0x19, 0x11, 0xc0, 0xd4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  49",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xea, 0x2e, 0x6b, 0x5e, 0xf1, 0x82, 0xb7, 0xdf,
			0xf3, 0x62, 0x9a, 0xbd, 0x6a, 0x12, 0x04, 0x5f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  50",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x22, 0x32, 0x23, 0x27, 0xe0, 0x17, 0x80, 0xb1,
			0x73, 0x97, 0xf2, 0x40, 0x87, 0xf8, 0xcc, 0x6f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  51",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc9, 0xca, 0xcb, 0x5c, 0xd1, 0x16, 0x92, 0xc3,
			0x73, 0xb2, 0x41, 0x17, 0x68, 0x14, 0x9e, 0xe7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  52",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa1, 0x8e, 0x3d, 0xbb, 0xca, 0x57, 0x78, 0x60,
			0xda, 0xb6, 0xb8, 0x0d, 0xa3, 0x13, 0x92, 0x56,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  53",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x79, 0xb6, 0x1c, 0x37, 0xbf, 0x32, 0x8e, 0xcc,
			0xa8, 0xd7, 0x43, 0x26, 0x5a, 0x3d, 0x42, 0x5c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  54",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0xd9, 0x9c, 0x6b, 0xcc, 0x1f, 0x06, 0xfd,
			0xa8, 0xe2, 0x7e, 0x8a, 0xe3, 0xf1, 0xcc, 0xc7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  55",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0xfd, 0x4b, 0x91, 0xc7, 0x01, 0xfd, 0x6b,
			0x61, 0xb7, 0xf9, 0x97, 0x82, 0x9d, 0x66, 0x3b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  56",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0x00, 0x5d, 0x52, 0xf2, 0x5f, 0x16, 0xbd,
			0xc9, 0x54, 0x5a, 0x87, 0x6a, 0x63, 0x49, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  57",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0x4d, 0x35, 0x4f, 0x02, 0xbb, 0x5a, 0x5e,
			0x47, 0xd3, 0x96, 0x66, 0x86, 0x7f, 0x24, 0x6a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  58",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd4, 0x51, 0xb8, 0xd6, 0xe1, 0xe1, 0xa0, 0xeb,
			0xb1, 0x55, 0xfb, 0xbf, 0x6e, 0x7b, 0x7d, 0xc3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  59",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x68, 0x98, 0xd4, 0xf4, 0x2f, 0xa7, 0xba, 0x6a,
			0x10, 0xac, 0x05, 0xe8, 0x7b, 0x9f, 0x20, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  60",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb6, 0x11, 0x29, 0x5e, 0x73, 0x9c, 0xa7, 0xd9,
			0xb5, 0x0f, 0x8e, 0x4c, 0x0e, 0x75, 0x4a, 0x3f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  61",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7d, 0x33, 0xfc, 0x7d, 0x8a, 0xbe, 0x3c, 0xa1,
			0x93, 0x67, 0x59, 0xf8, 0xf5, 0xde, 0xaf, 0x20,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  62",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3b, 0x5e, 0x0f, 0x56, 0x6d, 0xc9, 0x6c, 0x29,
			0x8f, 0x0c, 0x12, 0x63, 0x75, 0x39, 0xb2, 0x5c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  63",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf8, 0x07, 0xc3, 0xe7, 0x98, 0x5f, 0xe0, 0xf5,
			0xa5, 0x0e, 0x2c, 0xdb, 0x25, 0xc5, 0x10, 0x9e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  64",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x41, 0xf9, 0x92, 0xa8, 0x56, 0xfb, 0x27, 0x8b,
			0x38, 0x9a, 0x62, 0xf5, 0xd2, 0x74, 0xd7, 0xe9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  65",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x10, 0xd3, 0xed, 0x7a, 0x6f, 0xe1, 0x5a, 0xb4,
			0xd9, 0x1a, 0xcb, 0xc7, 0xd0, 0x76, 0x7a, 0xb1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  66",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x21, 0xfe, 0xec, 0xd4, 0x5b, 0x2e, 0x67, 0x59,
			0x73, 0xac, 0x33, 0xbf, 0x0c, 0x54, 0x24, 0xfc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  67",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x14, 0x80, 0xcb, 0x39, 0x55, 0xba, 0x62, 0xd0,
			0x9e, 0xea, 0x66, 0x8f, 0x7c, 0x70, 0x88, 0x17,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  68",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x66, 0x40, 0x40, 0x33, 0xd6, 0xb7, 0x2b, 0x60,
			0x93, 0x54, 0xd5, 0x49, 0x6e, 0x7e, 0xb5, 0x11,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  69",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1c, 0x31, 0x7a, 0x22, 0x0a, 0x7d, 0x70, 0x0d,
			0xa2, 0xb1, 0xe0, 0x75, 0xb0, 0x02, 0x66, 0xe1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  70",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xab, 0x3b, 0x89, 0x54, 0x22, 0x33, 0xf1, 0x27,
			0x1b, 0xf8, 0xfd, 0x0c, 0x0f, 0x40, 0x35, 0x45,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  71",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd9, 0x3e, 0xae, 0x96, 0x6f, 0xac, 0x46, 0xdc,
			0xa9, 0x27, 0xd6, 0xb1, 0x14, 0xfa, 0x3f, 0x9e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  72",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0xde, 0xc5, 0x21, 0x31, 0x65, 0x03, 0xd9,
			0xd5, 0xee, 0x65, 0xdf, 0x3e, 0xa9, 0x4d, 0xdf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  73",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xee, 0xf4, 0x56, 0x43, 0x1d, 0xea, 0x8b, 0x4a,
			0xcf, 0x83, 0xbd, 0xae, 0x37, 0x17, 0xf7, 0x5f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  74",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x06, 0xf2, 0x51, 0x9a, 0x2f, 0xaf, 0xaa, 0x59,
			0x6b, 0xfe, 0xf5, 0xcf, 0xa1, 0x5c, 0x21, 0xb9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  75",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x25, 0x1a, 0x7e, 0xac, 0x7e, 0x2f, 0xe8, 0x09,
			0xe4, 0xaa, 0x8d, 0x0d, 0x70, 0x12, 0x53, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  76",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3b, 0xff, 0xc1, 0x6e, 0x4c, 0x49, 0xb2, 0x68,
			0xa2, 0x0f, 0x8d, 0x96, 0xa6, 0x0b, 0x40, 0x58,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  77",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe8, 0x86, 0xf9, 0x28, 0x19, 0x99, 0xc5, 0xbb,
			0x3b, 0x3e, 0x88, 0x62, 0xe2, 0xf7, 0xc9, 0x88,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  78",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0x3b, 0xf9, 0x0d, 0x61, 0xbe, 0xef, 0x39,
			0xf4, 0x8d, 0xd6, 0x25, 0xfc, 0xef, 0x13, 0x61,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  79",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4d, 0x37, 0xc8, 0x50, 0x64, 0x45, 0x63, 0xc6,
			0x9f, 0xd0, 0xac, 0xd9, 0xa0, 0x49, 0x32, 0x5b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  80",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb8, 0x7c, 0x92, 0x1b, 0x91, 0x82, 0x9e, 0xf3,
			0xb1, 0x3c, 0xa5, 0x41, 0xee, 0x11, 0x30, 0xa6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  81",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2e, 0x65, 0xeb, 0x6b, 0x6e, 0xa3, 0x83, 0xe1,
			0x09, 0xac, 0xcc, 0xe8, 0x32, 0x6b, 0x03, 0x93,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  82",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9c, 0xa5, 0x47, 0xf7, 0x43, 0x9e, 0xdc, 0x3e,
			0x25, 0x5c, 0x0f, 0x4d, 0x49, 0xaa, 0x89, 0x90,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  83",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa5, 0xe6, 0x52, 0x61, 0x4c, 0x93, 0x00, 0xf3,
			0x78, 0x16, 0xb1, 0xf9, 0xfd, 0x0c, 0x87, 0xf9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  84",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x14, 0x95, 0x4f, 0x0b, 0x46, 0x97, 0x77, 0x6f,
			0x44, 0x49, 0x4f, 0xe4, 0x58, 0xd8, 0x14, 0xed,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  85",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7c, 0x8d, 0x9a, 0xb6, 0xc2, 0x76, 0x17, 0x23,
			0xfe, 0x42, 0xf8, 0xbb, 0x50, 0x6c, 0xbc, 0xf7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  86",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdb, 0x7e, 0x19, 0x32, 0x67, 0x9f, 0xdd, 0x99,
			0x74, 0x2a, 0xab, 0x04, 0xaa, 0x0d, 0x5a, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  87",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4c, 0x6a, 0x1c, 0x83, 0xe5, 0x68, 0xcd, 0x10,
			0xf2, 0x7c, 0x2d, 0x73, 0xde, 0xd1, 0x9c, 0x28,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  88",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0xec, 0xbe, 0x61, 0x77, 0xe6, 0x74, 0xc9,
			0x8d, 0xe4, 0x12, 0x41, 0x3f, 0x7a, 0xc9, 0x15,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  89",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0x68, 0x4a, 0x2a, 0xc5, 0x5f, 0xe1, 0xec,
			0x2b, 0x8e, 0xbd, 0x56, 0x22, 0x52, 0x0b, 0x73,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  90",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x74, 0x72, 0xf9, 0xa7, 0x98, 0x86, 0x07, 0xca,
			0x79, 0x70, 0x77, 0x95, 0x99, 0x10, 0x35, 0xe6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  91",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0xaf, 0xf0, 0x89, 0x87, 0x8b, 0xf3, 0x35,
			0x2f, 0x8d, 0xf1, 0x72, 0xa3, 0xae, 0x47, 0xd8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  92",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x65, 0xc0, 0x52, 0x6c, 0xbe, 0x40, 0x16, 0x1b,
			0x80, 0x19, 0xa2, 0xa3, 0x17, 0x1a, 0xbd, 0x23,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  93",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x37, 0x7b, 0xe0, 0xbe, 0x33, 0xb4, 0xe3, 0xe3,
			0x10, 0xb4, 0xaa, 0xbd, 0xa1, 0x73, 0xf8, 0x4f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  94",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x94, 0x02, 0xe9, 0xaa, 0x6f, 0x69, 0xde, 0x65,
			0x04, 0xda, 0x8d, 0x20, 0xc4, 0xfc, 0xaa, 0x2f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  95",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x12, 0x3c, 0x1f, 0x4a, 0xf3, 0x13, 0xad, 0x8c,
			0x2c, 0xe6, 0x48, 0xb2, 0xe7, 0x1f, 0xb6, 0xe1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  96",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1f, 0xfc, 0x62, 0x6d, 0x30, 0x20, 0x3d, 0xcd,
			0xb0, 0x01, 0x9f, 0xb8, 0x0f, 0x72, 0x6c, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  97",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x76, 0xda, 0x1f, 0xbe, 0x3a, 0x50, 0x72, 0x8c,
			0x50, 0xfd, 0x2e, 0x62, 0x1b, 0x5a, 0xd8, 0x85,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  98",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x08, 0x2e, 0xb8, 0xbe, 0x35, 0xf4, 0x42, 0xfb,
			0x52, 0x66, 0x8e, 0x16, 0xa5, 0x91, 0xd1, 0xd6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128  99",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe6, 0x56, 0xf9, 0xec, 0xf5, 0xfe, 0x27, 0xec,
			0x3e, 0x4a, 0x73, 0xd0, 0x0c, 0x28, 0x2f, 0xb3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 100",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2c, 0xa8, 0x20, 0x9d, 0x63, 0x27, 0x4c, 0xd9,
			0xa2, 0x9b, 0xb7, 0x4b, 0xcd, 0x77, 0x68, 0x3a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 101",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x79, 0xbf, 0x5d, 0xce, 0x14, 0xbb, 0x7d, 0xd7,
			0x3a, 0x8e, 0x36, 0x11, 0xde, 0x7c, 0xe0, 0x26,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 102",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3c, 0x84, 0x99, 0x39, 0xa5, 0xd2, 0x93, 0x99,
			0xf3, 0x44, 0xc4, 0xa0, 0xec, 0xa8, 0xa5, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 103",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xed, 0x3c, 0x0a, 0x94, 0xd5, 0x9b, 0xec, 0xe9,
			0x88, 0x35, 0xda, 0x7a, 0xa4, 0xf0, 0x7c, 0xa2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 104",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
		},
		.ctext	 = {
			0x63, 0x91, 0x9e, 0xd4, 0xce, 0x10, 0x19, 0x64,
			0x38, 0xb6, 0xad, 0x09, 0xd9, 0x9c, 0xd7, 0x95,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 105",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
		},
		.ctext	 = {
			0x76, 0x78, 0xf3, 0xa8, 0x33, 0xf1, 0x9f, 0xea,
			0x95, 0xf3, 0xc6, 0x02, 0x9e, 0x2b, 0xc6, 0x10,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 106",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0xa4, 0x26, 0x83, 0x10, 0x67, 0xd3, 0x6b,
			0x92, 0xbe, 0x7c, 0x5f, 0x81, 0xc1, 0x3c, 0x56,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 107",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
		},
		.ctext	 = {
			0x92, 0x72, 0xe2, 0xd2, 0xcd, 0xd1, 0x10, 0x50,
			0x99, 0x8c, 0x84, 0x50, 0x77, 0xa3, 0x0e, 0xa0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 108",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
		},
		.ctext	 = {
			0x08, 0x8c, 0x4b, 0x53, 0xf5, 0xec, 0x0f, 0xf8,
			0x14, 0xc1, 0x9a, 0xda, 0xe7, 0xf6, 0x24, 0x6c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 109",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0x10, 0xa5, 0xe4, 0x01, 0xfd, 0xf0, 0xa0,
			0x35, 0x4d, 0xdb, 0xcc, 0x0d, 0x01, 0x2b, 0x17,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 110",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0x7a, 0x38, 0x57, 0x36, 0xc0, 0xa6, 0x18,
			0x9b, 0xd6, 0x58, 0x9b, 0xd8, 0x44, 0x5a, 0x93,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 111",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
		},
		.ctext	 = {
			0x54, 0x5f, 0x2b, 0x83, 0xd9, 0x61, 0x6d, 0xcc,
			0xf6, 0x0f, 0xa9, 0x83, 0x0e, 0x9c, 0xd2, 0x87,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 112",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
		},
		.ctext	 = {
			0x4b, 0x70, 0x6f, 0x7f, 0x92, 0x40, 0x63, 0x52,
			0x39, 0x40, 0x37, 0xa6, 0xd4, 0xf4, 0x68, 0x8d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 113",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
		},
		.ctext	 = {
			0xb7, 0x97, 0x2b, 0x39, 0x41, 0xc4, 0x4b, 0x90,
			0xaf, 0xa7, 0xb2, 0x64, 0xbf, 0xba, 0x73, 0x87,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 114",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
		},
		.ctext	 = {
			0x6f, 0x45, 0x73, 0x2c, 0xf1, 0x08, 0x81, 0x54,
			0x6f, 0x0f, 0xd2, 0x38, 0x96, 0xd2, 0xbb, 0x60,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 115",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
		},
		.ctext	 = {
			0x2e, 0x35, 0x79, 0xca, 0x15, 0xaf, 0x27, 0xf6,
			0x4b, 0x3c, 0x95, 0x5a, 0x5b, 0xfc, 0x30, 0xba,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 116",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
		},
		.ctext	 = {
			0x34, 0xa2, 0xc5, 0xa9, 0x1a, 0xe2, 0xae, 0xc9,
			0x9b, 0x7d, 0x1b, 0x5f, 0xa6, 0x78, 0x04, 0x47,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 117",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
		},
		.ctext	 = {
			0xa4, 0xd6, 0x61, 0x6b, 0xd0, 0x4f, 0x87, 0x33,
			0x5b, 0x0e, 0x53, 0x35, 0x12, 0x27, 0xa9, 0xee,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 118",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
		},
		.ctext	 = {
			0x7f, 0x69, 0x2b, 0x03, 0x94, 0x58, 0x67, 0xd1,
			0x61, 0x79, 0xa8, 0xce, 0xfc, 0x83, 0xea, 0x3f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 119",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
		},
		.ctext	 = {
			0x3b, 0xd1, 0x41, 0xee, 0x84, 0xa0, 0xe6, 0x41,
			0x4a, 0x26, 0xe7, 0xa4, 0xf2, 0x81, 0xf8, 0xa2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 120",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
		},
		.ctext	 = {
			0xd1, 0x78, 0x8f, 0x57, 0x2d, 0x98, 0xb2, 0xb1,
			0x6e, 0xc5, 0xd5, 0xf3, 0x92, 0x2b, 0x99, 0xbc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 121",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
		},
		.ctext	 = {
			0x08, 0x33, 0xff, 0x6f, 0x61, 0xd9, 0x8a, 0x57,
			0xb2, 0x88, 0xe8, 0xc3, 0x58, 0x6b, 0x85, 0xa6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 122",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
		},
		.ctext	 = {
			0x85, 0x68, 0x26, 0x17, 0x97, 0xde, 0x17, 0x6b,
			0xf0, 0xb4, 0x3b, 0xec, 0xc6, 0x28, 0x5a, 0xfb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 123",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
		},
		.ctext	 = {
			0xf9, 0xb0, 0xfd, 0xa0, 0xc4, 0xa8, 0x98, 0xf5,
			0xb9, 0xe6, 0xf6, 0x61, 0xc4, 0xce, 0x4d, 0x07,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 124",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
		},
		.ctext	 = {
			0x8a, 0xde, 0x89, 0x59, 0x13, 0x68, 0x5c, 0x67,
			0xc5, 0x26, 0x9f, 0x8a, 0xae, 0x42, 0x98, 0x3e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 125",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
		},
		.ctext	 = {
			0x39, 0xbd, 0xe6, 0x7d, 0x5c, 0x8e, 0xd8, 0xa8,
			0xb1, 0xc3, 0x7e, 0xb8, 0xfa, 0x9f, 0x5a, 0xc0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 126",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
		},
		.ctext	 = {
			0x5c, 0x00, 0x5e, 0x72, 0xc1, 0x41, 0x8c, 0x44,
			0xf5, 0x69, 0xf2, 0xea, 0x33, 0xba, 0x54, 0xf3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt128 127",
		.keylen	 = 128,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		},
		.ctext	 = {
			0x3f, 0x5b, 0x8c, 0xc9, 0xea, 0x85, 0x5a, 0x0a,
			0xfa, 0x73, 0x47, 0xd2, 0x3e, 0x8d, 0x66, 0x4e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   0",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6c, 0xd0, 0x25, 0x13, 0xe8, 0xd4, 0xdc, 0x98,
			0x6b, 0x4a, 0xfe, 0x08, 0x7a, 0x60, 0xbd, 0x0c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   1",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2c, 0xe1, 0xf8, 0xb7, 0xe3, 0x06, 0x27, 0xc1,
			0xc4, 0x51, 0x9e, 0xad, 0xa4, 0x4b, 0xc4, 0x36,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   2",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x99, 0x46, 0xb5, 0xf8, 0x7a, 0xf4, 0x46, 0xf5,
			0x79, 0x6c, 0x1f, 0xee, 0x63, 0xa2, 0xda, 0x24,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   3",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2a, 0x56, 0x03, 0x64, 0xce, 0x52, 0x9e, 0xfc,
			0x21, 0x78, 0x87, 0x79, 0x56, 0x8d, 0x55, 0x55,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   4",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x35, 0xc1, 0x47, 0x18, 0x37, 0xaf, 0x44, 0x61,
			0x53, 0xbc, 0xe5, 0x5d, 0x5b, 0xa7, 0x2a, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   5",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xce, 0x60, 0xbc, 0x52, 0x38, 0x62, 0x34, 0xf1,
			0x58, 0xf8, 0x43, 0x41, 0xe5, 0x34, 0xcd, 0x9e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   6",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8c, 0x7c, 0x27, 0xff, 0x32, 0xbc, 0xf8, 0xdc,
			0x2d, 0xc5, 0x7c, 0x90, 0xc2, 0x90, 0x39, 0x61,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   7",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x32, 0xbb, 0x6a, 0x7e, 0xc8, 0x44, 0x99, 0xe1,
			0x66, 0xf9, 0x36, 0x00, 0x3d, 0x55, 0xa5, 0xbb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   8",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa5, 0xc7, 0x72, 0xe5, 0xc6, 0x26, 0x31, 0xef,
			0x66, 0x0e, 0xe1, 0xd5, 0x87, 0x7f, 0x6d, 0x1b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192   9",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x03, 0x0d, 0x7e, 0x5b, 0x64, 0xf3, 0x80, 0xa7,
			0xe4, 0xea, 0x53, 0x87, 0xb5, 0xcd, 0x7f, 0x49,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  10",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0d, 0xc9, 0xa2, 0x61, 0x00, 0x37, 0x00, 0x9b,
			0x69, 0x8f, 0x11, 0xbb, 0x7e, 0x86, 0xc8, 0x3e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  11",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x00, 0x46, 0x61, 0x2c, 0x76, 0x6d, 0x18, 0x40,
			0xc2, 0x26, 0x36, 0x4f, 0x1f, 0xa7, 0xed, 0x72,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  12",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x48, 0x80, 0xc7, 0xe0, 0x8f, 0x27, 0xbe, 0xfe,
			0x78, 0x59, 0x07, 0x43, 0xc0, 0x5e, 0x69, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  13",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x25, 0x20, 0xce, 0x82, 0x9a, 0x26, 0x57, 0x7f,
			0x0f, 0x48, 0x22, 0xc4, 0xec, 0xc8, 0x74, 0x01,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  14",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x87, 0x65, 0xe8, 0xac, 0xc1, 0x69, 0x75, 0x83,
			0x19, 0xcb, 0x46, 0xdc, 0x7b, 0xcf, 0x3d, 0xca,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  15",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe9, 0x8f, 0x4b, 0xa4, 0xf0, 0x73, 0xdf, 0x4b,
			0xaa, 0x11, 0x6d, 0x01, 0x1d, 0xc2, 0x4a, 0x28,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  16",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf3, 0x78, 0xf6, 0x8c, 0x5d, 0xbf, 0x59, 0xe2,
			0x11, 0xb3, 0xa6, 0x59, 0xa7, 0x31, 0x7d, 0x94,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  17",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x28, 0x3d, 0x3b, 0x06, 0x9d, 0x8e, 0xb9, 0xfb,
			0x43, 0x2d, 0x74, 0xb9, 0x6c, 0xa7, 0x62, 0xb4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  18",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa7, 0xe1, 0x84, 0x2e, 0x8a, 0x87, 0x86, 0x1c,
			0x22, 0x1a, 0x50, 0x08, 0x83, 0x24, 0x5c, 0x51,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  19",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0xaa, 0x27, 0x04, 0x71, 0x88, 0x1b, 0xe0,
			0x70, 0xfb, 0x52, 0xc7, 0x06, 0x7c, 0xe7, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  20",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x01, 0xb0, 0xf4, 0x76, 0xd4, 0x84, 0xf4, 0x3f,
			0x1a, 0xeb, 0x6e, 0xfa, 0x93, 0x61, 0xa8, 0xac,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  21",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1c, 0x3a, 0x94, 0xf1, 0xc0, 0x52, 0xc5, 0x5c,
			0x2d, 0x83, 0x59, 0xaf, 0xf2, 0x16, 0x3b, 0x4f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  22",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe8, 0xa0, 0x67, 0xb6, 0x04, 0xd5, 0x37, 0x3d,
			0x8b, 0x0f, 0x2e, 0x05, 0xa0, 0x3b, 0x34, 0x1b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  23",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa7, 0x87, 0x6e, 0xc8, 0x7f, 0x5a, 0x09, 0xbf,
			0xea, 0x42, 0xc7, 0x7d, 0xa3, 0x0f, 0xd5, 0x0e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  24",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0c, 0xf3, 0xe9, 0xd3, 0xa4, 0x2b, 0xe5, 0xb8,
			0x54, 0xca, 0x65, 0xb1, 0x3f, 0x35, 0xf4, 0x8d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  25",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6c, 0x62, 0xf6, 0xbb, 0xca, 0xb7, 0xc3, 0xe8,
			0x21, 0xc9, 0x29, 0x0f, 0x08, 0x89, 0x2d, 0xda,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  26",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7f, 0x5e, 0x05, 0xbd, 0x20, 0x68, 0x73, 0x81,
			0x96, 0xfe, 0xe7, 0x9a, 0xce, 0x7e, 0x3a, 0xec,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  27",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x44, 0x0e, 0x0d, 0x73, 0x32, 0x55, 0xcd, 0xa9,
			0x2f, 0xb4, 0x6e, 0x84, 0x2f, 0xe5, 0x80, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  28",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaa, 0x5d, 0x5b, 0x1c, 0x4e, 0xa1, 0xb7, 0xa2,
			0x2e, 0x55, 0x83, 0xac, 0x2e, 0x9e, 0xd8, 0xa7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  29",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0xe5, 0x37, 0xe8, 0x9e, 0x84, 0x91, 0xe8,
			0x66, 0x2a, 0xae, 0x3b, 0xc8, 0x09, 0x42, 0x1d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  30",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x99, 0x7d, 0xd3, 0xe9, 0xf1, 0x59, 0x8b, 0xfa,
			0x73, 0xf7, 0x59, 0x73, 0xf7, 0xe9, 0x3b, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  31",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0x38, 0xd4, 0xf7, 0x45, 0x2a, 0xfe, 0xfc,
			0xb7, 0xfc, 0x72, 0x12, 0x44, 0xe4, 0xb7, 0x2e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  32",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0b, 0xe2, 0xb1, 0x82, 0x52, 0xe7, 0x74, 0xdd,
			0xa3, 0x0c, 0xdd, 0xa0, 0x2c, 0x69, 0x06, 0xe3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  33",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd2, 0x69, 0x5e, 0x59, 0xc2, 0x03, 0x61, 0xd8,
			0x26, 0x52, 0xd7, 0xd5, 0x8b, 0x6f, 0x11, 0xb2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  34",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0x2d, 0x88, 0xd1, 0x3e, 0xae, 0x52, 0x08,
			0x9a, 0xbd, 0x61, 0x43, 0xcf, 0xe3, 0x94, 0xe9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  35",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd4, 0x9b, 0xce, 0xb3, 0xb8, 0x23, 0xfe, 0xdd,
			0x60, 0x2c, 0x30, 0x53, 0x45, 0x73, 0x4b, 0xd2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  36",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0x7b, 0x1d, 0xbb, 0x0f, 0xfa, 0x40, 0xef,
			0x7d, 0x95, 0xde, 0xf4, 0x21, 0x23, 0x3f, 0xae,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  37",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7c, 0xa0, 0xc1, 0xd9, 0x33, 0x56, 0xd9, 0xeb,
			0x8a, 0xa9, 0x52, 0x08, 0x4d, 0x75, 0xf9, 0x13,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  38",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf2, 0xcb, 0xf9, 0xcb, 0x18, 0x6e, 0x27, 0x0d,
			0xd7, 0xbd, 0xb0, 0xc2, 0x8f, 0xeb, 0xc5, 0x7d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  39",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc9, 0x43, 0x37, 0xc3, 0x7c, 0x4e, 0x79, 0x0a,
			0xb4, 0x57, 0x80, 0xbd, 0x9c, 0x36, 0x74, 0xa0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  40",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8e, 0x35, 0x58, 0xc1, 0x35, 0x25, 0x2f, 0xb9,
			0xc9, 0xf3, 0x67, 0xed, 0x60, 0x94, 0x67, 0xa1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  41",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1b, 0x72, 0xee, 0xae, 0xe4, 0x89, 0x9b, 0x44,
			0x39, 0x14, 0xe5, 0xb3, 0xa5, 0x7f, 0xba, 0x92,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  42",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x01, 0x18, 0x65, 0xf9, 0x1b, 0xc5, 0x68, 0x68,
			0xd0, 0x51, 0xe5, 0x2c, 0x9e, 0xfd, 0x59, 0xb7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  43",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe4, 0x77, 0x13, 0x18, 0xad, 0x7a, 0x63, 0xdd,
			0x68, 0x0f, 0x6e, 0x58, 0x3b, 0x77, 0x47, 0xea,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  44",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x61, 0xe3, 0xd1, 0x94, 0x08, 0x8d, 0xc8, 0xd9,
			0x7e, 0x9e, 0x6d, 0xb3, 0x74, 0x57, 0xea, 0xc5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  45",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0xff, 0x1e, 0xc9, 0xcc, 0xfb, 0xc3, 0x49,
			0xe5, 0xd3, 0x56, 0xd0, 0x63, 0x69, 0x3a, 0xd6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  46",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3c, 0xc9, 0xe9, 0xa9, 0xbe, 0x8c, 0xc3, 0xf6,
			0xfb, 0x2e, 0xa2, 0x40, 0x88, 0xe9, 0xbb, 0x19,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  47",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1e, 0xe5, 0xab, 0x00, 0x3d, 0xc8, 0x72, 0x2e,
			0x74, 0x90, 0x5d, 0x9a, 0x8f, 0xe3, 0xd3, 0x50,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  48",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x24, 0x53, 0x39, 0x31, 0x95, 0x84, 0xb0, 0xa4,
			0x12, 0x41, 0x28, 0x69, 0xd6, 0xc2, 0xea, 0xda,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  49",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7b, 0xd4, 0x96, 0x91, 0x81, 0x15, 0xd1, 0x4e,
			0xd5, 0x38, 0x08, 0x52, 0x71, 0x6c, 0x88, 0x14,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  50",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x27, 0x3a, 0xb2, 0xf2, 0xb4, 0xa3, 0x66, 0xa5,
			0x7d, 0x58, 0x2a, 0x33, 0x93, 0x13, 0xc8, 0xb1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  51",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0x33, 0x65, 0xa9, 0xff, 0xbe, 0x3b, 0x0c,
			0xa6, 0x1e, 0x98, 0x50, 0x75, 0x54, 0x16, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  52",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaf, 0xa9, 0x9c, 0x99, 0x7a, 0xc4, 0x78, 0xa0,
			0xde, 0xa4, 0x11, 0x9c, 0x9e, 0x45, 0xf8, 0xb1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  53",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x92, 0x16, 0x30, 0x9a, 0x78, 0x42, 0x43, 0x0b,
			0x83, 0xff, 0xb9, 0x86, 0x38, 0x01, 0x15, 0x12,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  54",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x62, 0xab, 0xc7, 0x92, 0x28, 0x82, 0x58, 0x49,
			0x2a, 0x7c, 0xb4, 0x51, 0x45, 0xf4, 0xb7, 0x59,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  55",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x53, 0x49, 0x23, 0xc1, 0x69, 0xd5, 0x04, 0xd7,
			0x51, 0x9c, 0x15, 0xd3, 0x0e, 0x75, 0x6c, 0x50,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  56",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfa, 0x75, 0xe0, 0x5b, 0xcd, 0xc7, 0xe0, 0x0c,
			0x27, 0x3f, 0xa3, 0x3f, 0x6e, 0xe4, 0x41, 0xd2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  57",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7d, 0x35, 0x0f, 0xa6, 0x05, 0x70, 0x80, 0xf1,
			0x08, 0x6a, 0x56, 0xb1, 0x7e, 0xc2, 0x40, 0xdb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  58",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf3, 0x4e, 0x4a, 0x63, 0x24, 0xea, 0x4a, 0x5c,
			0x39, 0xa6, 0x61, 0xc8, 0xfe, 0x5a, 0xda, 0x8f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  59",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x08, 0x82, 0xa1, 0x6f, 0x44, 0x08, 0x8d, 0x42,
			0x44, 0x7a, 0x29, 0xac, 0x09, 0x0e, 0xc1, 0x7e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  60",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3a, 0x3c, 0x15, 0xbf, 0xc1, 0x1a, 0x95, 0x37,
			0xc1, 0x30, 0x68, 0x70, 0x04, 0xe1, 0x36, 0xee,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  61",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x22, 0xc0, 0xa7, 0x67, 0x8d, 0xc6, 0xd8, 0xcf,
			0x5c, 0x8a, 0x6d, 0x5a, 0x99, 0x60, 0x76, 0x7c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  62",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb4, 0x6b, 0x09, 0x80, 0x9d, 0x68, 0xb9, 0xa4,
			0x56, 0x43, 0x2a, 0x79, 0xbd, 0xc2, 0xe3, 0x8c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  63",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x93, 0xba, 0xaf, 0xfb, 0x35, 0xfb, 0xe7, 0x39,
			0xc1, 0x7c, 0x6a, 0xc2, 0x2e, 0xec, 0xf1, 0x8f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  64",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc8, 0xaa, 0x80, 0xa7, 0x85, 0x06, 0x75, 0xbc,
			0x00, 0x7c, 0x46, 0xdf, 0x06, 0xb4, 0x98, 0x68,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  65",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x12, 0xc6, 0xf3, 0x87, 0x7a, 0xf4, 0x21, 0xa9,
			0x18, 0xa8, 0x4b, 0x77, 0x58, 0x58, 0x02, 0x1d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  66",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x33, 0xf1, 0x23, 0x28, 0x2c, 0x5d, 0x63, 0x39,
			0x24, 0xf7, 0xd5, 0xba, 0x3f, 0x3c, 0xab, 0x11,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  67",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa8, 0xf1, 0x61, 0x00, 0x27, 0x33, 0xe9, 0x3c,
			0xa4, 0x52, 0x7d, 0x22, 0xc1, 0xa0, 0xc5, 0xbb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  68",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb7, 0x2f, 0x70, 0xeb, 0xf3, 0xe3, 0xfd, 0xa2,
			0x3f, 0x50, 0x8e, 0xec, 0x76, 0xb4, 0x2c, 0x02,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  69",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6a, 0x9d, 0x96, 0x5e, 0x62, 0x74, 0x14, 0x3f,
			0x25, 0xaf, 0xdc, 0xfc, 0x88, 0xff, 0xd7, 0x7c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  70",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa0, 0xc7, 0x4f, 0xd0, 0xb9, 0x36, 0x17, 0x64,
			0xce, 0x91, 0xc5, 0x20, 0x0b, 0x09, 0x53, 0x57,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  71",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x09, 0x1d, 0x1f, 0xdc, 0x2b, 0xd2, 0xc3, 0x46,
			0xcd, 0x50, 0x46, 0xa8, 0xc6, 0x20, 0x91, 0x46,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  72",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe2, 0xa3, 0x75, 0x80, 0x11, 0x6c, 0xfb, 0x71,
			0x85, 0x62, 0x54, 0x49, 0x6a, 0xb0, 0xac, 0xa8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  73",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe0, 0xb3, 0xa0, 0x07, 0x85, 0x91, 0x7c, 0x7e,
			0xfc, 0x9a, 0xdb, 0xa3, 0x22, 0x81, 0x35, 0x71,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  74",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x73, 0x3d, 0x41, 0xf4, 0x72, 0x7b, 0x5e, 0xf0,
			0xdf, 0x4a, 0xf4, 0xcf, 0x3c, 0xff, 0xa0, 0xcb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  75",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa9, 0x9e, 0xbb, 0x03, 0x02, 0x60, 0x82, 0x6f,
			0x98, 0x1a, 0xd3, 0xe6, 0x44, 0x90, 0xaa, 0x4f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  76",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x73, 0xf3, 0x4c, 0x7d, 0x3e, 0xae, 0x5e, 0x80,
			0x08, 0x2c, 0x16, 0x47, 0x52, 0x43, 0x08, 0xee,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  77",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0xeb, 0xd5, 0xad, 0x08, 0x23, 0x45, 0xb7,
			0xa2, 0x09, 0x7c, 0xcd, 0x34, 0x64, 0xda, 0x02,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  78",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7c, 0xc4, 0xae, 0x9a, 0x42, 0x4b, 0x2c, 0xec,
			0x90, 0xc9, 0x71, 0x53, 0xc2, 0x45, 0x7e, 0xc5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  79",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x54, 0xd6, 0x32, 0xd0, 0x3a, 0xba, 0x0b, 0xd0,
			0xf9, 0x18, 0x77, 0xeb, 0xdd, 0x4d, 0x09, 0xcb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  80",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd3, 0x42, 0x7b, 0xe7, 0xe4, 0xd2, 0x7c, 0xd5,
			0x4f, 0x5f, 0xe3, 0x7b, 0x03, 0xcf, 0x08, 0x97,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  81",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb2, 0x09, 0x97, 0x95, 0xe8, 0x8c, 0xc1, 0x58,
			0xfd, 0x75, 0xea, 0x13, 0x3d, 0x7e, 0x7f, 0xbe,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  82",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa6, 0xca, 0xe4, 0x6f, 0xb6, 0xfa, 0xdf, 0xe7,
			0xa2, 0xc3, 0x02, 0xa3, 0x42, 0x42, 0x81, 0x7b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  83",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x02, 0x6a, 0x70, 0x24, 0xd6, 0xa9, 0x02, 0xe0,
			0xb3, 0xff, 0xcc, 0xba, 0xa9, 0x10, 0xcc, 0x3f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  84",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x15, 0x6f, 0x07, 0x76, 0x7a, 0x85, 0xa4, 0x31,
			0x23, 0x21, 0xf6, 0x39, 0x68, 0x33, 0x8a, 0x01,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  85",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x15, 0xee, 0xc9, 0xeb, 0xf4, 0x2b, 0x9c, 0xa7,
			0x68, 0x97, 0xd2, 0xcd, 0x6c, 0x5a, 0x12, 0xe2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  86",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdb, 0x0d, 0x3a, 0x6f, 0xdc, 0xc1, 0x3f, 0x91,
			0x5e, 0x2b, 0x30, 0x2c, 0xee, 0xb7, 0x0f, 0xd8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  87",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x71, 0xdb, 0xf3, 0x7e, 0x87, 0xa2, 0xe3, 0x4d,
			0x15, 0xb2, 0x0e, 0x8f, 0x10, 0xe4, 0x89, 0x24,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  88",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc7, 0x45, 0xc4, 0x51, 0xe9, 0x6f, 0xf3, 0xc0,
			0x45, 0xe4, 0x36, 0x7c, 0x83, 0x3e, 0x3b, 0x54,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  89",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x34, 0x0d, 0xa0, 0x9c, 0x2d, 0xd1, 0x1c, 0x3b,
			0x67, 0x9d, 0x08, 0xcc, 0xd2, 0x7d, 0xd5, 0x95,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  90",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0x79, 0xf7, 0xc0, 0xc2, 0xa0, 0x3e, 0xe6,
			0x60, 0xc6, 0xd3, 0x92, 0xdb, 0x02, 0x5d, 0x18,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  91",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa4, 0xb2, 0xc7, 0xd8, 0xeb, 0xa5, 0x31, 0xff,
			0x47, 0xc5, 0x04, 0x1a, 0x55, 0xfb, 0xd1, 0xec,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  92",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x74, 0x56, 0x9a, 0x2c, 0xa5, 0xa7, 0xbd, 0x51,
			0x31, 0xce, 0x8d, 0xc7, 0xcb, 0xfb, 0xf7, 0x2f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  93",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x37, 0x13, 0xda, 0x0c, 0x02, 0x19, 0xb6, 0x34,
			0x54, 0x03, 0x56, 0x13, 0xb5, 0xa4, 0x03, 0xdd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  94",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x88, 0x27, 0x55, 0x1d, 0xdc, 0xc9, 0xdf, 0x23,
			0xfa, 0x72, 0xa3, 0xde, 0x4e, 0x9f, 0x0b, 0x07,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  95",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2e, 0x3f, 0xeb, 0xfd, 0x62, 0x5b, 0xfc, 0xd0,
			0xa2, 0xc0, 0x6e, 0xb4, 0x60, 0xda, 0x17, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  96",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xee, 0x82, 0xe6, 0xba, 0x48, 0x81, 0x56, 0xf7,
			0x64, 0x96, 0x31, 0x1d, 0xa6, 0x94, 0x1d, 0xeb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  97",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x47, 0x70, 0x44, 0x6f, 0x01, 0xd1, 0xf3, 0x91,
			0x25, 0x6e, 0x85, 0xa1, 0xb3, 0x0d, 0x89, 0xd3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  98",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xaf, 0x04, 0xb6, 0x8f, 0x10, 0x4f, 0x21, 0xef,
			0x2a, 0xfb, 0x47, 0x67, 0xcf, 0x74, 0x14, 0x3c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192  99",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcf, 0x35, 0x79, 0xa9, 0xba, 0x38, 0xc8, 0xe4,
			0x36, 0x53, 0x17, 0x3e, 0x14, 0xf3, 0xa4, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 100",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb3, 0xbb, 0xa9, 0x04, 0xf4, 0x95, 0x3e, 0x09,
			0xb5, 0x48, 0x00, 0xaf, 0x2f, 0x62, 0xe7, 0xd4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 101",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfc, 0x42, 0x49, 0x65, 0x6e, 0x14, 0xb2, 0x9e,
			0xb9, 0xc4, 0x48, 0x29, 0xb4, 0xc5, 0x9a, 0x46,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 102",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9b, 0x31, 0x56, 0x8f, 0xeb, 0xe8, 0x1c, 0xfc,
			0x2e, 0x65, 0xaf, 0x1c, 0x86, 0xd1, 0xa3, 0x08,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 103",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9c, 0xa0, 0x9c, 0x25, 0xf2, 0x73, 0xa7, 0x66,
			0xdb, 0x98, 0xa4, 0x80, 0xce, 0x8d, 0xfe, 0xdc,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 104",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
		},
		.ctext	 = {
			0xb9, 0x09, 0x92, 0x57, 0x86, 0xf3, 0x4c, 0x3c,
			0x92, 0xd9, 0x71, 0x88, 0x3c, 0x9f, 0xbe, 0xdf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 105",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
		},
		.ctext	 = {
			0x82, 0x64, 0x7f, 0x13, 0x32, 0xfe, 0x57, 0x0a,
			0x9d, 0x4d, 0x92, 0xb2, 0xee, 0x77, 0x1d, 0x3b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 106",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0x04, 0xa7, 0xe8, 0x08, 0x32, 0xb3, 0xa9,
			0x99, 0x54, 0xbc, 0xa6, 0xf5, 0xb9, 0xf5, 0x01,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 107",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
		},
		.ctext	 = {
			0x88, 0x46, 0x07, 0xb1, 0x28, 0xc5, 0xde, 0x3a,
			0xb3, 0x9a, 0x52, 0x9a, 0x1e, 0xf5, 0x1b, 0xef,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 108",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
		},
		.ctext	 = {
			0x67, 0x0c, 0xfa, 0x09, 0x3d, 0x1d, 0xbd, 0xb2,
			0x31, 0x70, 0x41, 0x40, 0x41, 0x02, 0x43, 0x5e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 109",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
		},
		.ctext	 = {
			0x7a, 0x86, 0x71, 0x95, 0xf3, 0xce, 0x87, 0x69,
			0xcb, 0xd3, 0x36, 0x50, 0x2f, 0xbb, 0x51, 0x30,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 110",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
		},
		.ctext	 = {
			0x52, 0xef, 0xcf, 0x64, 0xc7, 0x2b, 0x2f, 0x7c,
			0xa5, 0xb3, 0xc8, 0x36, 0xb1, 0x07, 0x8c, 0x15,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 111",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
		},
		.ctext	 = {
			0x40, 0x19, 0x25, 0x0f, 0x6e, 0xef, 0xb2, 0xac,
			0x5c, 0xcb, 0xca, 0xe0, 0x44, 0xe7, 0x5c, 0x7e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 112",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
		},
		.ctext	 = {
			0x02, 0x2c, 0x4f, 0x6f, 0x5a, 0x01, 0x7d, 0x29,
			0x27, 0x85, 0x62, 0x76, 0x67, 0xdd, 0xef, 0x24,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 113",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
		},
		.ctext	 = {
			0xe9, 0xc2, 0x10, 0x78, 0xa2, 0xeb, 0x7e, 0x03,
			0x25, 0x0f, 0x71, 0x00, 0x0f, 0xa9, 0xe3, 0xed,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 114",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
		},
		.ctext	 = {
			0xa1, 0x3e, 0xae, 0xeb, 0x9c, 0xd3, 0x91, 0xda,
			0x4e, 0x2b, 0x09, 0x49, 0x0b, 0x3e, 0x7f, 0xad,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 115",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
		},
		.ctext	 = {
			0xc9, 0x58, 0xa1, 0x71, 0xdc, 0xa1, 0xd4, 0xed,
			0x53, 0xe1, 0xaf, 0x1d, 0x38, 0x08, 0x03, 0xa9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 116",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
		},
		.ctext	 = {
			0x21, 0x44, 0x2e, 0x07, 0xa1, 0x10, 0x66, 0x7f,
			0x25, 0x83, 0xea, 0xee, 0xee, 0x44, 0xdc, 0x8c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 117",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
		},
		.ctext	 = {
			0x59, 0xbb, 0xb3, 0x53, 0xcf, 0x1d, 0xd8, 0x67,
			0xa6, 0xe3, 0x37, 0x37, 0xaf, 0x65, 0x5e, 0x99,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 118",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
		},
		.ctext	 = {
			0x43, 0xcd, 0x3b, 0x25, 0x37, 0x5d, 0x0c, 0xe4,
			0x10, 0x87, 0xff, 0x9f, 0xe2, 0x82, 0x96, 0x39,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 119",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
		},
		.ctext	 = {
			0x6b, 0x98, 0xb1, 0x7e, 0x80, 0xd1, 0x11, 0x8e,
			0x35, 0x16, 0xbd, 0x76, 0x8b, 0x28, 0x5a, 0x84,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 120",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
		},
		.ctext	 = {
			0xae, 0x47, 0xed, 0x36, 0x76, 0xca, 0x0c, 0x08,
			0xde, 0xea, 0x02, 0xd9, 0x5b, 0x81, 0xdb, 0x58,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 121",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
		},
		.ctext	 = {
			0x34, 0xec, 0x40, 0xdc, 0x20, 0x41, 0x37, 0x95,
			0xed, 0x53, 0x62, 0x8e, 0xa7, 0x48, 0x72, 0x0b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 122",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
		},
		.ctext	 = {
			0x4d, 0xc6, 0x81, 0x63, 0xf8, 0xe9, 0x83, 0x54,
			0x73, 0x25, 0x35, 0x42, 0xc8, 0xa6, 0x5d, 0x46,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 123",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
		},
		.ctext	 = {
			0x2a, 0xab, 0xb9, 0x99, 0xf4, 0x36, 0x93, 0x17,
			0x5a, 0xf6, 0x5c, 0x6c, 0x61, 0x2c, 0x46, 0xfb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 124",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
		},
		.ctext	 = {
			0xe0, 0x1f, 0x94, 0x49, 0x9d, 0xac, 0x35, 0x47,
			0x51, 0x5c, 0x5b, 0x1d, 0x75, 0x6f, 0x0f, 0x58,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 125",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
		},
		.ctext	 = {
			0x9d, 0x12, 0x43, 0x5a, 0x46, 0x48, 0x0c, 0xe0,
			0x0e, 0xa3, 0x49, 0xf7, 0x17, 0x99, 0xdf, 0x9a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 126",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
		},
		.ctext	 = {
			0xce, 0xf4, 0x1d, 0x16, 0xd2, 0x66, 0xbd, 0xfe,
			0x46, 0x93, 0x8a, 0xd7, 0x88, 0x4c, 0xc0, 0xcf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt192 127",
		.keylen	 = 192,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		},
		.ctext	 = {
			0xb1, 0x3d, 0xb4, 0xda, 0x1f, 0x71, 0x8b, 0xc6,
			0x90, 0x47, 0x97, 0xc8, 0x2b, 0xcf, 0x2d, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   0",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdd, 0xc6, 0xbf, 0x79, 0x0c, 0x15, 0x76, 0x0d,
			0x8d, 0x9a, 0xeb, 0x6f, 0x9a, 0x75, 0xfd, 0x4e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   1",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0a, 0x6b, 0xdc, 0x6d, 0x4c, 0x1e, 0x62, 0x80,
			0x30, 0x1f, 0xd8, 0xe9, 0x7d, 0xdb, 0xe6, 0x01,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   2",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9b, 0x80, 0xee, 0xfb, 0x7e, 0xbe, 0x2d, 0x2b,
			0x16, 0x24, 0x7a, 0xa0, 0xef, 0xc7, 0x2f, 0x5d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   3",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7f, 0x2c, 0x5e, 0xce, 0x07, 0xa9, 0x8d, 0x8b,
			0xee, 0x13, 0xc5, 0x11, 0x77, 0x39, 0x5f, 0xf7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   4",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x78, 0x18, 0xd8, 0x00, 0xdc, 0xf6, 0xf4, 0xbe,
			0x1e, 0x0e, 0x94, 0xf4, 0x03, 0xd1, 0xe4, 0xc2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   5",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe7, 0x4c, 0xd1, 0xc9, 0x2f, 0x09, 0x19, 0xc3,
			0x5a, 0x03, 0x24, 0x12, 0x3d, 0x61, 0x77, 0xd3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   6",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x80, 0x92, 0xa4, 0xdc, 0xf2, 0xda, 0x7e, 0x77,
			0xe9, 0x3b, 0xdd, 0x37, 0x1d, 0xfe, 0xd8, 0x2e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   7",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x49, 0xaf, 0x6b, 0x37, 0x21, 0x35, 0xac, 0xef,
			0x10, 0x13, 0x2e, 0x54, 0x8f, 0x21, 0x7b, 0x17,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   8",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8b, 0xcd, 0x40, 0xf9, 0x4e, 0xbb, 0x63, 0xb9,
			0xf7, 0x90, 0x96, 0x76, 0xe6, 0x67, 0xf1, 0xe7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256   9",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfe, 0x1c, 0xff, 0xb8, 0x3f, 0x45, 0xdc, 0xfb,
			0x38, 0xb2, 0x9b, 0xe4, 0x38, 0xdb, 0xd3, 0xab,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  10",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0d, 0xc5, 0x8a, 0x8d, 0x88, 0x66, 0x23, 0x70,
			0x5a, 0xec, 0x15, 0xcb, 0x1e, 0x70, 0xdc, 0x0e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  11",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc2, 0x18, 0xfa, 0xa1, 0x60, 0x56, 0xbd, 0x07,
			0x74, 0xc3, 0xe8, 0xd7, 0x9c, 0x35, 0xa5, 0xe4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  12",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x04, 0x7b, 0xba, 0x83, 0xf7, 0xaa, 0x84, 0x17,
			0x31, 0x50, 0x4e, 0x01, 0x22, 0x08, 0xfc, 0x9e,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  13",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdc, 0x8f, 0x0e, 0x49, 0x15, 0xfd, 0x81, 0xba,
			0x70, 0xa3, 0x31, 0x31, 0x08, 0x82, 0xf6, 0xda,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  14",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x15, 0x69, 0x85, 0x9e, 0xa6, 0xb7, 0x20, 0x6c,
			0x30, 0xbf, 0x4f, 0xd0, 0xcb, 0xfa, 0xc3, 0x3c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  15",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x30, 0x0a, 0xde, 0x92, 0xf8, 0x8f, 0x48, 0xfa,
			0x2d, 0xf7, 0x30, 0xec, 0x16, 0xef, 0x44, 0xcd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  16",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1f, 0xe6, 0xcc, 0x3c, 0x05, 0x96, 0x5d, 0xc0,
			0x8e, 0xb0, 0x59, 0x0c, 0x95, 0xac, 0x71, 0xd0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  17",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x59, 0xe8, 0x58, 0xea, 0xaa, 0x97, 0xfe, 0xc3,
			0x81, 0x11, 0x27, 0x5b, 0x6c, 0xf5, 0xab, 0xc0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  18",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x22, 0x39, 0x45, 0x5e, 0x7a, 0xfe, 0x3b, 0x06,
			0x16, 0x10, 0x02, 0x88, 0xcc, 0x5a, 0x72, 0x3b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  19",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3e, 0xe5, 0x00, 0xc5, 0xc8, 0xd6, 0x34, 0x79,
			0x71, 0x71, 0x63, 0xe5, 0x5c, 0x5c, 0x45, 0x22,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  20",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd5, 0xe3, 0x8b, 0xf1, 0x5f, 0x16, 0xd9, 0x0e,
			0x3e, 0x21, 0x40, 0x41, 0xd7, 0x74, 0xda, 0xa8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  21",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb1, 0xf4, 0x06, 0x6e, 0x6f, 0x4f, 0x18, 0x7d,
			0xfe, 0x5f, 0x2a, 0xd1, 0xb1, 0x78, 0x19, 0xd0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  22",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6e, 0xf4, 0xcc, 0x4d, 0xe4, 0x9b, 0x11, 0x06,
			0x5d, 0x7a, 0xf2, 0x90, 0x98, 0x54, 0x79, 0x4a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  23",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xac, 0x86, 0xbc, 0x60, 0x6b, 0x66, 0x40, 0xc3,
			0x09, 0xe7, 0x82, 0xf2, 0x32, 0xbf, 0x36, 0x7f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  24",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x36, 0xaf, 0xf0, 0xef, 0x7b, 0xf3, 0x28, 0x07,
			0x72, 0xcf, 0x4c, 0xac, 0x80, 0xa0, 0xd2, 0xb2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  25",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1f, 0x8e, 0xed, 0xea, 0x0f, 0x62, 0xa1, 0x40,
			0x6d, 0x58, 0xcf, 0xc3, 0xec, 0xea, 0x72, 0xcf,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  26",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xab, 0xf4, 0x15, 0x4a, 0x33, 0x75, 0xa1, 0xd3,
			0xe6, 0xb1, 0xd4, 0x54, 0x43, 0x8f, 0x95, 0xa6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  27",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0xf9, 0x6e, 0x9d, 0x60, 0x7f, 0x66, 0x15,
			0xfc, 0x19, 0x20, 0x61, 0xee, 0x64, 0x8b, 0x07,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  28",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcf, 0x37, 0xcd, 0xaa, 0xa0, 0xd2, 0xd5, 0x36,
			0xc7, 0x18, 0x57, 0x63, 0x4c, 0x79, 0x20, 0x64,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  29",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfb, 0xd6, 0x64, 0x0c, 0x80, 0x24, 0x5c, 0x2b,
			0x80, 0x53, 0x73, 0xf1, 0x30, 0x70, 0x31, 0x27,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  30",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8d, 0x6a, 0x8a, 0xfe, 0x55, 0xa6, 0xe4, 0x81,
			0xba, 0xda, 0xe0, 0xd1, 0x46, 0xf4, 0x36, 0xdb,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  31",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6a, 0x49, 0x81, 0xf2, 0x91, 0x5e, 0x3e, 0x68,
			0xaf, 0x6c, 0x22, 0x38, 0x5d, 0xd0, 0x67, 0x56,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  32",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x42, 0xa1, 0x13, 0x6e, 0x5f, 0x8d, 0x8d, 0x21,
			0xd3, 0x10, 0x19, 0x98, 0x64, 0x2d, 0x57, 0x3b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  33",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9b, 0x47, 0x15, 0x96, 0xdc, 0x69, 0xae, 0x15,
			0x86, 0xce, 0xe6, 0x15, 0x8b, 0x0b, 0x01, 0x81,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  34",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x75, 0x36, 0x65, 0xc4, 0xaf, 0x1e, 0xff, 0x33,
			0xaa, 0x8b, 0x62, 0x8b, 0xf8, 0x74, 0x1c, 0xfd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  35",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9a, 0x68, 0x2a, 0xcf, 0x40, 0xbe, 0x01, 0xf5,
			0xb2, 0xa4, 0x19, 0x3c, 0x9a, 0x82, 0x40, 0x4d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  36",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x54, 0xfa, 0xfe, 0x26, 0xe4, 0x28, 0x7f, 0x17,
			0xd1, 0x93, 0x5f, 0x87, 0xeb, 0x9a, 0xde, 0x01,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  37",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x49, 0xd5, 0x41, 0xb2, 0xe7, 0x4c, 0xfe, 0x73,
			0xe6, 0xa8, 0xe8, 0x22, 0x5f, 0x7b, 0xd4, 0x49,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  38",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x11, 0xa4, 0x55, 0x30, 0xf6, 0x24, 0xff, 0x6f,
			0x76, 0xa1, 0xb3, 0x82, 0x66, 0x26, 0xff, 0x7b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  39",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xf9, 0x6b, 0x0c, 0x4a, 0x8b, 0xc6, 0xc8, 0x61,
			0x30, 0x28, 0x9f, 0x60, 0xb4, 0x3b, 0x8f, 0xba,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  40",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x48, 0xc7, 0xd0, 0xe8, 0x08, 0x34, 0xeb, 0xdc,
			0x35, 0xb6, 0x73, 0x5f, 0x76, 0xb4, 0x6c, 0x8b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  41",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x24, 0x63, 0x53, 0x1a, 0xb5, 0x4d, 0x66, 0x95,
			0x5e, 0x73, 0xed, 0xc4, 0xcb, 0x8e, 0xaa, 0x45,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  42",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xac, 0x9b, 0xd8, 0xe2, 0x53, 0x04, 0x69, 0x13,
			0x4b, 0x9d, 0x5b, 0x06, 0x5d, 0x4f, 0x56, 0x5b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  43",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3f, 0x5f, 0x91, 0x06, 0xd0, 0xe5, 0x2f, 0x97,
			0x3d, 0x48, 0x90, 0xe6, 0xf3, 0x7e, 0x8a, 0x00,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  44",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x20, 0xeb, 0xc8, 0x6f, 0x13, 0x04, 0xd2, 0x72,
			0xe2, 0xe2, 0x07, 0xe5, 0x9d, 0xb6, 0x39, 0xf0,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  45",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe6, 0x7a, 0xe6, 0x42, 0x6b, 0xf9, 0x52, 0x6c,
			0x97, 0x2c, 0xff, 0x07, 0x2b, 0x52, 0x25, 0x2c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  46",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1a, 0x51, 0x8d, 0xdd, 0xaf, 0x9e, 0xfa, 0x0d,
			0x00, 0x2c, 0xc5, 0x8d, 0x10, 0x7e, 0xdf, 0xc8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  47",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xea, 0xd7, 0x31, 0xaf, 0x4d, 0x3a, 0x2f, 0xe3,
			0xb3, 0x4b, 0xed, 0x04, 0x79, 0x42, 0xa4, 0x9f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  48",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb1, 0xd4, 0xef, 0xe4, 0x02, 0x42, 0xf8, 0x3e,
			0x93, 0xb6, 0xc8, 0xd7, 0xef, 0xb5, 0xea, 0xe9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  49",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcd, 0x2b, 0x1f, 0xec, 0x11, 0xfd, 0x90, 0x6c,
			0x5c, 0x76, 0x30, 0x09, 0x94, 0x43, 0x61, 0x0a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  50",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa1, 0x85, 0x3f, 0xe4, 0x7f, 0xe2, 0x92, 0x89,
			0xd1, 0x53, 0x16, 0x1d, 0x06, 0x38, 0x7d, 0x21,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  51",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x46, 0x32, 0x15, 0x41, 0x79, 0xa5, 0x55, 0xc1,
			0x7e, 0xa6, 0x04, 0xd0, 0x88, 0x9f, 0xab, 0x14,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  52",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xdd, 0x27, 0xca, 0xc6, 0x40, 0x1a, 0x02, 0x2e,
			0x8f, 0x38, 0xf9, 0xf9, 0x3e, 0x77, 0x44, 0x17,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  53",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc0, 0x90, 0x31, 0x3e, 0xb9, 0x86, 0x74, 0xf3,
			0x5f, 0x31, 0x23, 0x38, 0x5f, 0xb9, 0x5d, 0x4d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  54",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xcc, 0x35, 0x26, 0x26, 0x2b, 0x92, 0xf0, 0x2e,
			0xdc, 0xe5, 0x48, 0xf7, 0x16, 0xb9, 0xf4, 0x5c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  55",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xc0, 0x83, 0x8d, 0x1a, 0x2b, 0x16, 0xa7, 0xc7,
			0xf0, 0xdf, 0xcc, 0x43, 0x3c, 0x39, 0x9c, 0x33,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  56",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x0d, 0x9a, 0xc7, 0x56, 0xeb, 0x29, 0x76, 0x95,
			0xee, 0xd4, 0xd3, 0x82, 0xeb, 0x12, 0x6d, 0x26,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  57",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x56, 0xed, 0xe9, 0xdd, 0xa3, 0xf6, 0xf1, 0x41,
			0xbf, 0xf1, 0x75, 0x7f, 0xa6, 0x89, 0xc3, 0xe1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  58",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x76, 0x8f, 0x52, 0x0e, 0xfe, 0x0f, 0x23, 0xe6,
			0x1d, 0x3e, 0xc8, 0xad, 0x9c, 0xe9, 0x17, 0x74,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  59",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb1, 0x14, 0x4d, 0xdf, 0xa7, 0x57, 0x55, 0x21,
			0x33, 0x90, 0xe7, 0xc5, 0x96, 0x66, 0x04, 0x90,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  60",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1d, 0x7c, 0x0c, 0x40, 0x40, 0xb3, 0x55, 0xb9,
			0xd1, 0x07, 0xa9, 0x93, 0x25, 0xe3, 0xb0, 0x50,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  61",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd8, 0xe2, 0xbb, 0x1a, 0xe8, 0xee, 0x3d, 0xcf,
			0x5b, 0xf7, 0xd6, 0xc3, 0x8d, 0xa8, 0x2a, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  62",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xfa, 0xf8, 0x2d, 0x17, 0x8a, 0xf2, 0x5a, 0x98,
			0x86, 0xa4, 0x7e, 0x7f, 0x78, 0x9b, 0x98, 0xd7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  63",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9b, 0x58, 0xdb, 0xfd, 0x77, 0xfe, 0x5a, 0xca,
			0x9c, 0xfc, 0x19, 0x0c, 0xd1, 0xb8, 0x2d, 0x19,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  64",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x77, 0xf3, 0x92, 0x08, 0x90, 0x42, 0xe4, 0x78,
			0xac, 0x16, 0xc0, 0xc8, 0x6a, 0x0b, 0x5d, 0xb5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  65",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x19, 0xf0, 0x8e, 0x34, 0x20, 0xee, 0x69, 0xb4,
			0x77, 0xca, 0x14, 0x20, 0x28, 0x1c, 0x47, 0x82,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  66",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa1, 0xb1, 0x9b, 0xee, 0xe4, 0xe1, 0x17, 0x13,
			0x9f, 0x74, 0xb3, 0xc5, 0x3f, 0xdc, 0xb8, 0x75,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  67",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa3, 0x7a, 0x58, 0x69, 0xb2, 0x18, 0xa9, 0xf3,
			0xa0, 0x86, 0x8d, 0x19, 0xae, 0xa0, 0xad, 0x6a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  68",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xbc, 0x35, 0x94, 0xe8, 0x65, 0xbc, 0xd0, 0x26,
			0x1b, 0x13, 0x20, 0x27, 0x31, 0xf3, 0x35, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  69",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x81, 0x14, 0x41, 0xce, 0x1d, 0x30, 0x9e, 0xee,
			0x71, 0x85, 0xe8, 0xc7, 0x52, 0xc0, 0x75, 0x57,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  70",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x95, 0x99, 0x71, 0xce, 0x41, 0x34, 0x19, 0x05,
			0x63, 0x51, 0x8e, 0x70, 0x0b, 0x98, 0x74, 0xd1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  71",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x76, 0xb5, 0x61, 0x4a, 0x04, 0x27, 0x07, 0xc9,
			0x8e, 0x21, 0x32, 0xe2, 0xe8, 0x05, 0xfe, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  72",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7d, 0x9f, 0xa6, 0xa5, 0x75, 0x30, 0xd0, 0xf0,
			0x36, 0xfe, 0xc3, 0x1c, 0x23, 0x0b, 0x0c, 0xc6,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  73",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x96, 0x41, 0x53, 0xa8, 0x3b, 0xf6, 0x98, 0x9a,
			0x4b, 0xa8, 0x0d, 0xaa, 0x91, 0xc3, 0xe0, 0x81,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  74",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xa0, 0x13, 0x01, 0x4d, 0x4c, 0xe8, 0x05, 0x4c,
			0xf2, 0x59, 0x1d, 0x06, 0xf6, 0xf2, 0xf1, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  75",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xd1, 0xc5, 0xf6, 0x39, 0x9b, 0xf3, 0x82, 0x50,
			0x2e, 0x38, 0x5e, 0xee, 0x14, 0x74, 0xa8, 0x69,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  76",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x00, 0x07, 0xe2, 0x0b, 0x82, 0x98, 0xec, 0x35,
			0x4f, 0x0f, 0x5f, 0xe7, 0x47, 0x0f, 0x36, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  77",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xb9, 0x5b, 0xa0, 0x5b, 0x33, 0x2d, 0xa6, 0x1e,
			0xf6, 0x3a, 0x2b, 0x31, 0xfc, 0xad, 0x98, 0x79,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  78",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x46, 0x20, 0xa4, 0x9b, 0xd9, 0x67, 0x49, 0x15,
			0x61, 0x66, 0x9a, 0xb2, 0x5d, 0xce, 0x45, 0xf4,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  79",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x12, 0xe7, 0x12, 0x14, 0xae, 0x8e, 0x04, 0xf0,
			0xbb, 0x63, 0xd7, 0x42, 0x5c, 0x6f, 0x14, 0xd5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  80",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x4c, 0xc4, 0x2f, 0xc1, 0x40, 0x7b, 0x00, 0x8f,
			0xe3, 0x50, 0x90, 0x7c, 0x09, 0x2e, 0x80, 0xac,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  81",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x08, 0xb2, 0x44, 0xce, 0x7c, 0xbc, 0x8e, 0xe9,
			0x7f, 0xbb, 0xa8, 0x08, 0xcb, 0x14, 0x6f, 0xda,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  82",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x39, 0xb3, 0x33, 0xe8, 0x69, 0x4f, 0x21, 0x54,
			0x6a, 0xd1, 0xed, 0xd9, 0xd8, 0x7e, 0xd9, 0x5b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  83",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x3b, 0x27, 0x1f, 0x8a, 0xb2, 0xe6, 0xe4, 0xa2,
			0x0b, 0xa8, 0x09, 0x0f, 0x43, 0xba, 0x78, 0xf3,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  84",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x9a, 0xd9, 0x83, 0xf3, 0xbf, 0x65, 0x1c, 0xd0,
			0x39, 0x3f, 0x0a, 0x73, 0xcc, 0xcd, 0xea, 0x50,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  85",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x8f, 0x47, 0x6c, 0xbf, 0xf7, 0x5c, 0x1f, 0x72,
			0x5c, 0xe1, 0x8e, 0x4b, 0xbc, 0xd1, 0x9b, 0x32,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  86",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0x5b, 0x62, 0x67, 0xf1, 0xd6, 0xab, 0x53,
			0x20, 0x83, 0x5a, 0x13, 0x3f, 0x09, 0x6f, 0x2a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  87",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x14, 0x5b, 0x60, 0xd6, 0xd0, 0x19, 0x3c, 0x23,
			0xf4, 0x22, 0x18, 0x48, 0xa8, 0x92, 0xd6, 0x1a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  88",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x55, 0xcf, 0xb3, 0xfb, 0x6d, 0x75, 0xca, 0xd0,
			0x44, 0x5b, 0xbc, 0x8d, 0xaf, 0xa2, 0x5b, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  89",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x7b, 0x8e, 0x70, 0x98, 0xe3, 0x57, 0xef, 0x71,
			0x23, 0x7d, 0x46, 0xd8, 0xb0, 0x75, 0xb0, 0xf5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  90",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2b, 0xf2, 0x72, 0x29, 0x90, 0x1e, 0xb4, 0x0f,
			0x2d, 0xf9, 0xd8, 0x39, 0x8d, 0x15, 0x05, 0xae,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  91",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x83, 0xa6, 0x34, 0x02, 0xa7, 0x7f, 0x9a, 0xd5,
			0xc1, 0xe9, 0x31, 0xa9, 0x31, 0xec, 0xd7, 0x06,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  92",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x6f, 0x8b, 0xa6, 0x52, 0x11, 0x52, 0xd3, 0x1f,
			0x2b, 0xad, 0xa1, 0x84, 0x3e, 0x26, 0xb9, 0x73,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  93",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe5, 0xc3, 0xb8, 0xe3, 0x0f, 0xd2, 0xd8, 0xe6,
			0x23, 0x9b, 0x17, 0xb4, 0x4b, 0xd2, 0x3b, 0xbd,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  94",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1a, 0xc1, 0xf7, 0x10, 0x2c, 0x59, 0x93, 0x3e,
			0x8b, 0x2d, 0xdc, 0x3f, 0x14, 0xe9, 0x4b, 0xaa,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  95",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x21, 0xd9, 0xba, 0x49, 0xf2, 0x76, 0xb4, 0x5f,
			0x11, 0xaf, 0x8f, 0xc7, 0x1a, 0x08, 0x8e, 0x3d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  96",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x64, 0x9f, 0x1c, 0xdd, 0xc3, 0x79, 0x2b, 0x46,
			0x38, 0x63, 0x5a, 0x39, 0x2b, 0xc9, 0xba, 0xde,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  97",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xe2, 0x77, 0x5e, 0x4b, 0x59, 0xc1, 0xbc, 0x2e,
			0x31, 0xa2, 0x07, 0x8c, 0x11, 0xb5, 0xa0, 0x8c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  98",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x2b, 0xe1, 0xfa, 0xe5, 0x04, 0x8a, 0x25, 0x58,
			0x2a, 0x67, 0x9c, 0xa1, 0x09, 0x05, 0xeb, 0x80,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256  99",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0xda, 0x86, 0xf2, 0x92, 0xc6, 0xf4, 0x1e, 0xa3,
			0x4f, 0xb2, 0x06, 0x8d, 0xf7, 0x5e, 0xcc, 0x29,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 100",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x22, 0x0d, 0xf1, 0x9f, 0x85, 0xd6, 0x9b, 0x1b,
			0x56, 0x2f, 0xa6, 0x9a, 0x3c, 0x5b, 0xec, 0xa5,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 101",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x1f, 0x11, 0xd5, 0xd0, 0x35, 0x5e, 0x0b, 0x55,
			0x6c, 0xcd, 0xb6, 0xc7, 0xf5, 0x08, 0x3b, 0x4d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 102",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x62, 0x52, 0x6b, 0x78, 0xbe, 0x79, 0xcb, 0x38,
			0x46, 0x33, 0xc9, 0x1f, 0x83, 0xb4, 0x15, 0x1b,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 103",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
		},
		.ctext	 = {
			0x90, 0xdd, 0xbc, 0xb9, 0x50, 0x84, 0x35, 0x92,
			0xdd, 0x47, 0xbb, 0xef, 0x00, 0xfd, 0xc8, 0x76,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 104",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00,
		},
		.ctext	 = {
			0x2f, 0xd0, 0xe4, 0x1c, 0x5b, 0x84, 0x02, 0x27,
			0x73, 0x54, 0xa7, 0x39, 0x1d, 0x26, 0x18, 0xe2,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 105",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00, 0x00,
		},
		.ctext	 = {
			0x3c, 0xdf, 0x13, 0xe7, 0x2d, 0xee, 0x4c, 0x58,
			0x1b, 0xaf, 0xec, 0x70, 0xb8, 0x5f, 0x96, 0x60,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 106",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 0x00,
		},
		.ctext	 = {
			0xaf, 0xa2, 0xff, 0xc1, 0x37, 0x57, 0x70, 0x92,
			0xe2, 0xb6, 0x54, 0xfa, 0x19, 0x9d, 0x2c, 0x43,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 107",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x00,
		},
		.ctext	 = {
			0x8d, 0x68, 0x3e, 0xe6, 0x3e, 0x60, 0xd2, 0x08,
			0xe3, 0x43, 0xce, 0x48, 0xdb, 0xc4, 0x4c, 0xac,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 108",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
		},
		.ctext	 = {
			0x70, 0x5a, 0x4e, 0xf8, 0xba, 0x21, 0x33, 0x72,
			0x9c, 0x20, 0x18, 0x5c, 0x3d, 0x3a, 0x47, 0x63,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 109",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00,
		},
		.ctext	 = {
			0x08, 0x61, 0xa8, 0x61, 0xc3, 0xdb, 0x4e, 0x94,
			0x19, 0x42, 0x11, 0xb7, 0x7e, 0xd7, 0x61, 0xb9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 110",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00,
		},
		.ctext	 = {
			0x4b, 0x00, 0xc2, 0x7e, 0x8b, 0x26, 0xda, 0x7e,
			0xab, 0x9d, 0x3a, 0x88, 0xde, 0xc8, 0xb0, 0x31,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 111",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
		},
		.ctext	 = {
			0x5f, 0x39, 0x7b, 0xf0, 0x30, 0x84, 0x82, 0x0c,
			0xc8, 0x81, 0x0d, 0x52, 0xe5, 0xb6, 0x66, 0xe9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 112",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00,
		},
		.ctext	 = {
			0x63, 0xfa, 0xfa, 0xbb, 0x72, 0xc0, 0x7b, 0xfb,
			0xd3, 0xdd, 0xc9, 0xb1, 0x20, 0x31, 0x04, 0xb8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 113",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x00,
		},
		.ctext	 = {
			0x68, 0x3e, 0x21, 0x40, 0x58, 0x5b, 0x18, 0x45,
			0x2d, 0xd4, 0xff, 0xbb, 0x93, 0xc9, 0x5d, 0xf9,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 114",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00,
		},
		.ctext	 = {
			0x28, 0x68, 0x94, 0xe4, 0x8e, 0x53, 0x7f, 0x87,
			0x63, 0xb5, 0x67, 0x07, 0xd7, 0xd1, 0x55, 0xc8,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 115",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00,
		},
		.ctext	 = {
			0xa4, 0x23, 0xde, 0xab, 0xc1, 0x73, 0xdc, 0xf7,
			0xe2, 0xc4, 0xc5, 0x3e, 0x77, 0xd3, 0x7c, 0xd1,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 116",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00,
		},
		.ctext	 = {
			0xeb, 0x81, 0x68, 0x31, 0x3e, 0x1c, 0xfd, 0xfd,
			0xb5, 0xe9, 0x86, 0xd5, 0x42, 0x9c, 0xf1, 0x72,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 117",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00,
		},
		.ctext	 = {
			0x27, 0x12, 0x7d, 0xaa, 0xfc, 0x9a, 0xcc, 0xd2,
			0xfb, 0x33, 0x4e, 0xc3, 0xeb, 0xa5, 0x23, 0x23,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 118",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00,
		},
		.ctext	 = {
			0xee, 0x07, 0x15, 0xb9, 0x6f, 0x72, 0xe3, 0xf7,
			0xa2, 0x2a, 0x50, 0x64, 0xfc, 0x59, 0x2f, 0x4c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 119",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00,
		},
		.ctext	 = {
			0x29, 0xee, 0x52, 0x67, 0x70, 0xf2, 0xa1, 0x1d,
			0xcf, 0xa9, 0x89, 0xd1, 0xce, 0x88, 0x83, 0x0f,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 120",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80,
		},
		.ctext	 = {
			0x04, 0x93, 0x37, 0x0e, 0x05, 0x4b, 0x09, 0x87,
			0x11, 0x30, 0xfe, 0x49, 0xaf, 0x73, 0x0a, 0x5a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 121",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0,
		},
		.ctext	 = {
			0x9b, 0x7b, 0x94, 0x0f, 0x6c, 0x50, 0x9f, 0x9e,
			0x44, 0xa4, 0xee, 0x14, 0x04, 0x48, 0xee, 0x46,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 122",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0,
		},
		.ctext	 = {
			0x29, 0x15, 0xbe, 0x4a, 0x1e, 0xcf, 0xdc, 0xbe,
			0x3e, 0x02, 0x38, 0x11, 0xa1, 0x2b, 0xb6, 0xc7,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 123",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0,
		},
		.ctext	 = {
			0x72, 0x40, 0xe5, 0x24, 0xbc, 0x51, 0xd8, 0xc4,
			0xd4, 0x40, 0xb1, 0xbe, 0x55, 0xd1, 0x06, 0x2c,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 124",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
		},
		.ctext	 = {
			0xda, 0x63, 0x03, 0x9d, 0x38, 0xcb, 0x46, 0x12,
			0xb2, 0xdc, 0x36, 0xba, 0x26, 0x68, 0x4b, 0x93,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 125",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
		},
		.ctext	 = {
			0x0f, 0x59, 0xcb, 0x5a, 0x4b, 0x52, 0x2e, 0x2a,
			0xc5, 0x6c, 0x1a, 0x64, 0xf5, 0x58, 0xad, 0x9a,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 126",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe,
		},
		.ctext	 = {
			0x7b, 0xfe, 0x9d, 0x87, 0x6c, 0x6d, 0x63, 0xc1,
			0xd0, 0x35, 0xda, 0x8f, 0xe2, 0x1c, 0x40, 0x9d,
		},
	},
	{
		.desc	 = "NIST CAVS 11.1 ECBVarTxt256 127",
		.keylen	 = 256,
		.key	 = {
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		},
		.ptext	 = {
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
			0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		},
		.ctext	 = {
			0xac, 0xda, 0xce, 0x80, 0x78, 0xa3, 0x2b, 0x1a,
			0x18, 0x2b, 0xfa, 0x49, 0x87, 0xca, 0x13, 0x47,
		},
	},
};

/***************************************************************************
 * Test functions
 */

static int
t_aes_enc(char **desc, void *arg)
{
	struct t_case *t = arg;
	uint8_t out[AES_BLOCK_LEN];
	aes_ctx ctx;
	size_t len;
	int ret;

	(void)asprintf(desc, "%s (encrypt)", t->desc);
	aes_init(&ctx, CIPHER_MODE_ENCRYPT, t->key, t->keylen / 8);
	len = aes_encrypt(&ctx, t->ptext, out, AES_BLOCK_LEN);
	aes_finish(&ctx);
	ret = t_compare_sz(AES_BLOCK_LEN, len) &
	    t_compare_mem(t->ctext, out, AES_BLOCK_LEN);
	return (ret);
}

static int
t_aes_dec(char **desc, void *arg)
{
	struct t_case *t = arg;
	uint8_t out[AES_BLOCK_LEN];
	aes_ctx ctx;
	size_t len;
	int ret;

	(void)asprintf(desc, "%s (decrypt)", t->desc);
	aes_init(&ctx, CIPHER_MODE_DECRYPT, t->key, t->keylen / 8);
	len = aes_decrypt(&ctx, t->ctext, out, AES_BLOCK_LEN);
	aes_finish(&ctx);
	ret = t_compare_sz(AES_BLOCK_LEN, len) &
	    t_compare_mem(t->ptext, out, AES_BLOCK_LEN);
	return (ret);
}


/***************************************************************************
 * Boilerplate
 */

static int
t_prepare(int argc, char *argv[])
{
	unsigned int i, n;

	(void)argc;
	(void)argv;
	n = sizeof t_cases / sizeof t_cases[0];
	for (i = 0; i < n; ++i) {
		t_add_test(t_aes_enc, &t_cases[i], "%s", t_cases[i].desc);
		t_add_test(t_aes_dec, &t_cases[i], "%s", t_cases[i].desc);
	}
	return (0);
}

int
main(int argc, char *argv[])
{

	t_main(t_prepare, NULL, argc, argv);
}
